@@ -282,7 +282,7 @@ appear."
           (setf binding :function
                 localp nil
                 ftype (when (eq :declared (info :function :where-from name))
-                        (info :function :type name))
+                        (proclaimed-ftype name))
                 inlinep (info :function :inlinep name))))))
     (values binding
             localp
@@ -295,10 +295,9 @@ appear."
                    (list-cons-when (and ftype (neq *universal-fun-type* ftype))
                      'ftype (type-specifier ftype))
                    (list-cons-when dx 'dynamic-extent t)
-                   ;; FIXME: a local name shadowing a deprecated global
-                   ;; wrongly returns deprecation info.
-                   (maybe-deprecation-entry
-                    (info :function :deprecated name))
+                   (unless localp
+                     (maybe-deprecation-entry
+                      (info :function :deprecated name)))
                    (extra-pairs :function name fun *lexenv*)))))
 
 (declaim (ftype (sfunction
@@ -500,7 +499,7 @@ the condition types that have been muffled."
          (sb-int:awhen (car (rassoc 'muffle-warning handled-conditions))
            (sb-kernel:type-specifier it))))
       (declaration
-       (copy-list sb-c::*recognized-declarations*))
+       (copy-list sb-int:*recognized-declarations*))
       (t (if (info :declaration :handler declaration-name)
              (extra-decl-info
               declaration-name
@@ -518,8 +517,11 @@ lambda expression will parse its form argument, binding the variables in
 LAMBDA-LIST appropriately, and then execute BODY with those bindings in
 effect."
   (declare (ignore env))
-  (make-macro-lambda (if (and name (symbolp name)) (string name) "PARSE-MACRO")
-                     lambda-list body 'parse-macro name))
+  (values
+   ;; lp#1545148 says that some people don't like NAMED-LAMBDA,
+   ;; so pass NIL instead of a name.
+   (make-macro-lambda nil ; (if (and name (symbolp name)) (string name) "PARSE-MACRO")
+                      lambda-list body 'parse-macro name)))
 
 (defun enclose (lambda-expression &optional environment)
   "Return a function consistent with LAMBDA-EXPRESSION in ENVIRONMENT: the
@@ -530,7 +532,7 @@ is referred to by the expression."
   (let ((env (if environment
                  (sb-c::make-restricted-lexenv environment)
                  (make-null-lexenv))))
-    (compile-in-lexenv nil lambda-expression env)))
+    (compile-in-lexenv lambda-expression env)))
 
 ;;; Add a bit of user-data to a lexenv.
 ;;;
@@ -559,10 +561,17 @@ is referred to by the expression."
             do (push (list* :variable name binding key value) user-data)))
         (:function
          (loop
-            for (name key value) in data
-            for binding1 = (find name pd-fvars :key #'sb-c::leaf-source-name :test #'equal)
-            for binding = (if binding1 binding1 (lexenv-find name funs))
-            do (push (list* :function name binding key value) user-data)))
+           for (name key value) in data
+           for binding = (or (loop for fvar in pd-fvars
+                                   for source-name = (if (consp fvar) ;; MACROLET
+                                                         (car fvar)
+                                                         (sb-c::leaf-source-name fvar))
+                                   when (equal name source-name)
+                                   return (if (consp fvar)
+                                              (cdr fvar)
+                                              fvar))
+                             (lexenv-find name funs))
+           do (push (list* :function name binding key value) user-data)))
         (:declare
          (destructuring-bind (decl-name . value) data
            (push (list* :declare decl-name value) user-data)))))
