@@ -805,13 +805,15 @@ Experimental: interface subject to change."
                (let* ((addr (sb-kernel:get-lisp-obj-address object))
                       (space
                        (cond ((< sb-vm:read-only-space-start addr
-                                 (ash sb-vm:*read-only-space-free-pointer*
-                                      sb-vm:n-fixnum-tag-bits))
+                                 (sb-sys:sap-int sb-vm:*read-only-space-free-pointer*))
                               :read-only)
                              ((< sb-vm:static-space-start addr
-                                 (ash sb-vm:*static-space-free-pointer*
-                                      sb-vm:n-fixnum-tag-bits))
+                                 (sb-sys:sap-int sb-vm:*static-space-free-pointer*))
                               :static)
+                             #+immobile-space
+                             ((< sb-vm:immobile-space-start addr
+                                 (sb-sys:sap-int sb-vm:*immobile-space-free-pointer*))
+                              :immobile)
                              ((< (sb-kernel:current-dynamic-space-start) addr
                                  (sb-sys:sap-int (sb-kernel:dynamic-space-free-pointer)))
                               :dynamic))))
@@ -839,11 +841,11 @@ Experimental: interface subject to change."
                  ;; FIXME: Check other stacks as well.
                  #+sb-thread
                  (dolist (thread (sb-thread:list-all-threads))
-                   (let ((c-start (sb-di::descriptor-sap
+                   (let ((c-start (sb-int:descriptor-sap
                                    (sb-thread::%symbol-value-in-thread
                                     'sb-vm:*control-stack-start*
                                     thread)))
-                         (c-end (sb-di::descriptor-sap
+                         (c-end (sb-int:descriptor-sap
                                  (sb-thread::%symbol-value-in-thread
                                   'sb-vm:*control-stack-end*
                                   thread))))
@@ -878,15 +880,14 @@ conservative roots from the thread registers and interrupt contexts.
 Experimental: interface subject to change."
   (let ((fun (coerce function 'function))
         (seen (sb-int:alloc-xset)))
-    (flet ((call (part)
-             (when (and (member (sb-kernel:lowtag-of part)
-                                `(,sb-vm:instance-pointer-lowtag
-                                  ,sb-vm:list-pointer-lowtag
-                                  ,sb-vm:fun-pointer-lowtag
-                                  ,sb-vm:other-pointer-lowtag))
-                        (not (sb-int:xset-member-p part seen)))
-               (sb-int:add-to-xset part seen)
-               (funcall fun part))))
+    (labels ((call (part)
+               (when (and (is-lisp-pointer part)
+                          (not (sb-int:xset-member-p part seen)))
+                 (sb-int:add-to-xset part seen)
+                 (funcall fun part)))
+             (is-lisp-pointer (obj)
+               #+64-bit (= (logand (sb-kernel:get-lisp-obj-address obj) 3) 3)
+               #-64-bit (oddp (sb-kernel:get-lisp-obj-address obj))))
       (when ext
         (let ((table sb-pcl::*eql-specializer-table*))
           (call (sb-int:with-locked-system-table (table)
@@ -938,7 +939,7 @@ Experimental: interface subject to change."
              (dotimes (i (length object))
                (call (aref object i)))
              (when (sb-kernel:array-header-p object)
-               (call (sb-kernel::%array-data-vector object))
+               (call (sb-kernel:%array-data object))
                (call (sb-kernel::%array-displaced-p object))
                (unless simple
                  (call (sb-kernel::%array-displaced-from object))))))
