@@ -9,6 +9,13 @@
 
 (in-package "SB-SIMPLE-STREAMS")
 
+(eval-when (:compile-toplevel)
+  (defun optional+key-style-warning-p (condition)
+    (and (typep condition '(and simple-condition style-warning))
+         (search "&OPTIONAL and &KEY found"
+                 (simple-condition-format-control condition))))
+  (proclaim '(sb-ext:muffle-conditions (satisfies optional+key-style-warning-p))))
+
 ;;;
 ;;; **********************************************************************
 ;;;
@@ -727,7 +734,8 @@
 
 (defun read-byte (stream &optional (eof-error-p t) eof-value)
   "Returns the next byte of the Stream."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (let ((byte (%read-byte stream eof-error-p eof-value)))
@@ -745,7 +753,8 @@
 (defun read-char (&optional (stream *standard-input*) (eof-error-p t)
                             eof-value recursive-p)
   "Inputs a character from Stream and returns it."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (let ((char (%read-char stream eof-error-p eof-value recursive-p t)))
@@ -764,7 +773,8 @@
 (defun read-char-no-hang (&optional (stream *standard-input*) (eof-error-p t)
                                     eof-value recursive-p)
   "Returns the next character from the Stream if one is availible, or nil."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%check stream :input)
@@ -785,7 +795,8 @@
 
 (defun unread-char (character &optional (stream *standard-input*))
   "Puts the Character back on the front of the input Stream."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%unread-char stream character))
@@ -800,7 +811,8 @@
 (defun peek-char (&optional (peek-type nil) (stream *standard-input*)
                             (eof-error-p t) eof-value recursive-p)
   "Peeks at the next character in the input Stream.  See manual for details."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (let ((char
@@ -843,9 +855,10 @@
   "Returns T if WIDTH octets are available on STREAM.  If WIDTH is
 given as 'CHARACTER, check for a character.  Note: the WIDTH argument
 is supported only on simple-streams."
+  (declare (sb-int:explicit-check))
   ;; WIDTH is number of octets which must be available; any value
   ;; other than 1 is treated as 'character.
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%listen stream width))
@@ -859,7 +872,8 @@ is supported only on simple-streams."
                             eof-value recursive-p)
   "Returns a line of text read from the Stream as a string, discarding the
   newline character."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%read-line stream eof-error-p eof-value recursive-p))
@@ -879,7 +893,7 @@ is supported only on simple-streams."
   for STREAM is reached before copying all elements of the subsequence,
   then the extra elements near the end of sequence are not updated, and
   the index of the next element is returned."
-  (let ((stream (sb-impl::in-synonym-of stream))
+  (let ((stream (in-stream-from-designator stream))
         (end (or end (length seq))))
     (etypecase stream
       (simple-stream
@@ -892,7 +906,8 @@ is supported only on simple-streams."
 
 (defun clear-input (&optional (stream *standard-input*) buffer-only)
   "Clears any buffered input associated with the Stream."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%clear-input stream buffer-only))
@@ -904,7 +919,8 @@ is supported only on simple-streams."
 
 (defun write-byte (integer stream)
   "Outputs an octet to the Stream."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%write-byte stream integer))
@@ -916,7 +932,8 @@ is supported only on simple-streams."
 
 (defun write-char (character &optional (stream *standard-output*))
   "Outputs the Character to the Stream."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%write-char stream character))
@@ -929,7 +946,8 @@ is supported only on simple-streams."
 (defun write-string (string &optional (stream *standard-output*)
                             &key (start 0) (end nil))
   "Outputs the String to the given Stream."
-  (let ((stream (sb-impl::out-synonym-of stream))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream))
         (end (sb-impl::%check-vector-sequence-bounds string start end)))
     (etypecase stream
       (simple-stream
@@ -943,7 +961,8 @@ is supported only on simple-streams."
 (defun write-line (string &optional (stream *standard-output*)
                           &key (start 0) end)
   (declare (type string string))
-  (let ((stream (sb-impl::out-synonym-of stream))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream))
         (end (sb-impl::%check-vector-sequence-bounds string start end)))
     (etypecase stream
       (simple-stream
@@ -961,7 +980,7 @@ is supported only on simple-streams."
 
 (defun write-sequence (seq stream &key (start 0) (end nil))
   "Write the elements of SEQ bounded by START and END to STREAM."
-  (let ((stream (sb-impl::out-synonym-of stream))
+  (let ((stream (out-stream-from-designator stream))
         (end (or end (length seq))))
     (etypecase stream
       (simple-stream
@@ -973,7 +992,8 @@ is supported only on simple-streams."
 
 (defun terpri (&optional (stream *standard-output*))
   "Outputs a new line to the Stream."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%check stream :output)
@@ -988,7 +1008,8 @@ is supported only on simple-streams."
 (defun fresh-line (&optional (stream *standard-output*))
   "Outputs a new line to the Stream if it is not positioned at the beginning of
    a line.  Returns T if it output a new line, nil otherwise."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%fresh-line stream))
@@ -1000,7 +1021,8 @@ is supported only on simple-streams."
 (defun finish-output (&optional (stream *standard-output*))
   "Attempts to ensure that all output sent to the Stream has reached its
    destination, and only then returns."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%finish-output stream))
@@ -1012,7 +1034,8 @@ is supported only on simple-streams."
 
 (defun force-output (&optional (stream *standard-output*))
   "Attempts to force any buffered output to be sent."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%force-output stream))
@@ -1024,7 +1047,8 @@ is supported only on simple-streams."
 
 (defun clear-output (&optional (stream *standard-output*))
   "Clears the given output Stream."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (declare (sb-int:explicit-check))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%clear-output stream))
@@ -1059,7 +1083,7 @@ is supported only on simple-streams."
 (defun charpos (&optional (stream *standard-output*))
   "Returns the number of characters on the current line of output of the given
   Stream, or Nil if that information is not availible."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (with-stream-class (simple-stream stream)
@@ -1073,7 +1097,7 @@ is supported only on simple-streams."
 (defun line-length (&optional (stream *standard-output*))
   "Returns the number of characters in a line of output of the given
   Stream, or Nil if that information is not availible."
-  (let ((stream (sb-impl::out-synonym-of stream)))
+  (let ((stream (out-stream-from-designator stream)))
     (etypecase stream
       (simple-stream
        (%check stream :output)
@@ -1089,7 +1113,7 @@ is supported only on simple-streams."
 (defun wait-for-input-available (stream &optional timeout)
   "Waits for input to become available on the Stream and returns T.  If
   Timeout expires, Nil is returned."
-  (let ((stream (sb-impl::in-synonym-of stream)))
+  (let ((stream (in-stream-from-designator stream)))
     (etypecase stream
       (fixnum
        (sb-sys:wait-until-fd-usable stream :input timeout))
