@@ -202,6 +202,11 @@
          (!cold-lose "bogus operation in *!COLD-TOPLEVELS*")))))
   (/show0 "done with loop over cold toplevel forms and fixups")
 
+  ;; Precise GC seems to think these symbols are live during the final GC
+  ;; which in turn enlivens a bunch of other "*!foo*" symbols.
+  ;; Setting them to NIL helps a little bit.
+  (setq *!cold-defuns* nil *!cold-defconstants* nil *!cold-toplevels* nil)
+
   ;; Now that L-T-V forms have executed, the symbol output chooser works.
   (setf (symbol-function 'choose-symbol-out-fun) real-choose-symbol-out-fun)
 
@@ -223,7 +228,6 @@
 
   (show-and-call os-cold-init-or-reinit)
   (show-and-call !pathname-cold-init)
-  (show-and-call !debug-info-cold-init)
 
   (show-and-call stream-cold-init-or-reset)
   (show-and-call !loader-cold-init)
@@ -317,7 +321,7 @@ When ABORT is true, SBCL exits immediately by calling _exit(2) without
 unwinding stack, or calling exit hooks. Note that _exit(2) does not
 call atexit(3) functions unlike exit(3).
 
-Recursive calls to EXIT cause EXIT to behave as it ABORT was true.
+Recursive calls to EXIT cause EXIT to behave as if ABORT was true.
 
 TIMEOUT controls waiting for other threads to terminate when ABORT is
 NIL. Once current thread has been unwound and *EXIT-HOOKS* have been
