@@ -74,6 +74,7 @@
              (not (mismatch (layout-inherits olayout) new-inherits)))
         olayout
         (make-layout :classoid (make-undefined-classoid name)
+                     :%flags +condition-layout-flag+
                      :inherits new-inherits
                      :depthoid -1
                      :length (layout-length cond-layout)))))
@@ -147,7 +148,7 @@
 
 (defun condition-slot-value (condition name)
   (let ((val (getf (condition-assigned-slots condition) name sb!pcl:+slot-unbound+)))
-    (if (eq val sb!pcl:+slot-unbound+)
+    (if (unbound-marker-p val)
         (let ((class (layout-classoid (%instance-layout condition))))
           (dolist (cslot
                    (condition-classoid-class-slots class)
@@ -222,14 +223,13 @@
     (dolist (cslot (condition-classoid-class-slots classoid))
       (dolist (initarg (condition-slot-initargs cslot))
         (let ((val (getf initargs initarg sb!pcl:+slot-unbound+)))
-          (unless (eq val sb!pcl:+slot-unbound+)
+          (unless (unbound-marker-p val)
             (setf (car (condition-slot-cell cslot)) val)))))
 
     ;; Default any slots with non-constant defaults now.
     (dolist (hslot (condition-classoid-hairy-slots classoid))
       (when (dolist (initarg (condition-slot-initargs hslot) t)
-              (unless (eq (getf initargs initarg sb!pcl:+slot-unbound+)
-                          sb!pcl:+slot-unbound+)
+              (unless (unbound-marker-p (getf initargs initarg sb!pcl:+slot-unbound+))
                 (return nil)))
         (setf (getf (condition-assigned-slots condition)
                     (condition-slot-name hslot))
@@ -592,6 +592,9 @@
              "The variable ~S is unbound."
              (cell-error-name condition)))))
 
+(define-condition retry-unbound-variable
+    (simple-condition unbound-variable) ())
+
 (define-condition undefined-function (cell-error) ()
   (:report
    (lambda (condition stream)
@@ -808,7 +811,7 @@
 (define-condition duplicate-definition (reference-condition warning)
   ((name :initarg :name :reader duplicate-definition-name))
   (:report report-duplicate-definition)
-  (:default-initargs :references (list '(:ansi-cl :section (3 2 2 3)))))
+  (:default-initargs :references '((:ansi-cl :section (3 2 2 3)))))
 ;; To my thinking, DUPLICATE-DEFINITION should be the ancestor condition,
 ;; and not fatal. But changing the meaning of that concept would be a bad idea,
 ;; so instead there is a new condition for the softer variant, which does not
@@ -826,18 +829,18 @@
              (format s "~@<Destructive function ~S called on ~
                         constant data.~@:>"
                      (constant-modified-fun-name c))))
-  (:default-initargs :references (list '(:ansi-cl :special-operator quote)
-                                       '(:ansi-cl :section (3 2 2 3)))))
+  (:default-initargs :references '((:ansi-cl :special-operator quote)
+                                   (:ansi-cl :section (3 2 2 3)))))
 
 (define-condition package-at-variance (reference-condition simple-warning)
   ()
-  (:default-initargs :references (list '(:ansi-cl :macro defpackage)
-                                       '(:sbcl :variable *on-package-variance*))))
+  (:default-initargs :references '((:ansi-cl :macro defpackage)
+                                   (:sbcl :variable *on-package-variance*))))
 
 (define-condition package-at-variance-error (reference-condition simple-condition
                                              package-error)
   ()
-  (:default-initargs :references (list '(:ansi-cl :macro defpackage))))
+  (:default-initargs :references '((:ansi-cl :macro defpackage))))
 
 (define-condition defconstant-uneql (reference-condition error)
   ((name :initarg :name :reader defconstant-uneql-name)
@@ -850,31 +853,30 @@
              (defconstant-uneql-name condition)
              (defconstant-uneql-old-value condition)
              (defconstant-uneql-new-value condition))))
-  (:default-initargs :references (list '(:ansi-cl :macro defconstant)
-                                       '(:sbcl :node "Idiosyncrasies"))))
+  (:default-initargs :references '((:ansi-cl :macro defconstant)
+                                   (:sbcl :node "Idiosyncrasies"))))
 
 (define-condition array-initial-element-mismatch
     (reference-condition simple-warning)
   ()
   (:default-initargs
-      :references (list
-                   '(:ansi-cl :function make-array)
-                   '(:ansi-cl :function sb!xc:upgraded-array-element-type))))
+      :references '((:ansi-cl :function make-array)
+                    (:ansi-cl :function sb!xc:upgraded-array-element-type))))
 
 (define-condition type-warning (reference-condition simple-warning)
   ()
-  (:default-initargs :references (list '(:sbcl :node "Handling of Types"))))
+  (:default-initargs :references '((:sbcl :node "Handling of Types"))))
 (define-condition type-style-warning (reference-condition simple-style-warning)
   ()
-  (:default-initargs :references (list '(:sbcl :node "Handling of Types"))))
+  (:default-initargs :references '((:sbcl :node "Handling of Types"))))
 
 (define-condition local-argument-mismatch (reference-condition simple-warning)
   ()
-  (:default-initargs :references (list '(:ansi-cl :section (3 2 2 3)))))
+  (:default-initargs :references '((:ansi-cl :section (3 2 2 3)))))
 
 (define-condition format-args-mismatch (reference-condition)
   ()
-  (:default-initargs :references (list '(:ansi-cl :section (22 3 10 2)))))
+  (:default-initargs :references '((:ansi-cl :section (22 3 10 2)))))
 
 (define-condition format-too-few-args-warning
     (format-args-mismatch simple-warning)
@@ -888,7 +890,7 @@
   (:report
    (lambda (condition stream)
      (format stream "~@<Implicitly creating new generic function ~
-                     ~/sb-impl::print-symbol-with-prefix/.~:@>"
+                     ~/sb-ext:print-symbol-with-prefix/.~:@>"
              (implicit-generic-function-name condition)))))
 
 (define-condition extension-failure (reference-condition simple-error)
@@ -897,7 +899,7 @@
 (define-condition structure-initarg-not-keyword
     (reference-condition simple-style-warning)
   ()
-  (:default-initargs :references (list '(:ansi-cl :section (2 4 8 13)))))
+  (:default-initargs :references '((:ansi-cl :section (2 4 8 13)))))
 
 #!+sb-package-locks
 (progn
@@ -1078,9 +1080,9 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
                      "An attempt to access an array of element-type ~
                       NIL was made.  Congratulations!")))
   (:default-initargs
-      :references (list '(:ansi-cl :function sb!xc:upgraded-array-element-type)
-                        '(:ansi-cl :section (15 1 2 1))
-                        '(:ansi-cl :section (15 1 2 2)))))
+      :references '((:ansi-cl :function sb!xc:upgraded-array-element-type)
+                    (:ansi-cl :section (15 1 2 1))
+                    (:ansi-cl :section (15 1 2 2)))))
 
 (define-condition namestring-parse-error (parse-error)
   ((complaint :reader namestring-parse-error-complaint :initarg :complaint)
@@ -1165,10 +1167,10 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
                                                    simple-error)
   ()
   (:default-initargs
-   :format-control  "Symbol ~/sb-impl:print-symbol-with-prefix/ cannot ~
+   :format-control  "Symbol ~/sb-ext:print-symbol-with-prefix/ cannot ~
                      be both the name of a type and the name of a ~
                      declaration"
-   :references (list '(:ansi-cl :section (3 8 21)))))
+   :references '((:ansi-cl :section (3 8 21)))))
 
 ;;; Single stepping conditions
 
@@ -1251,21 +1253,21 @@ handled by any other handler, it will be muffled.")
 (define-condition redefinition-with-defun (function-redefinition-warning)
   ()
   (:report (lambda (warning stream)
-             (format stream "redefining ~/sb-impl::print-symbol-with-prefix/ ~
+             (format stream "redefining ~/sb-ext:print-symbol-with-prefix/ ~
                              in DEFUN"
                      (redefinition-warning-name warning)))))
 
 (define-condition redefinition-with-defmacro (function-redefinition-warning)
   ()
   (:report (lambda (warning stream)
-             (format stream "redefining ~/sb-impl::print-symbol-with-prefix/ ~
+             (format stream "redefining ~/sb-ext:print-symbol-with-prefix/ ~
                              in DEFMACRO"
                      (redefinition-warning-name warning)))))
 
 (define-condition redefinition-with-defgeneric (redefinition-warning)
   ()
   (:report (lambda (warning stream)
-             (format stream "redefining ~/sb-impl::print-symbol-with-prefix/ ~
+             (format stream "redefining ~/sb-ext:print-symbol-with-prefix/ ~
                              in DEFGENERIC"
                      (redefinition-warning-name warning)))))
 
@@ -1598,7 +1600,7 @@ conditions."))
   ;; warm load. (!CALL-A-METHOD does not understand method qualifiers)
   (define-deprecation-warning early-deprecation-warning style-warning nil
      "~%~@<~:@_In future~@[ ~A~] versions ~
-      ~/sb-impl:print-symbol-with-prefix/ will signal a full warning ~
+      ~/sb-ext:print-symbol-with-prefix/ will signal a full warning ~
       at compile-time.~:@>"
     "This warning is signaled when the use of a variable,
 function, type, etc. in :EARLY deprecation is detected at
@@ -1607,7 +1609,7 @@ error.")
 
   (define-deprecation-warning late-deprecation-warning warning t
      "~%~@<~:@_In future~@[ ~A~] versions ~
-      ~/sb-impl:print-symbol-with-prefix/ will signal a runtime ~
+      ~/sb-ext:print-symbol-with-prefix/ will signal a runtime ~
       error.~:@>"
     "This warning is signaled when the use of a variable,
 function, type, etc. in :LATE deprecation is detected at
@@ -1616,7 +1618,7 @@ error.")
 
   (define-deprecation-warning final-deprecation-warning warning t
      "~%~@<~:@_~*An error will be signaled at runtime for ~
-      ~/sb-impl:print-symbol-with-prefix/.~:@>"
+      ~/sb-ext:print-symbol-with-prefix/.~:@>"
     "This warning is signaled when the use of a variable,
 function, type, etc. in :FINAL deprecation is detected at
 compile-time. An error will be signaled at run-time."))
