@@ -49,6 +49,8 @@
            ,value)
        ,@doc)))
 
+(defmacro define-load-time-global (&rest args) `(defvar ,@args))
+
 ;;; The GENESIS function works with fasl code which would, in the
 ;;; target SBCL, work on ANSI-STREAMs (streams which aren't extended
 ;;; Gray streams). In ANSI Common Lisp, an ANSI-STREAM is just a
@@ -210,3 +212,5 @@
 (defun sap-int (x) (error "can't take SAP-INT ~S" x))
 ;;; Needed for FIXUP-CODE-OBJECT
 (defmacro without-gcing (&body body) `(progn ,@body))
+
+(defun logically-readonlyize (x) x)
