@@ -58,27 +58,26 @@
 (def!type index-or-minus-1 () `(integer -1 (,sb!xc:array-dimension-limit)))
 
 ;;; A couple of VM-related types that are currently used only on the
-;;; alpha platform. -- CSR, 2002-06-24
-(def!type unsigned-byte-with-a-bite-out (s bite)
-  (cond ((eq s '*) 'integer)
-        ((and (integerp s) (> s 0))
-         (let ((bound (ash 1 s)))
-           `(integer 0 ,(- bound bite 1))))
-        (t
-         (error "Bad size specified for UNSIGNED-BYTE type specifier: ~
-                  ~/sb!impl:print-type-specifier/."
-                s))))
+;;; alpha and mips platforms. -- CSR, 2002-06-24
+(def!type unsigned-byte-with-a-bite-out (size bite)
+  (unless (typep size '(integer 1))
+    (error "Bad size for the ~S type specifier: ~S."
+           'unsigned-byte-with-a-bite-out size))
+  (let ((bound (ash 1 size)))
+    `(integer 0 ,(- bound bite 1))))
 
-;;; Motivated by the mips port. -- CSR, 2002-08-22
-(def!type signed-byte-with-a-bite-out (s bite)
-  (cond ((eq s '*) 'integer)
-        ((and (integerp s) (> s 1))
-         (let ((bound (ash 1 (1- s))))
-           `(integer ,(- bound) ,(- bound bite 1))))
-        (t
-         (error "Bad size specified for SIGNED-BYTE type specifier: ~
-                  ~/sb!impl:print-type-specifier/."
-                s))))
+(def!type signed-byte-with-a-bite-out (size bite)
+  (unless (typep size '(integer 2))
+    (error "Bad size for ~S type specifier: ~S."
+            'signed-byte-with-a-bite-out size))
+  (let ((bound (ash 1 (1- size))))
+    `(integer ,(- bound) ,(- bound bite 1))))
+
+;;; The smallest power of two that is equal to or greater than X.
+(declaim (inline power-of-two-ceiling))
+(defun power-of-two-ceiling (x)
+  (declare (index x))
+  (ash 1 (integer-length (1- x))))
 
 (def!type load/store-index (scale lowtag min-offset
                                  &optional (max-offset min-offset))
@@ -203,10 +202,6 @@
       (and (consp x)
            (list-of-length-at-least-p (cdr x) (1- n)))))
 
-(declaim (inline ensure-list))
-(defun ensure-list (thing)
-  (if (listp thing) thing (list thing)))
-
 ;;; Is X is a positive prime integer?
 (defun positive-primep (x)
   ;; This happens to be called only from one place in sbcl-0.7.0, and
@@ -449,6 +444,19 @@
              (setq ,val (pop ,tail))
              (progn ,@body)))))
 
+;;; Like GETHASH if HASH-TABLE contains an entry for KEY.
+;;; Otherwise, evaluate DEFAULT, store the resulting value in
+;;; HASH-TABLE and return two values: 1) the result of evaluating
+;;; DEFAULT 2) NIL.
+(defmacro ensure-gethash (key hash-table &optional default)
+  (with-unique-names (n-key n-hash-table value foundp)
+    `(let ((,n-key ,key)
+           (,n-hash-table ,hash-table))
+       (multiple-value-bind (,value ,foundp) (gethash ,n-key ,n-hash-table)
+         (if ,foundp
+             (values ,value t)
+             (values (setf (gethash ,n-key ,n-hash-table) ,default) nil))))))
+
 ;;; (binding* ({(names initial-value [flag])}*) body)
 ;;; FLAG may be NIL or :EXIT-IF-NULL
 ;;;
@@ -884,7 +892,7 @@ NOTE: This interface is experimental and subject to change."
 ;;; Is NAME a legal variable/function name?
 (declaim (inline legal-variable-name-p))
 (defun legal-variable-name-p (name)
-  (typep name '(and symbol (not keyword))))
+  (typep name '(and symbol (not keyword) (not null))))
 
 (declaim (inline legal-fun-name-p))
 (defun legal-fun-name-p (name)
@@ -1218,6 +1226,12 @@ NOTE: This interface is experimental and subject to change."
 (defun print-type (stream type &optional colon at)
   (print-type-specifier stream (type-specifier type) colon at))
 
+(declaim (ftype (sfunction (index &key (:comma-interval (and (integer 1) index))) index)
+                decimal-with-grouped-digits-width))
+(defun decimal-with-grouped-digits-width (value &key (comma-interval 3))
+  (let ((digits (length (write-to-string value :base 10))))
+    (+ digits (floor (1- digits) comma-interval))))
+
 
 ;;;; etc.
 
@@ -1412,11 +1426,11 @@ NOTE: This interface is experimental and subject to change."
   ;; I don't think this is callable during cross-compilation, is it?
   (apply #'format stream
          "~#[~;~
-             Use ~/sb-impl:print-symbol-with-prefix/ instead.~;~
-             Use ~/sb-impl:print-symbol-with-prefix/ or ~
-             ~/sb-impl:print-symbol-with-prefix/ instead.~:;~
+             Use ~/sb-ext:print-symbol-with-prefix/ instead.~;~
+             Use ~/sb-ext:print-symbol-with-prefix/ or ~
+             ~/sb-ext:print-symbol-with-prefix/ instead.~:;~
              Use~@{~#[~; or~] ~
-             ~/sb-impl:print-symbol-with-prefix/~^,~} instead.~
+             ~/sb-ext:print-symbol-with-prefix/~^,~} instead.~
            ~]"
          replacements))
 
