@@ -14,17 +14,13 @@
 ;;;
 ;;; FIXME: These should be listed once and only once, instead of
 ;;; listed here and then listed separately (and by now, 2001-06-06,
-;;; slightly differently) elsewhere.
+;;; slightly differently) elsewhere. (Maybe this is resolved?)
 (declaim (special *posix-argv*
                   *core-string*
                   *stdin*
                   *stdout*
                   *stderr*
                   *tty*
-                  sb!vm:*read-only-space-free-pointer*
-                  sb!vm:*static-space-free-pointer*
-                  #!+immobile-space sb!vm:*immobile-fixedobj-free-pointer*
-                  #!+immobile-space sb!vm:*immobile-space-free-pointer*
                   sb!vm:*current-catch-block*
                   sb!vm::*current-unwind-protect-block*
                   sb!vm::*alien-stack-pointer*
@@ -44,7 +40,7 @@
                   sb!vm::*binding-stack-pointer*
                   sb!pcl::*cache-miss-values-stack*
                   sb!pcl::*dfun-miss-gfs-on-stack*))
-(!defvar sb!vm:*alloc-signal* nil)
+(defvar sb!vm:*alloc-signal*) ; initialized by create_thread_struct()
 ;;; This is a slot of 'struct thread' if multithreaded,
 ;;; and the symbol-global-value should never be used.
 ;;; (And in any case it is not really a special var)
