@@ -123,7 +123,7 @@
        (tagbody retry
           (let ((fdefn (,lookup-fn name)))
             (when fdefn
-              (let ((f (fdefn-fun  (truly-the fdefn fdefn))))
+              (let ((f (fdefn-fun (truly-the fdefn fdefn))))
                 ;; If STRICTLY-FUNCTIONP is true, we make sure not to return an error
                 ;; trampoline. This extra check ensures that full calls such as
                 ;; (MAPCAR 'OR '()) signal an error that OR isn't a function.
@@ -135,20 +135,25 @@
                          '(unless (macro/special-guard-fun-p f))
                          '(progn))
                    (return f)))))
-            (restart-case (error 'undefined-function :name name)
-              (continue ()
-                :report (lambda (stream)
-                          (format stream "Retry using ~s." name)))
-              (use-value (value)
-                :report (lambda (stream)
-                          (format stream "Use specified function"))
-                :interactive read-evaluated-form
-                (when (functionp value)
-                  (return value))
-                (setf name (the ,(if (eq lookup-fn 'symbol-fdefn)
-                                     'symbol
-                                     t)
-                                value))))
+            (setf name
+                  (let ((name name))
+                    ;; Avoid making the initial NAME a value cell,
+                    ;; it will cons even if no restarts are reached
+                    (restart-case (error 'undefined-function :name name)
+                      (continue ()
+                        :report (lambda (stream)
+                                  (format stream "Retry using ~s." name))
+                        name)
+                      (use-value (value)
+                        :report (lambda (stream)
+                                  (format stream "Use specified function"))
+                        :interactive read-evaluated-form
+                        (when (functionp value)
+                          (return value))
+                        (the ,(if (eq lookup-fn 'symbol-fdefn)
+                                  'symbol
+                                  t)
+                             value)))))
             (go retry))))))
 
 ;; Return T if FUNCTION is the error-signaling trampoline
@@ -344,10 +349,9 @@
 ;; so happens to be a function in SBCL, but which must not be
 ;; bound to a function-name by way of (SETF FEDFINITION).
 (defun err-if-unacceptable-function (object setter)
-  (declare (notinline macro/special-guard-fun-p)) ; not performance-critical
   (when (macro/special-guard-fun-p object)
     (error 'simple-reference-error
-           :references (list '(:ansi-cl :function fdefinition))
+           :references '((:ansi-cl :function fdefinition))
            :format-control "~S is not acceptable to ~S."
            :format-arguments (list object setter))))
 
