@@ -182,8 +182,15 @@
             (incf ptr)))))
     res))
 
-(!define-fop 45 (fop-layout (name inherits depthoid length bitmap))
-  (find-and-init-or-check-layout name length inherits depthoid bitmap))
+(!define-fop 45 :not-host (fop-layout (name inherits depthoid length bitmap))
+  (let ((flags
+         (case (and (>= (length (the simple-vector inherits)) 2)
+                    (elt inherits 1))
+           (#.(find-layout 'structure-object) +structure-layout-flag+)
+           (#.(find-layout 'condition) +condition-layout-flag+)
+           ;; DUMP-LAYOUT avers that +pcl-object-flag+ layouts are never dumped
+           (t 0))))
+    (find-and-init-or-check-layout name length flags inherits depthoid bitmap)))
 
 ;; Allocate a CLOS object. This is used when the compiler detects that
 ;; MAKE-LOAD-FORM returned a simple use of MAKE-LOAD-FORM-SAVING-SLOTS,
@@ -622,27 +629,34 @@
   code-object)
 
 #!+immobile-space
-(!define-fop 134 :not-host (fop-immobile-obj-fixup (code-object kind obj))
-  (sb!vm:fixup-code-object code-object
-                           (read-word-arg (fasl-input-stream))
-                           (get-lisp-obj-address obj) ; OBJ can't move
-                           kind :immobile-object)
-  code-object)
+(progn
+  (!define-fop 133 :not-host (fop-layout-fixup (code-object kind obj))
+    (sb!vm:fixup-code-object code-object
+                             (read-word-arg (fasl-input-stream))
+                             (get-lisp-obj-address (find-layout obj))
+                             kind :immobile-object)
+    code-object)
+  (!define-fop 134 :not-host (fop-immobile-obj-fixup (code-object kind obj))
+    (sb!vm:fixup-code-object code-object
+                             (read-word-arg (fasl-input-stream))
+                             (get-lisp-obj-address obj) ; OBJ can't move
+                             kind :immobile-object)
+    code-object))
 
 #!+immobile-code
 (progn
-  (!define-fop 133 :not-host (fop-named-call-fixup (code-object kind name))
+  (!define-fop 135 :not-host (fop-named-call-fixup (code-object kind name))
     (sb!vm:fixup-code-object code-object
                              (read-word-arg (fasl-input-stream))
                              (sb!vm::fdefn-entry-address name)
                              kind :named-call)
     code-object)
 
-  (!define-fop 135 :not-host (fop-static-call-fixup (code-object kind name))
+  (!define-fop 136 :not-host (fop-static-call-fixup (code-object kind name))
     (sb!vm:fixup-code-object code-object
                              (read-word-arg (fasl-input-stream))
                              (sb!vm::function-raw-address name)
-                             kind)
+                             kind :static-call)
     code-object))
 
 (!define-fop 147 :not-host (fop-foreign-fixup (code-object kind))
@@ -660,7 +674,7 @@
     (unless found
       (error "undefined assembler routine: ~S" routine))
     (sb!vm:fixup-code-object code-object (read-word-arg (fasl-input-stream))
-                             value kind))
+                             value kind :assembly-routine))
   code-object)
 
 (!define-fop 149 :not-host (fop-code-object-fixup (code-object kind))
@@ -735,6 +749,7 @@
                           specs))))
   (frob (#x6c t)
         (#x6d structure-object)
+        (#x6e condition)
         (#x6f definition-source-location)
         (#x70 sb!c::debug-fun)
         (#x71 sb!c::compiled-debug-fun)
