@@ -13,6 +13,20 @@
 
 ;;;; DYNAMIC-USAGE and friends
 
+#!+(and relocatable-heap gencgc)
+(define-alien-variable ("DYNAMIC_SPACE_START" sb!vm:dynamic-space-start) unsigned-long)
+#!-sb-fluid
+(declaim (inline current-dynamic-space-start))
+(defun current-dynamic-space-start ()
+  #!+gencgc sb!vm:dynamic-space-start
+  #!-gencgc (extern-alien "current_dynamic_space" unsigned-long))
+
+#!+(or x86 x86-64)
+(progn
+  (declaim (inline dynamic-space-free-pointer))
+  (defun dynamic-space-free-pointer ()
+    (extern-alien "dynamic_space_free_pointer" system-area-pointer)))
+
 #!-sb-fluid
 (declaim (inline dynamic-usage))
 #!+gencgc
@@ -24,73 +38,36 @@
              (- (sap-int (sb!c::dynamic-space-free-pointer))
                 (current-dynamic-space-start))))
 
+#!+immobile-space
+(progn
+  #!+relocatable-heap
+  (define-alien-variable ("IMMOBILE_SPACE_START" sb!vm:immobile-space-start) unsigned-long)
+  (defun immobile-space-addr-p (addr)
+    (declare (type word addr))
+    (let ((start sb!vm:immobile-space-start))
+      (<= start addr (truly-the word (+ start sb!vm:immobile-space-size)))))
+  (defun immobile-space-obj-p (obj)
+    (immobile-space-addr-p (get-lisp-obj-address obj))))
+
 (defun static-space-usage ()
-  (- (ash sb!vm:*static-space-free-pointer* sb!vm:n-fixnum-tag-bits)
-     sb!vm:static-space-start))
+  (- (sap-int sb!vm:*static-space-free-pointer*) sb!vm:static-space-start))
 
 (defun read-only-space-usage ()
-  (- (ash sb!vm::*read-only-space-free-pointer* sb!vm:n-fixnum-tag-bits)
-     sb!vm:read-only-space-start))
+  (- (sap-int sb!vm:*read-only-space-free-pointer*) sb!vm:read-only-space-start))
+
+;;; Convert the descriptor into a SAP. The bits all stay the same, we just
+;;; change our notion of what we think they are.
+(declaim (inline descriptor-sap))
+(defun descriptor-sap (x) (int-sap (get-lisp-obj-address x)))
 
 (defun control-stack-usage ()
   #!-stack-grows-downward-not-upward
-  (- (sap-int (sb!c::control-stack-pointer-sap))
-     (sap-int (sb!di::descriptor-sap sb!vm:*control-stack-start*)))
+  (sap- (control-stack-pointer-sap) (descriptor-sap sb!vm:*control-stack-start*))
   #!+stack-grows-downward-not-upward
-  (- (sap-int (sb!di::descriptor-sap sb!vm:*control-stack-end*))
-     (sap-int (sb!c::control-stack-pointer-sap))))
+  (sap- (descriptor-sap sb!vm:*control-stack-end*) (control-stack-pointer-sap)))
 
 (defun binding-stack-usage ()
-  (- (sap-int (sb!c::binding-stack-pointer-sap))
-     (sap-int (sb!di::descriptor-sap sb!vm:*binding-stack-start*))))
-
-;;;; ROOM
-
-(defun room-minimal-info ()
-  (format t "Dynamic space usage is:   ~10:D bytes.~%" (dynamic-usage))
-  (format t "Read-only space usage is: ~10:D bytes.~%" (read-only-space-usage))
-  (format t "Static space usage is:    ~10:D bytes.~%" (static-space-usage))
-  (format t "Control stack usage is:   ~10:D bytes.~%" (control-stack-usage))
-  (format t "Binding stack usage is:   ~10:D bytes.~%" (binding-stack-usage))
-  #!+sb-thread
-  (format t
-          "Control and binding stack usage is for the current thread only.~%")
-  (format t "Garbage collection is currently ~:[enabled~;DISABLED~].~%"
-          *gc-inhibit*))
-
-(defun room-intermediate-info ()
-  (room-minimal-info)
-  (sb!vm:memory-usage :count-spaces '(:dynamic)
-                      :print-spaces t
-                      :cutoff 0.05f0
-                      :print-summary nil))
-
-(defun room-maximal-info ()
-  ;; FIXME: SB!VM:INSTANCE-USAGE calls suppressed until bug 344 is fixed
-  (room-intermediate-info)
-  ;; old way, could be restored when bug 344 fixed:
-  ;;x (room-minimal-info)
-  ;;x (sb!vm:memory-usage :count-spaces '(:static :dynamic))
-  ;;x (sb!vm:instance-usage :dynamic :top-n 10)
-  ;;x (sb!vm:instance-usage :static :top-n 10)
-  )
-
-(defun room (&optional (verbosity :default))
-  "Print to *STANDARD-OUTPUT* information about the state of internal
-  storage and its management. The optional argument controls the
-  verbosity of output. If it is T, ROOM prints out a maximal amount of
-  information. If it is NIL, ROOM prints out a minimal amount of
-  information. If it is :DEFAULT or it is not supplied, ROOM prints out
-  an intermediate amount of information."
-  (fresh-line)
-  (ecase verbosity
-    ((t)
-     (room-maximal-info))
-    ((nil)
-     (room-minimal-info))
-    (:default
-     (room-intermediate-info)))
-  (values))
+  (sap- (binding-stack-pointer-sap) (descriptor-sap sb!vm:*binding-stack-start*)))
 
 ;;;; GET-BYTES-CONSED
 
@@ -101,7 +78,7 @@
 ;;; ever allocated by adding this to the number of bytes currently
 ;;; allocated and never freed.)
 (declaim (type unsigned-byte *n-bytes-freed-or-purified*))
-(defvar *n-bytes-freed-or-purified* 0)
+(defglobal *n-bytes-freed-or-purified* 0)
 (defun gc-reinit ()
   (setq *gc-inhibit* nil)
   (gc)
@@ -190,7 +167,7 @@ statistics are appended to it."
 
 (define-load-time-global *already-in-gc* (sb!thread:make-mutex :name "GC lock"))
 
-(defun sub-gc (&key (gen 0))
+(defun sub-gc (gen)
   (cond (*gc-inhibit*
          (setf *gc-pending* t)
          nil)
@@ -203,7 +180,7 @@ statistics are appended to it."
                         (new-usage 0)
                         (start-time (get-internal-run-time)))
                     (collect-garbage gen)
-                    (setf *gc-epoch* (cons nil nil))
+                    (setf *gc-epoch* (cons 0 0))
                     (let ((run-time (- (get-internal-run-time) start-time)))
                       ;; KLUDGE: Sometimes we see the second getrusage() call
                       ;; return a smaller value than the first, which can
@@ -333,7 +310,7 @@ If GEN is provided, it can be used to specify the oldest generation
 guaranteed to be collected."
   #!-gencgc (declare (ignore full))
   (let (#!+gencgc (gen (if full sb!vm:+pseudo-static-generation+ gen)))
-    (when (eq t (sub-gc :gen gen))
+    (when (eq t (sub-gc gen))
       (post-gc))))
 
 (define-alien-routine scrub-control-stack void)
