@@ -233,6 +233,27 @@
              (declare (type index i)))))
     (typecase n
       (index (fast-nthcdr n list))
+      ;; Such a large list can only be circular
+      (t (do ((i 0 (1+ i))
+              (r-i list (cdr r-i))
+              (r-2i list (cddr r-2i)))
+             ((and (eq r-i r-2i) (not (zerop i)))
+              (fast-nthcdr (mod n i) r-i))
+           (declare (type index i)))))))
+
+;;; For [n]butlast
+(defun dotted-nthcdr (n list)
+  (flet ((fast-nthcdr (n list)
+           (declare (type index n))
+           (do ((i n (1- i))
+                (result list (cdr result)))
+               ((not (plusp i)) result)
+             (declare (type index i))
+             (when (atom result)
+               (return)))))
+    (typecase n
+      (index (fast-nthcdr n list))
+      ;; Such a large list can only be circular
       (t (do ((i 0 (1+ i))
               (r-i list (cdr r-i))
               (r-2i list (cddr r-2i)))
@@ -514,13 +535,14 @@
       ((atom 2nd) 3rd)
     (rplacd 2nd 3rd)))
 
+
 (defun butlast (list &optional (n 1))
   (cond ((zerop n)
          (copy-list list))
         ((not (typep n 'index))
          nil)
         (t
-         (let ((head (nthcdr (1- n) list)))
+         (let ((head (dotted-nthcdr (1- n) list)))
            (and (consp head)      ; there are at least n
                 (collect ((copy)) ; conses; copy!
                   (do ((trail list (cdr trail))
@@ -538,7 +560,7 @@
         ((not (typep n 'index))
          nil)
         (t
-         (let ((head (nthcdr (1- n) list)))
+         (let ((head (dotted-nthcdr (1- n) list)))
            (and (consp head)       ; there are more than n
                 (consp (cdr head)) ; conses.
                 ;; TRAIL trails by n cons to be able to
@@ -766,18 +788,19 @@
 (defmacro nsublis-macro ()
   (let ((key-tmp (gensym)))
     `(let ((,key-tmp (apply-key key subtree)))
-      (if notp
-          (assoc ,key-tmp alist :test-not test-not)
-          (assoc ,key-tmp alist :test test)))))
+       (if notp
+           (assoc ,key-tmp alist :test-not test-not)
+           (assoc ,key-tmp alist :test test)))))
 
 (defun nsublis (alist tree &key key (test #'eql testp) (test-not #'eql notp))
-  "Substitute from ALIST into TRUE destructively."
+  "Substitute from ALIST into TREE destructively."
   (when (and testp notp)
     (error ":TEST and :TEST-NOT were both supplied."))
   (let ((key (and key (%coerce-callable-to-fun key)))
         (test (if testp (%coerce-callable-to-fun test) test))
         (test-not (if notp (%coerce-callable-to-fun test-not) test-not)))
-    (declare (inline assoc))
+    (declare (type function test test-not)
+             (inline assoc))
     (let (temp)
       (labels ((s (subtree)
                  (cond ((setq temp (nsublis-macro))
