@@ -24,9 +24,9 @@
 ;;;
 ;;; ASSERT-ERROR isn't defined until a later file because it uses the
 ;;; macro RESTART-CASE, which isn't defined until a later file.
-(defmacro-mundanely assert (test-form &optional places datum &rest arguments
+;;;
+(sb!xc:defmacro assert (test-form &optional places datum &rest arguments
                             &environment env)
-  #!+sb-doc
   "Signals an error if the value of TEST-FORM is NIL. Returns NIL.
 
    Optional DATUM and ARGUMENTS can be used to change the signaled
@@ -45,53 +45,52 @@
    CL:OR, CL:AND, etc.) the results of evaluating the ARGs will be
    included in the error report if the assertion fails."
   (collect ((bindings) (infos))
-    (let ((new-test
-            (flet ((process-place (place)
-                     (if (sb!xc:constantp place env)
-                         place
-                         (with-unique-names (temp)
-                           (bindings `(,temp ,place))
-                           (infos `(list ',place ,temp))
-                           temp))))
-              (cond
-                ;; TEST-FORM looks like a function call. We do not
-                ;; attempt this if TEST-FORM is the application of a
-                ;; special operator because of argument evaluation
-                ;; order issues.
-               ((when (typep test-form '(cons symbol list))
-                   (let* ((name (first test-form))
-                          (global-fun-p
-                           (eq (info :function :kind name) :function)))
-                     (when (typecase env
-                             (sb!kernel:lexenv
-                              (let ((f (cdr (assoc name (sb!c::lexenv-funs env)))))
-                                (if (not f) global-fun-p (sb!c::functional-p f))))
-                             #!+sb-fasteval
-                             (sb!interpreter:basic-env
-                              (let ((kind
-                                     (sb!interpreter::find-lexical-fun env name)))
-                                (if (null kind) global-fun-p (eq kind :function))))
-                             (null global-fun-p))
-                       `(,name ,@(mapcar #'process-place (rest test-form)))))))
-                ;; For all other cases, just evaluate TEST-FORM and do
-                ;; not report any details if the assertion fails.
-                (t
-                 test-form)))))
+    (let* ((func (if (listp test-form) (car test-form)))
+           (new-test
+            (if (and (typep func '(and symbol (not null)))
+                     (not (sb!xc:macro-function func env))
+                     (not (sb!xc:special-operator-p func))
+                     (proper-list-p (cdr test-form)))
+                ;; TEST-FORM is a function call. We do not attempt this
+                ;; if TEST-FORM is a macro invocation or special form.
+                `(,func ,@(mapcar (lambda (place)
+                                    (if (sb!xc:constantp place env)
+                                        place
+                                        (with-unique-names (temp)
+                                          (bindings `(,temp ,place))
+                                          (infos `(list ',place ,temp))
+                                          temp)))
+                                  (rest test-form)))
+                ;; For all other cases, just evaluate TEST-FORM
+                ;; and don't report any details if the assertion fails.
+                test-form))
+           (try '#:try)
+           (done  '#:done))
       ;; If TEST-FORM, potentially using values from BINDINGS, does not
       ;; hold, enter a loop which reports the assertion error,
       ;; potentially changes PLACES, and retries TEST-FORM.
       `(tagbody
-        :try
+        ,try
           (let ,(bindings)
             (when ,new-test
-              (go :done))
-            (assert-error ',test-form (list ,@(infos))
-                          ',places ,datum ,@arguments))
+              (go ,done))
+
+            (assert-error ',test-form
+                          ,@(and (or (infos) places datum
+                                     arguments)
+                                 `((list ,@(infos))))
+                          ,@(and (or places datum
+                                     arguments)
+                                 `(',places))
+                          ,@(and (or places datum
+                                     arguments)
+                                 `(,datum))
+                          ,@arguments))
           ,@(mapcar (lambda (place)
                       `(setf ,place (assert-prompt ',place ,place)))
                     places)
-          (go :try)
-        :done))))
+          (go ,try)
+        ,done))))
 
 (defun assert-prompt (name value)
   (cond ((y-or-n-p "The old value of ~S is ~S.~
@@ -114,9 +113,8 @@
 ;;;
 ;;; CHECK-TYPE-ERROR isn't defined until a later file because it uses
 ;;; the macro RESTART-CASE, which isn't defined until a later file.
-(defmacro-mundanely check-type (place type &optional type-string
+(sb!xc:defmacro check-type (place type &optional type-string
                                 &environment env)
-  #!+sb-doc
   "Signal a restartable error of type TYPE-ERROR if the value of PLACE
 is not of the specified type. If an error is signalled and the restart
 is used to return, this can only return if the STORE-VALUE restart is
@@ -126,7 +124,7 @@ invoked. In that case it will store into PLACE and start over."
     (error 'simple-reference-error
            :format-control "Quoted type specifier in ~S: ~S"
            :format-arguments (list 'check-type type)
-           :references (list '(:ansi-cl :macro check-type))))
+           :references '((:ansi-cl :macro check-type))))
   ;; KLUDGE: We use a simpler form of expansion if PLACE is just a
   ;; variable to work around Python's blind spot in type derivation.
   ;; For more complex places getting the type derived should not
@@ -135,16 +133,20 @@ invoked. In that case it will store into PLACE and start over."
     (if (symbolp expanded)
         `(do ()
              ((typep ,place ',type))
-          (setf ,place (check-type-error ',place ,place ',type ,type-string)))
+           (setf ,place (check-type-error ',place ,place ',type
+                                          ,@(and type-string
+                                                 `(,type-string)))))
         (let ((value (gensym)))
           `(do ((,value ,place ,place))
                ((typep ,value ',type))
-            (setf ,place
-                  (check-type-error ',place ,value ',type ,type-string)))))))
+             (setf ,place
+                   (check-type-error ',place ,value ',type
+                                     ,@(and type-string
+                                            `(,type-string)))))))))
 
 ;;;; DEFINE-SYMBOL-MACRO
 
-(defmacro-mundanely define-symbol-macro (name expansion)
+(sb!xc:defmacro define-symbol-macro (name expansion)
   `(eval-when (:compile-toplevel :load-toplevel :execute)
     (sb!c::%define-symbol-macro ',name ',expansion (sb!c:source-location))))
 
@@ -158,7 +160,7 @@ invoked. In that case it will store into PLACE and start over."
   (let ((kind (info :variable :kind name)))
     (case kind
      ((:macro :unknown)
-      (sb!c:with-source-location (source-location)
+      (when source-location
         (setf (info :source-location :symbol-macro name) source-location))
       (setf (info :variable :kind name) :macro)
       (setf (info :variable :macro-expansion name) expansion))
@@ -176,8 +178,7 @@ invoked. In that case it will store into PLACE and start over."
 
 ;;;; DEFINE-COMPILER-MACRO
 
-(defmacro-mundanely define-compiler-macro (name lambda-list &body body)
-  #!+sb-doc
+(sb!xc:defmacro define-compiler-macro (name lambda-list &body body)
   "Define a compiler-macro for NAME."
   (legal-fun-name-or-type-error name)
   (when (and (symbolp name) (special-operator-p name))
@@ -250,77 +251,74 @@ invoked. In that case it will store into PLACE and start over."
          ((null cases) nil)
       (flet ((check-clause (case-keys)
                (loop for k in case-keys
-                     for existing = (gethash k keys-seen)
-                     do (when existing
-                          (let ((sb!c::*current-path*
-                                  (when (boundp 'sb!c::*source-paths*)
-                                    (or (sb!c::get-source-path case)
-                                        (and (boundp 'sb!c::*current-path*)
-                                             sb!c::*current-path*)))))
-                            (warn 'duplicate-case-key-warning
-                                  :key k
-                                  :case-kind name
-                                  :occurrences `(,existing (,case-position (,case)))))))
+                  for existing = (gethash k keys-seen)
+                  do (when existing
+                       (warn 'duplicate-case-key-warning
+                             :key k
+                             :case-kind name
+                             :occurrences `(,existing (,case-position (,case))))))
                (let ((record (list case-position (list case))))
                  (dolist (k case-keys)
                    (setf (gethash k keys-seen) record)))))
         (unless (list-of-length-at-least-p case 1)
-          (error "~S -- bad clause in ~S" case name))
-        (destructuring-bind (keyoid &rest forms) case
-          (cond (;; an OTHERWISE-CLAUSE
-                 ;;
-                 ;; By the way... The old code here tried gave
-                 ;; STYLE-WARNINGs for normal-clauses which looked as
-                 ;; though they might've been intended to be
-                 ;; otherwise-clauses. As Tony Martinez reported on
-                 ;; sbcl-devel 2004-11-09 there are sometimes good
-                 ;; reasons to write clauses like that; and as I noticed
-                 ;; when trying to understand the old code so I could
-                 ;; understand his patch, trying to guess which clauses
-                 ;; don't have good reasons is fundamentally kind of a
-                 ;; mess. SBCL does issue style warnings rather
-                 ;; enthusiastically, and I have often justified that by
-                 ;; arguing that we're doing that to detect issues which
-                 ;; are tedious for programmers to detect for by
-                 ;; proofreading (like small typoes in long symbol
-                 ;; names, or duplicate function definitions in large
-                 ;; files). This doesn't seem to be an issue like that,
-                 ;; and I can't think of a comparably good justification
-                 ;; for giving STYLE-WARNINGs for legal code here, so
-                 ;; now we just hope the programmer knows what he's
-                 ;; doing. -- WHN 2004-11-20
-                 (and (not errorp) ; possible only in CASE or TYPECASE,
-                                   ; not in [EC]CASE or [EC]TYPECASE
-                      (memq keyoid '(t otherwise))
-                      (null (cdr cases)))
-                 (push `(t nil ,@forms) clauses))
-                ((and multi-p (listp keyoid))
-                 (setf keys (nconc (reverse keyoid) keys))
-                 (check-clause keyoid)
-                 (push `((or ,@(mapcar (lambda (key)
-                                         `(,test ,keyform-value ',key))
-                                       keyoid))
-                         nil
-                         ,@forms)
-                       clauses))
-                (t
-                 (when (and (eq name 'case)
-                            (cdr cases)
-                            (memq keyoid '(t otherwise)))
-                   (error 'simple-reference-error
-                          :format-control
-                          "~@<~IBad ~S clause:~:@_  ~S~:@_~S allowed as the key ~
+          (with-current-source-form (cases)
+            (error "~S -- bad clause in ~S" case name)))
+        (with-current-source-form (case)
+          (destructuring-bind (keyoid &rest forms) case
+            (cond (;; an OTHERWISE-CLAUSE
+                   ;;
+                   ;; By the way... The old code here tried gave
+                   ;; STYLE-WARNINGs for normal-clauses which looked as
+                   ;; though they might've been intended to be
+                   ;; otherwise-clauses. As Tony Martinez reported on
+                   ;; sbcl-devel 2004-11-09 there are sometimes good
+                   ;; reasons to write clauses like that; and as I noticed
+                   ;; when trying to understand the old code so I could
+                   ;; understand his patch, trying to guess which clauses
+                   ;; don't have good reasons is fundamentally kind of a
+                   ;; mess. SBCL does issue style warnings rather
+                   ;; enthusiastically, and I have often justified that by
+                   ;; arguing that we're doing that to detect issues which
+                   ;; are tedious for programmers to detect for by
+                   ;; proofreading (like small typoes in long symbol
+                   ;; names, or duplicate function definitions in large
+                   ;; files). This doesn't seem to be an issue like that,
+                   ;; and I can't think of a comparably good justification
+                   ;; for giving STYLE-WARNINGs for legal code here, so
+                   ;; now we just hope the programmer knows what he's
+                   ;; doing. -- WHN 2004-11-20
+                   (and (not errorp) ; possible only in CASE or TYPECASE,
+                                        ; not in [EC]CASE or [EC]TYPECASE
+                        (memq keyoid '(t otherwise))
+                        (null (cdr cases)))
+                   (push `(t nil ,@forms) clauses))
+                  ((and multi-p (listp keyoid))
+                   (setf keys (nconc (reverse keyoid) keys))
+                   (check-clause keyoid)
+                   (push `((or ,@(mapcar (lambda (key)
+                                           `(,test ,keyform-value ',key))
+                                         keyoid))
+                           nil
+                           ,@forms)
+                         clauses))
+                  (t
+                   (when (and (eq name 'case)
+                              (cdr cases)
+                              (memq keyoid '(t otherwise)))
+                     (error 'simple-reference-error
+                            :format-control
+                            "~@<~IBad ~S clause:~:@_  ~S~:@_~S allowed as the key ~
                            designator only in the final otherwise-clause, not in a ~
                            normal-clause. Use (~S) instead, or move the clause the ~
                            correct position.~:@>"
-                          :format-arguments (list 'case case keyoid keyoid)
-                          :references `((:ansi-cl :macro case))))
-                 (push keyoid keys)
-                 (check-clause (list keyoid))
-                 (push `((,test ,keyform-value ',keyoid)
-                         nil
-                         ,@forms)
-                       clauses))))))
+                            :format-arguments (list 'case case keyoid keyoid)
+                            :references `((:ansi-cl :macro case))))
+                   (push keyoid keys)
+                   (check-clause (list keyoid))
+                   (push `((,test ,keyform-value ',keyoid)
+                           nil
+                           ,@forms)
+                         clauses)))))))
     (setq keys
           (nreverse (mapcon (lambda (tail)
                               (unless (member (car tail) (cdr tail))
@@ -347,93 +345,107 @@ invoked. In that case it will store into PLACE and start over."
         `(let ((,keyform-value ,keyform))
            (block ,block
              (tagbody
-              ,again
-              (return-from
-               ,block
-               (cond ,@(nreverse clauses)
-                     (t
-                      (setf ,keyform-value
-                            (setf ,keyform
-                                  (case-body-error
-                                   ',name ',keyform ,keyform-value
-                                   ',expected-type ',keys)))
-                      (go ,again))))))))
+                ,again
+                (return-from
+                 ,block
+                  (cond ,@(nreverse clauses)
+                        (t
+                         (setf ,keyform-value
+                               (setf ,keyform
+                                     (case-body-error
+                                      ',name ',keyform ,keyform-value
+                                      ',expected-type ',keys)))
+                         (go ,again))))))))
       `(let ((,keyform-value ,keyform))
          (declare (ignorable ,keyform-value)) ; e.g. (CASE KEY (T))
          (cond
-          ,@(nreverse clauses)
-          ,@(if errorp
-                `((t (case-failure ',name ,keyform-value ',keys))))))))
+           ,@(nreverse clauses)
+           ,@(when errorp
+               `((t (,(ecase name
+                        (etypecase 'etypecase-failure)
+                        (ecase 'ecase-failure))
+                     ,keyform-value ',keys))))))))
 ) ; EVAL-WHEN
 
-(defmacro-mundanely case (keyform &body cases)
-  #!+sb-doc
+(sb!xc:defmacro case (keyform &body cases)
   "CASE Keyform {({(Key*) | Key} Form*)}*
   Evaluates the Forms in the first clause with a Key EQL to the value of
   Keyform. If a singleton key is T then the clause is a default clause."
   (case-body 'case keyform cases t 'eql nil nil nil))
 
-(defmacro-mundanely ccase (keyform &body cases)
-  #!+sb-doc
+(sb!xc:defmacro ccase (keyform &body cases)
   "CCASE Keyform {({(Key*) | Key} Form*)}*
   Evaluates the Forms in the first clause with a Key EQL to the value of
   Keyform. If none of the keys matches then a correctable error is
   signalled."
   (case-body 'ccase keyform cases t 'eql t t t))
 
-(defmacro-mundanely ecase (keyform &body cases)
-  #!+sb-doc
+(sb!xc:defmacro ecase (keyform &body cases)
   "ECASE Keyform {({(Key*) | Key} Form*)}*
   Evaluates the Forms in the first clause with a Key EQL to the value of
   Keyform. If none of the keys matches then an error is signalled."
   (case-body 'ecase keyform cases t 'eql t nil t))
 
-(defmacro-mundanely typecase (keyform &body cases)
-  #!+sb-doc
+(sb!xc:defmacro typecase (keyform &body cases)
   "TYPECASE Keyform {(Type Form*)}*
   Evaluates the Forms in the first clause for which TYPEP of Keyform and Type
   is true."
   (case-body 'typecase keyform cases nil 'typep nil nil nil))
 
-(defmacro-mundanely ctypecase (keyform &body cases)
-  #!+sb-doc
+(sb!xc:defmacro ctypecase (keyform &body cases)
   "CTYPECASE Keyform {(Type Form*)}*
   Evaluates the Forms in the first clause for which TYPEP of Keyform and Type
   is true. If no form is satisfied then a correctable error is signalled."
   (case-body 'ctypecase keyform cases nil 'typep t t t))
 
-(defmacro-mundanely etypecase (keyform &body cases)
-  #!+sb-doc
+(sb!xc:defmacro etypecase (keyform &body cases)
   "ETYPECASE Keyform {(Type Form*)}*
   Evaluates the Forms in the first clause for which TYPEP of Keyform and Type
   is true. If no form is satisfied then an error is signalled."
   (case-body 'etypecase keyform cases nil 'typep t nil t))
+
+;;; Compile a version of BODY for all TYPES, and dispatch to the
+;;; correct one based on the value of VAR. This was originally used
+;;; only for strings, hence the name. Renaming it to something more
+;;; generic might not be a bad idea.
+(sb!xc:defmacro string-dispatch ((&rest types) var &body body)
+  (let ((fun (sb!xc:gensym "STRING-DISPATCH-FUN")))
+    `(flet ((,fun (,var)
+              ,@body))
+       (declare (inline ,fun))
+       (etypecase ,var
+         ,@(loop for type in types
+                 ;; TRULY-THE allows transforms to take advantage of the type
+                 ;; information without need for constraint propagation.
+                 collect `(,type (,fun (truly-the ,type ,var))))))))
 
 ;;;; WITH-FOO i/o-related macros
 
-(defmacro-mundanely with-open-stream ((var stream) &body forms-decls)
-  (multiple-value-bind (forms decls)
-      (parse-body forms-decls :doc-string-allowed nil)
+(sb!xc:defmacro with-open-stream ((var stream) &body body)
+  (multiple-value-bind (forms decls) (parse-body body nil)
+    `(let ((,var ,stream))
+       ,@decls
+       (unwind-protect
+            (progn ,@forms)
+         (close ,var)))))
+
+(sb!xc:defmacro with-open-file ((stream filespec &rest options)
+                                &body body)
+  (multiple-value-bind (forms decls) (parse-body body nil)
     (let ((abortp (gensym)))
-      `(let ((,var ,stream)
+      `(let ((,stream (open ,filespec ,@options))
              (,abortp t))
          ,@decls
          (unwind-protect
-             (multiple-value-prog1
-              (progn ,@forms)
-              (setq ,abortp nil))
-           (when ,var
-             (close ,var :abort ,abortp)))))))
+              (multiple-value-prog1
+                  (progn ,@forms)
+                (setq ,abortp nil))
+           (when ,stream
+             (close ,stream :abort ,abortp)))))))
 
-(defmacro-mundanely with-open-file ((stream filespec &rest options)
-                                    &body body)
-  `(with-open-stream (,stream (open ,filespec ,@options))
-     ,@body))
-
-(defmacro-mundanely with-input-from-string ((var string &key index start end)
+(sb!xc:defmacro with-input-from-string ((var string &key index start end)
                                             &body forms-decls)
-  (multiple-value-bind (forms decls)
-      (parse-body forms-decls :doc-string-allowed nil)
+  (multiple-value-bind (forms decls) (parse-body forms-decls nil)
     `(let ((,var
             ;; Should (WITH-INPUT-FROM-STRING (stream str :start nil :end 5))
             ;; pass the explicit NIL, and thus get an error? It's logical
@@ -450,11 +462,10 @@ invoked. In that case it will store into PLACE and start over."
            ,@(when index
                `((setf ,index (string-input-stream-current ,var))))))))
 
-(defmacro-mundanely with-output-to-string
+(sb!xc:defmacro with-output-to-string
     ((var &optional string &key (element-type ''character))
      &body forms-decls)
-  (multiple-value-bind (forms decls)
-      (parse-body forms-decls :doc-string-allowed nil)
+  (multiple-value-bind (forms decls) (parse-body forms-decls nil)
     (if string
         (let ((element-type-var (gensym)))
           `(let ((,var (make-fill-pointer-output-stream ,string))
@@ -480,8 +491,7 @@ invoked. In that case it will store into PLACE and start over."
 
 ;;;; miscellaneous macros
 
-(defmacro-mundanely nth-value (n form)
-  #!+sb-doc
+(sb!xc:defmacro nth-value (n form &environment env)
   "Evaluate FORM and return the Nth value (zero based)
  without consing a temporary list of values."
   ;; FIXME: The above is true, if slightly misleading.  The
@@ -491,21 +501,22 @@ invoked. In that case it will store into PLACE and start over."
   ;; form will take longer than can be described as adequate, as the
   ;; optional dispatch mechanism for the M-V-B gets increasingly
   ;; hairy.
-  (if (integerp n)
-      (let ((dummy-list (make-gensym-list n))
-            (keeper (sb!xc:gensym "KEEPER")))
-        `(multiple-value-bind (,@dummy-list ,keeper) ,form
-           (declare (ignore ,@dummy-list))
-           ,keeper))
+  (let ((val (and (sb!xc:constantp n env) (constant-form-value n env))))
+    (if (and (integerp val) (<= 0 val 10)) ; Arbitrary limit.
+        (let ((dummy-list (make-gensym-list val))
+              (keeper (sb!xc:gensym "KEEPER")))
+          `(multiple-value-bind (,@dummy-list ,keeper) ,form
+             (declare (ignore ,@dummy-list))
+             ,keeper))
       ;; &MORE conversion handily deals with non-constant N,
       ;; avoiding the unstylish practice of inserting FORM into the
       ;; expansion more than once to pick off a few small values.
-      `(multiple-value-call
-           (lambda (n &rest list) (nth (truly-the index n) list))
-         (the index ,n) ,form)))
+      ;; This is not as good as above, because it uses TAIL-CALL-VARIABLE.
+        `(multiple-value-call
+             (lambda (n &rest list) (nth (truly-the index n) list))
+           (the index ,n) ,form))))
 
-(defmacro-mundanely declaim (&rest specs)
-  #!+sb-doc
+(sb!xc:defmacro declaim (&rest specs)
   "DECLAIM Declaration*
   Do a declaration or declarations for the global environment."
   `(eval-when (:compile-toplevel :load-toplevel :execute)
@@ -516,21 +527,19 @@ invoked. In that case it will store into PLACE and start over."
 ;; Avoid unknown return values in emitted code for PRINT-UNREADABLE-OBJECT
 (declaim (ftype (sfunction (t t t t &optional t) null)
                 %print-unreadable-object))
-(defmacro-mundanely print-unreadable-object ((object stream &key type identity)
+(sb!xc:defmacro print-unreadable-object ((object stream &key type identity)
                                              &body body)
-  #!+sb-doc
   "Output OBJECT to STREAM with \"#<\" prefix, \">\" suffix, optionally
   with object-type prefix and object-identity suffix, and executing the
   code in BODY to provide possible further output."
   ;; Note: possibly out-of-order keyword argument evaluation.
-  (if body
-      (let ((fun (make-symbol "THUNK")))
-        `(dx-flet ((,fun () ,@body))
-           (%print-unreadable-object ,object ,stream ,type ,identity #',fun)))
-      `(%print-unreadable-object ,object ,stream ,type ,identity)))
+  (let ((call `(%print-unreadable-object ,object ,stream ,type ,identity)))
+    (if body
+        (let ((fun (make-symbol "THUNK")))
+          `(dx-flet ((,fun () ,@body)) (,@call #',fun)))
+        call)))
 
-(defmacro-mundanely ignore-errors (&rest forms)
-  #!+sb-doc
+(sb!xc:defmacro ignore-errors (&rest forms)
   "Execute FORMS handling ERROR conditions, returning the result of the last
   form, or (VALUES NIL the-ERROR-that-was-caught) if an ERROR was handled."
   `(handler-case (progn ,@forms)
@@ -540,7 +549,8 @@ invoked. In that case it will store into PLACE and start over."
 (defun funarg-bind/call-forms (funarg arg-forms)
   (if (typep funarg
              '(or (cons (eql function) (cons (satisfies legal-fun-name-p) null))
-                  (cons (eql quote) (cons symbol null))))
+                  (cons (eql quote) (cons symbol null))
+                  (cons (eql lambda))))
       (values nil `(funcall ,funarg . ,arg-forms))
     (let ((fn-sym (sb!xc:gensym))) ; for ONCE-ONLY-ish purposes
       (values `((,fn-sym (%coerce-callable-to-fun ,funarg)))
