@@ -308,29 +308,26 @@ variable: an unreadable object representing the error is printed instead.")
                    (output-ugly-object stream object))))
            (handle-it (stream)
              (if *suppress-print-errors*
-                 (handler-bind ((condition
-                                  (lambda (condition) nil
-                                    (when (typep condition *suppress-print-errors*)
-                                      (cond (*in-print-error*
-                                             (write-string "(error printing " stream)
-                                             (write-string *in-print-error* stream)
-                                             (write-string ")" stream))
-                                            (t
-                                             ;; Give outer handlers a chance.
-                                             (with-simple-restart
-                                                 (continue "Suppress the error.")
-                                               (signal condition))
-                                             (let ((*print-readably* nil)
-                                                   (*print-escape* t))
-                                               (write-string
-                                                "#<error printing a " stream)
-                                               (let ((*in-print-error* "type"))
-                                                 (output-object (type-of object) stream))
-                                               (write-string ": " stream)
-                                               (let ((*in-print-error* "condition"))
-                                                 (output-object condition stream))
-                                               (write-string ">" stream))))
-                                      (return-from handle-it object)))))
+                 (handler-bind
+                     ((condition
+                       (lambda (condition)
+                         (when (typep condition *suppress-print-errors*)
+                           (cond (*in-print-error*
+                                  (write-string "(error printing " stream)
+                                  (write-string *in-print-error* stream)
+                                  (write-string ")" stream))
+                                 (t
+                                  (let ((*print-readably* nil)
+                                        (*print-escape* t))
+                                    (write-string
+                                     "#<error printing a " stream)
+                                    (let ((*in-print-error* "type"))
+                                      (output-object (type-of object) stream))
+                                    (write-string ": " stream)
+                                    (let ((*in-print-error* "condition"))
+                                      (output-object condition stream))
+                                    (write-string ">" stream))))
+                           (return-from handle-it object)))))
                    (print-it stream))
                  (print-it stream)))
            (check-it (stream)
@@ -373,7 +370,8 @@ variable: an unreadable object representing the error is printed instead.")
       ;; Additionally, if the object is an obsolete CONDITION, don't crash.
       ;; (There is no update-instance protocol for conditions)
       (when (or (sb!kernel::undefined-classoid-p classoid)
-                (and (layout-invalid layout) (condition-classoid-p classoid)))
+                (and (layout-invalid layout)
+                     (logtest (layout-%flags layout) +condition-layout-flag+)))
         ;; not only is this unreadable, it's unprintable too.
         (return-from output-ugly-object
           (print-unreadable-object (object stream :identity t)
@@ -479,7 +477,7 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;; LETTER-ATTRIBUTE is a local of SYMBOL-QUOTEP. It matches letters
 ;;; that don't need to be escaped (according to READTABLE-CASE.)
-(defparameter *attribute-names*
+(defglobal *attribute-names*
   `((number . number-attribute) (lowercase . lowercase-attribute)
     (uppercase . uppercase-attribute) (letter . letter-attribute)
     (sign . sign-attribute) (extension . extension-attribute)
