@@ -546,28 +546,40 @@ new string COUNT long filled with the fill character."
                       (start)
                       (end)
                       :check-fill-pointer t)
-      (let* ((left-end (if left-p
-                           (do ((index start (1+ index)))
-                               ((or (= index (the fixnum end))
-                                    (not (find (schar string index)
-                                               char-bag
-                                               :test #'char=)))
-                                index)
-                             (declare (fixnum index)))
-                           start))
-             (right-end (if right-p
-                            (do ((index (1- (the fixnum end)) (1- index)))
-                                ((or (< index left-end)
-                                     (not (find (schar string index)
-                                                char-bag
-                                                :test #'char=)))
-                                 (1+ index))
-                              (declare (fixnum index)))
-                            end)))
-        (if (and (eql left-end start)
-                 (eql right-end end))
-            header
-            (subseq (the simple-string string) left-end right-end))))))
+      (flet ((trim-char-p (char)
+               (typecase char-bag
+                 (list
+                  (loop for bag-char in char-bag
+                        thereis (char= bag-char char)))
+                 (simple-string
+                  (loop for bag-char across char-bag
+                        thereis (char= bag-char char)))
+                 (simple-vector
+                  (loop for bag-char across char-bag
+                        thereis (char= bag-char char)))
+                 (vector
+                  (loop for bag-char across char-bag
+                        thereis (char= bag-char char)))
+                 (t
+                  (find char char-bag :test #'char=)))))
+        (let* ((left-end (if left-p
+                             (do ((index start (1+ index)))
+                                 ((or (= index (the fixnum end))
+                                      (not (trim-char-p (schar string index))))
+                                  index)
+                               (declare (fixnum index)))
+                             start))
+               (right-end (if right-p
+                              (do ((index (1- (the fixnum end)) (1- index)))
+                                  ((or (< index left-end)
+                                       (not (trim-char-p (schar string index))))
+                                   (1+ index))
+                                (declare (fixnum index)))
+                              end)))
+          (if (and (eql left-end start)
+                   (eql right-end end))
+              header
+              (subseq (the simple-string string) left-end right-end)))))))
 
 (defun string-left-trim (char-bag string)
   (generic-string-trim char-bag string t nil))
@@ -578,13 +590,13 @@ new string COUNT long filled with the fill character."
 (defun string-trim (char-bag string)
   (generic-string-trim char-bag string t t))
 
-(defun logically-readonlyize (string &optional (always-shareable t))
+(defun logically-readonlyize (vector &optional (always-shareable t))
   ;; "Always" means that regardless of whether the user want
   ;; coalescing of strings used as literals in code compiled to memory,
   ;; the string is shareable.
   ;;  #b01_ ; symbol name, literal compiled to fasl, some other stuff
   ;;  #b10_ ; literal compiled to core
-  (set-header-data (the string string)
+  (set-header-data (the (simple-array * 1) vector)
                    (if always-shareable
-                       +string-shareable+
-                       +string-shareable-nonstd+)))
+                       sb!vm:+vector-shareable+
+                       sb!vm:+vector-shareable-nonstd+)))
