@@ -17,40 +17,11 @@
 
 (declaim (maybe-inline get3 %put getf remprop %putf get-properties keywordp))
 
-;;; Used by [GLOBAL-]SYMBOL-VALUE compiler-macros:
-;;;
-;;; When SYMBOL is constant, check whether it names a deprecated
-;;; variable, potentially signaling a {EARLY,LATE}-DEPRECATION-WARNING
-;;; in the process. Furthermore, if the deprecation state is :FINAL,
-;;; replace FORM by SYMBOL, causing the symbol-macro on SYMBOL to
-;;; expand into a call to DEPRECATION-ERROR.
-;;;
-;;; See SB-IMPL:SETUP-VARIABLE-IN-FINAL-DEPRECATION.
-#-sb-xc-host
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defun maybe-handle-deprecated-global-variable (symbol env)
-    (when (sb!xc:constantp symbol env)
-      (let ((name (constant-form-value symbol env)))
-        (when (symbolp name)
-          (case (deprecated-thing-p 'variable name)
-            ((:early :late)
-             (check-deprecated-thing 'variable name)
-             nil)
-            ;; In this case, there is a symbol-macro for NAME that
-            ;; will signal the FINAL-DEPRECATION-WARNING when
-            ;; ir1converted and the DEPRECATION-ERROR at runtime.
-            (:final
-             name)))))))
-
 (defun symbol-value (symbol)
   "Return SYMBOL's current bound value."
   (declare (optimize (safety 1)))
   (symbol-value symbol))
 
-#-sb-xc-host
-(define-compiler-macro symbol-value (&whole form symbol &environment env)
-  (or (maybe-handle-deprecated-global-variable symbol env) form))
-
 (defun boundp (symbol)
   "Return non-NIL if SYMBOL is bound to a value."
   (boundp symbol))
@@ -71,18 +42,13 @@ distinct from the global value. Can also be SETF."
   (declare (optimize (safety 1)))
   (symbol-global-value symbol))
 
-#-sb-xc-host
-(define-compiler-macro symbol-global-value (&whole form symbol
-                                            &environment env)
-  (or (maybe-handle-deprecated-global-variable symbol env) form))
-
 (defun set-symbol-global-value (symbol new-value)
   (about-to-modify-symbol-value symbol 'set new-value)
   (%set-symbol-global-value symbol new-value))
 
 (declaim (inline %makunbound))
 (defun %makunbound (symbol)
-  (%set-symbol-value symbol (%primitive sb!c:make-unbound-marker)))
+  (%set-symbol-value symbol (make-unbound-marker)))
 
 (defun makunbound (symbol)
   "Make SYMBOL unbound, removing any value it may currently have."
@@ -358,7 +324,7 @@ distinct from the global value. Can also be SETF."
 #!+immobile-space
 (defun %make-symbol (kind name)
   (declare (ignorable kind) (type simple-string name))
-  (set-header-data name +string-shareable+) ; Set "logically read-only" bit
+  (set-header-data name sb!vm:+vector-shareable+) ; Set "logically read-only" bit
   (if #!-immobile-symbols
       (or (eql kind 1) ; keyword
           (and (eql kind 2) ; random interned symbol
