@@ -116,19 +116,19 @@ This is SETFable."
   ;;        The alien is allocated on the heap, and has infinite extent. The alien
   ;;        is allocated at load time, so the same piece of memory is used each time
   ;;        this form executes.
-  (/show "entering WITH-ALIEN" bindings)
+  (/noshow "entering WITH-ALIEN" bindings)
   (let (#!+c-stack-is-control-stack
         bind-alien-stack-pointer)
     (with-auxiliary-alien-types env
       (dolist (binding (reverse bindings))
-        (/show binding)
+        (/noshow binding)
         (destructuring-bind
             (symbol type &optional opt1 (opt2 nil opt2p))
             binding
-          (/show symbol type opt1 opt2)
+          (/noshow symbol type opt1 opt2)
           (let* ((alien-type (parse-alien-type type env))
                  (datap (not (alien-fun-type-p alien-type))))
-            (/show alien-type)
+            (/noshow alien-type)
             (multiple-value-bind (allocation initial-value)
                 (if opt2p
                     (values opt1 opt2)
@@ -139,7 +139,7 @@ This is SETFable."
                        (values opt1 nil))
                       (t
                        (values :local opt1))))
-              (/show allocation initial-value)
+              (/noshow allocation initial-value)
               (setf body
                     (ecase allocation
                       #+nil
@@ -155,14 +155,14 @@ This is SETFable."
                                    `((setq ,symbol ,initial-value)))
                                ,@body)))))
                       (:extern
-                       (/show0 ":EXTERN case")
+                       (/noshow0 ":EXTERN case")
                        `((symbol-macrolet
                              ((,symbol
                                 (%alien-value
                                  (foreign-symbol-sap ,initial-value ,datap) 0 ,alien-type)))
                            ,@body)))
                       (:local
-                       (/show0 ":LOCAL case")
+                       (/noshow0 ":LOCAL case")
                        (let* ((var (sb!xc:gensym "VAR"))
                               (initval (if initial-value (sb!xc:gensym "INITVAL")))
                               (info (make-local-alien-info :type alien-type))
@@ -177,7 +177,7 @@ This is SETFable."
                                     `((let ((,initval ,initial-value))
                                         ,@inner-body))
                                     inner-body)))
-                         (/show var initval info)
+                         (/noshow var initval info)
                          #!+c-stack-is-control-stack
                          (progn (setf bind-alien-stack-pointer t)
                                 `((let ((,var (make-local-alien ',info)))
@@ -195,9 +195,9 @@ This is SETFable."
                                ;; And in non-transformed case it
                                ;; performs finalization.
                                (dispose-local-alien ',info ,var))))))))))))
-      (/show "revised" body)
+      (/noshow "revised" body)
       (verify-local-auxiliaries-okay)
-      (/show0 "back from VERIFY-LOCAL-AUXILIARIES-OK, returning")
+      (/noshow0 "back from VERIFY-LOCAL-AUXILIARIES-OK, returning")
       `(symbol-macrolet ((&auxiliary-type-definitions&
                            ,(append *new-auxiliary-types*
                                     (auxiliary-type-definitions env))))
@@ -283,7 +283,7 @@ Examples:
                    (error
                     "cannot override the size of zero-dimensional arrays"))
                  (when (constantp size)
-                   (setf alien-type (copy-alien-array-type alien-type))
+                   (setf alien-type (copy-structure alien-type))
                    (setf (alien-array-type-dimensions alien-type)
                          (cons (constant-form-value size) (cdr dims)))))
                 (dims
@@ -868,7 +868,8 @@ way that the argument is passed.
 information we need to manipulate callbacks after their creation. Used for
 changing the lisp-side function they point to, invalidation, etc.")
 
-(defstruct callback-info
+(defstruct (callback-info
+            (:copier nil))
   specifier
   function ; NULL if invalid
   wrapper
@@ -897,39 +898,38 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
 (defun %alien-callback-sap (specifier result-type argument-types function wrapper
                             &optional call-type)
   (declare #!-x86 (ignore call-type))
-  (let ((key (list specifier function)))
-    (or (gethash key *alien-callbacks*)
-        (setf (gethash key *alien-callbacks*)
-              (let* ((index (fill-pointer *alien-callback-trampolines*))
-                     ;; Aside from the INDEX this is known at
-                     ;; compile-time, which could be utilized by
-                     ;; having the two-stage assembler tramp &
-                     ;; wrapper mentioned in [1] above: only the
-                     ;; per-function tramp would need assembler at
-                     ;; runtime. Possibly we could even pregenerate
-                     ;; the code and just patch the index in later.
-                     (assembler-wrapper
-                      (alien-callback-assembler-wrapper
-                       index result-type argument-types
-                       #!+x86
-                       (if (eq call-type :stdcall)
-                           (ceiling
-                            (apply #'+
-                                   (mapcar 'alien-type-word-aligned-bits
-                                           argument-types))
-                            8)
-                           0))))
-                (vector-push-extend
-                 (alien-callback-lisp-trampoline wrapper function)
-                 *alien-callback-trampolines*)
-                ;; Assembler-wrapper is static, so sap-taking is safe.
-                (let ((sap (vector-sap assembler-wrapper)))
-                  (push (cons sap (make-callback-info :specifier specifier
-                                                      :function function
-                                                      :wrapper wrapper
-                                                      :index index))
-                        *alien-callback-info*)
-                  sap))))))
+  (ensure-gethash
+   (list specifier function) *alien-callbacks*
+   (let* ((index (fill-pointer *alien-callback-trampolines*))
+          ;; Aside from the INDEX this is known at
+          ;; compile-time, which could be utilized by
+          ;; having the two-stage assembler tramp &
+          ;; wrapper mentioned in [1] above: only the
+          ;; per-function tramp would need assembler at
+          ;; runtime. Possibly we could even pregenerate
+          ;; the code and just patch the index in later.
+          (assembler-wrapper
+           (alien-callback-assembler-wrapper
+            index result-type argument-types
+            #!+x86
+            (if (eq call-type :stdcall)
+                (ceiling
+                 (apply #'+
+                        (mapcar 'alien-type-word-aligned-bits
+                                argument-types))
+                 8)
+                0))))
+     (vector-push-extend
+      (alien-callback-lisp-trampoline wrapper function)
+      *alien-callback-trampolines*)
+     ;; Assembler-wrapper is static, so sap-taking is safe.
+     (let ((sap (vector-sap assembler-wrapper)))
+       (push (cons sap (make-callback-info :specifier specifier
+                                           :function function
+                                           :wrapper wrapper
+                                           :index index))
+             *alien-callback-info*)
+       sap))))
 
 (defun alien-callback-lisp-trampoline (wrapper function)
   (declare (function wrapper) (optimize speed))
@@ -941,6 +941,9 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
          (argument-names arguments)
          (argument-specs (cddr specifier)))
     `(lambda (args-pointer result-pointer function)
+       ;; KLUDGE: the SAP shouldn't be consed but they are, don't
+       ;; bother anyone about that sad fact
+       (declare (muffle-conditions compiler-note))
        ;; FIXME: the saps are not gc safe
        (let ((args-sap (int-sap
                         (sb!kernel:get-lisp-obj-address args-pointer)))
@@ -1052,11 +1055,10 @@ one."
     `(%sap-alien
       (%alien-callback-sap ',specifier ',result-type ',argument-types
                            ,function
-                           (or (gethash ',specifier *alien-callback-wrappers*)
-                               (setf (gethash ',specifier *alien-callback-wrappers*)
-                                     (compile nil
-                                              ',(alien-callback-lisp-wrapper-lambda
-                                                 specifier result-type argument-types env))))
+                           (ensure-gethash
+                            ',specifier *alien-callback-wrappers*
+                            ,(alien-callback-lisp-wrapper-lambda
+                              specifier result-type argument-types env))
                            ,call-type)
       ',(parse-alien-type specifier env)))))
 
