@@ -32,11 +32,11 @@
              (flet ((coerce-it (array)
                       (!coerce-to-specialized array '(unsigned-byte 8)))
                     (file (name type)
-                      (merge-pathnames (make-pathname
-                                        :directory
-                                        '(:relative :up :up "output")
-                                        :name name :type type)
-                                       sb!xc:*compile-file-truename*))
+                      ;; FIXME: this fails on a certain build system,
+                      ;; as does the more obvious :directory '(:relative "output").
+                      ;; Why???
+                      (make-pathname :directory (pathname-directory (merge-pathnames "output/"))
+                                     :name name :type type))
                     (read-ub8-vector (pathname)
                       (with-open-file (stream pathname
                                               :element-type '(unsigned-byte 8))
@@ -195,7 +195,7 @@
                                           (incf index))
                                         (setf (gethash
                                                (apply #'pack-3-codepoints codepoints)
-                                               table) key)))
+                                               table) (logically-readonlyize key))))
                                 table))))
 
                     ,(with-open-file
@@ -284,7 +284,7 @@
   (frob))
 #+sb-xc-host (!character-name-database-cold-init)
 
-(defparameter *base-char-name-alist*
+(defglobal *base-char-name-alist*
   ;; Note: The *** markers here indicate character names which are
   ;; required by the ANSI specification of #'CHAR-NAME. For the others,
   ;; we prefer the ASCII standard name.
