@@ -79,7 +79,8 @@
 
 ;;; Hash table searching maps a logical pathname's host to its
 ;;; physical pathname translation.
-(defvar *logical-hosts* (make-hash-table :test 'equal :synchronized t))
+(define-load-time-global *logical-hosts*
+    (make-hash-table :test 'equal :synchronized t))
 
 ;;;; patterns
 
@@ -878,14 +879,8 @@ a host-structure or string."
                          host
                          (defaults *default-pathname-defaults*)
                          &key (start 0) end junk-allowed)
-  (declare (type pathname-designator thing defaults)
-           (type (or list host string (member :unspecific)) host)
-           (type index start)
-           (type (or index null) end)
-           (type (or t null) junk-allowed)
-           (values (or null pathname) (or null index)))
   (declare (ftype (function * (values (or null pathname) (or null index)))
-                  %parse-native-namestring))
+                  %parse-namestring))
   (with-host (found-host host)
     (let (;; According to ANSI defaults may be any valid pathname designator
           (defaults (etypecase defaults
@@ -898,11 +893,12 @@ a host-structure or string."
                        (truename defaults)))))
       (declare (type pathname defaults))
       (etypecase thing
-        (simple-string
-         (%parse-namestring thing found-host defaults start end junk-allowed))
         (string
-         (%parse-namestring (coerce thing 'simple-string)
-                            found-host defaults start end junk-allowed))
+         (with-array-data ((thing thing) (start start) (end end)
+                           :check-fill-pointer t)
+           (multiple-value-bind (pathname position)
+               (%parse-namestring thing found-host defaults start end junk-allowed)
+             (values pathname (- position start)))))
         (pathname
          (let ((defaulted-host (or found-host (%pathname-host defaults))))
            (declare (type host defaulted-host))
@@ -994,13 +990,14 @@ directory."
                        (truename defaults)))))
       (declare (type pathname defaults))
       (etypecase thing
-        (simple-string
-         (%parse-native-namestring
-          thing found-host defaults start end junk-allowed as-directory))
         (string
-         (%parse-native-namestring (coerce thing 'simple-string)
-                                   found-host defaults start end junk-allowed
-                                   as-directory))
+         (with-array-data ((thing thing) (start start) (end end)
+                           :check-fill-pointer t)
+           (multiple-value-bind (pathname position)
+               (%parse-native-namestring thing
+                                         found-host defaults start end junk-allowed
+                                         as-directory)
+             (values pathname (- position start)))))
         (pathname
          (let ((defaulted-host (or found-host (%pathname-host defaults))))
            (declare (type host defaulted-host))
@@ -1466,7 +1463,7 @@ unspecified elements into a completed to-pathname based on the to-wildname."
 (defun logical-chunkify (namestr start end)
   (collect ((chunks))
     (do ((i start (1+ i))
-         (prev 0))
+         (prev start))
         ((= i end)
          (when (> end prev)
             (chunks (cons (nstring-upcase (subseq namestr prev end)) prev))))
