@@ -339,7 +339,7 @@
                         (sxhash (char-code x)))) ; through DEFTRANSFORM
                ;; general, inefficient case of NUMBER
                (number (sxhash-number x))
-               (generic-function (fsc-instance-hash x))
+               (funcallable-instance (fsc-instance-hash x))
                (t 42))))
     (sxhash-recurse x +max-hash-depthoid+)))
 
@@ -363,6 +363,7 @@
   (typecase key
     (array (array-psxhash key depthoid))
     (hash-table (hash-table-psxhash key))
+    (pathname (sxhash key))
     (structure-object (structure-object-psxhash key depthoid))
     (cons (list-psxhash key depthoid))
     (number (number-psxhash key))
