@@ -464,14 +464,14 @@ HOLDING-MUTEX-P."
 (sb!ext:defglobal **deadlock-lock** nil)
 
 #!+(or (not sb-thread) sb-futex)
-(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
+(defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
   "Waitqueue type."
   (name nil :type (or null thread-name))
   #!+(and sb-thread sb-futex)
   (token nil))
 
 #!+(and sb-thread (not sb-futex))
-(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
+(defstruct (waitqueue (:copier nil) (:constructor make-waitqueue (&key name)))
   "Waitqueue type."
   (name nil :type (or null thread-name))
   ;; For WITH-CAS-LOCK: because CONDITION-WAIT must be able to call
@@ -1014,7 +1014,8 @@ must be held by this thread during this call."
 
 ;;;; Semaphores
 
-(defstruct (semaphore (:constructor make-semaphore
+(defstruct (semaphore (:copier nil)
+                      (:constructor make-semaphore
                           (&key name ((:count %count) 0))))
   "Semaphore type. The fact that a SEMAPHORE is a STRUCTURE-OBJECT
 should be considered an implementation detail, and may change in the
@@ -1179,7 +1180,7 @@ on this semaphore, then N of them is woken up."
 
 ;;;; Job control, independent listeners
 
-(defstruct session
+(defstruct (session (:copier nil))
   (lock (make-mutex :name "session lock"))
   (threads nil)
   (interactive-threads nil)
@@ -1418,6 +1419,11 @@ session."
   ;; accessible to users to secure their own libraries.
   ;;   --njf, 2006-07-15
   ;;
+  ;; FIXME: because #<"main thread"> does not come through here, the global
+  ;; values of these symbols matter. That's bad! It would be nicer for codegen
+  ;; if these symbols (and some others) never required testing for
+  ;; NO-TLS-VALUE-MARKER and potential fallback to loading symbol-global-value.
+  ;;
   ;; As it is, this lambda must not cons until we are ready to run
   ;; GC. Be very careful.
   (let* ((*current-thread* thread)
@@ -1431,7 +1437,8 @@ session."
          (sb!impl::*token-buf-pool* nil)
          (sb!impl::*ignored-package-locks* :invalid)
          (sb!impl::*descriptor-handlers* nil)) ; serve-event
-    ;; Binding from C
+    ;; *ALLOC-SIGNAL* is made thread-local by create_thread_struct()
+    ;; so this assigns into TLS, not the global value.
     (setf sb!vm:*alloc-signal* *default-alloc-signal*)
     (setf (thread-os-thread thread) (current-thread-os-thread))
     (with-mutex ((thread-result-lock thread))
@@ -1788,6 +1795,12 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
 ;;; should probably discuss with a professional psychiatrist first
 #!+sb-thread
 (progn
+
+  (sb!ext:defglobal sb!vm::*free-tls-index* 0)
+  ;; Keep in sync with 'compiler/generic/parms.lisp'
+  #!+ppc ; only PPC uses a separate symbol for the TLS index lock
+  (!defglobal sb!vm::*tls-index-lock* 0)
+
   (defun %thread-sap (thread)
     (let ((thread-sap (alien-sap (extern-alien "all_threads" (* t))))
           (target (thread-os-thread thread)))
