@@ -23,6 +23,46 @@
 
 (in-package "SB-COLD")
 
+(defun parse-make-host-parallelism (str)
+  (multiple-value-bind (value1 end) (parse-integer str :junk-allowed t)
+    (when value1
+      (let ((value2 (if (and value1
+                             (< end (1- (length str))) ; ~ /,[\d]+/
+                             (eql (char str end) #\,))
+                        (parse-integer str :start (1+ end)))))
+        ;; If only 1 integer, assume same parallelism for both passes.
+        (unless value2
+          (setq value2 value1))
+        ;; 0 means no parallelism. 1 means use at most one subjob,
+        ;; just in case you want to test the controlling loop.
+        (when (eql value1 0) (setq value1 nil))
+        (when (eql value2 0) (setq value2 nil))
+        ;; Parallelism on pass 1 works only if LOAD does not compile.
+        ;; Otherwise it's slower than compiling serially.
+        ;; (And this has only been tested with sb-fasteval, not sb-eval.)
+        (cons (and (find-package "SB-INTERPRETER") value1)
+              value2)))))
+
+(defvar *make-host-parallelism*
+  (or #+sbcl
+      (let ((envvar (sb-ext:posix-getenv "SBCL_MAKE_PARALLEL")))
+        (when envvar
+          (require :sb-posix)
+          (parse-make-host-parallelism envvar)))))
+
+(defun make-host-1-parallelism () (car *make-host-parallelism*))
+(defun make-host-2-parallelism () (cdr *make-host-parallelism*))
+
+#+sbcl
+(let ((f (multiple-value-bind (sym access) (find-symbol "OS-EXIT" "SB-SYS")
+           (if (eq access :external) sym 'sb-unix:unix-exit))))
+  (defun exit-process (arg) (funcall f arg)))
+
+;;; If TRUE, then COMPILE-FILE is being invoked only to process
+;;; :COMPILE-TOPLEVEL forms, not to produce an output file.
+;;; This is part of the implementation of parallelized make-host-2.
+(defvar *compile-for-effect-only* nil)
+
 ;;; prefixes for filename stems when cross-compiling. These are quite arbitrary
 ;;; (although of course they shouldn't collide with anything we don't want to
 ;;; write over). In particular, they can be either relative path names (e.g.
@@ -122,11 +162,6 @@
                                       (read-from-file customizer-file-name))
                              #'identity)))
         (funcall customizer default-features)))
-(let ((*print-length* nil)
-      (*print-level* nil))
-  (format t
-          "target features *SHEBANG-FEATURES*=~@<~S~:>~%"
-          *shebang-features*))
 
 (defvar *shebang-backend-subfeatures*
   (let* ((default-subfeatures nil)
@@ -136,16 +171,21 @@
                                   (read-from-file customizer-file-name))
                          #'identity)))
     (funcall customizer default-subfeatures)))
-(let ((*print-length* nil)
-      (*print-level* nil))
-  (format t
-          "target backend-subfeatures *SHEBANG-BACKEND-FEATURES*=~@<~S~:>~%"
-          *shebang-backend-subfeatures*))
 
-(let ((arch (intersection '(:alpha :arm :arm64 :hppa :mips :ppc :sparc :x86 :x86-64)
-                          *shebang-features*)))
-  (cond ((not arch) (error "No architecture selected"))
-        ((> (length arch) 1) (error "More than one architecture selected"))))
+(progn (write-string "target *FEATURES* = ")
+       (write *shebang-features* :pretty nil :length nil)
+       (terpri)
+       (write-string "*shebang-backend-subfeatures* = ")
+       (write *shebang-backend-subfeatures* :pretty nil :length nil)
+       (terpri))
+
+;;; Call for effect of signaling an error if no target picked.
+(target-platform-name)
+
+;;; You can get all the way through make-host-1 without either one of these
+;;; features, but then 'bit-bash' will fail to cross-compile.
+(unless (intersection '(:big-endian :little-endian) *shebang-features*)
+  (warn "You'll have bad time without either endian-ness defined"))
 
 ;;; Some feature combinations simply don't work, and sometimes don't
 ;;; fail until quite a ways into the build.  Pick off the more obvious
@@ -154,7 +194,7 @@
 (let ((feature-compatibility-tests
        '(("(and sb-thread (not gencgc))"
           ":SB-THREAD requires :GENCGC")
-         ("(and sb-thread (not (or ppc x86 x86-64)))"
+         ("(and sb-thread (not (or ppc x86 x86-64 arm64)))"
           ":SB-THREAD not supported on selected architecture")
          ("(and gencgc cheneygc)"
           ":GENCGC and :CHENEYGC are incompatible")
@@ -164,20 +204,28 @@
           ":GENCGC not supported on selected architecture")
          ("(not (or gencgc cheneygc))"
           "One of :GENCGC or :CHENEYGC must be enabled")
-         ("(and win32 (not (and sb-thread
-                                sb-safepoint sb-thruption sb-wtimer
-                                sb-dynamic-core)))"
-          ":SB-WIN32 requires :SB-THREAD and related features")
-         ("(and sb-dynamic-core (not (and linkage-table sb-thread)))"
-          ;; Subtle memory corruption follows when sb-dynamic-core is
-          ;; active, and non-threaded allocation routines have not been
-          ;; updated to take the additional indirection into account.
-          ;; Let's avoid this unusual combination.
-          ":SB-DYNAMIC-CORE requires :LINKAGE-TABLE and :SB-THREAD")
+         ("(and sb-dynamic-core (not linkage-table))"
+          ":SB-DYNAMIC-CORE requires :LINKAGE-TABLE")
+         ("(and relocatable-heap (or cheneygc win32))"
+          "Relocatable heap requires gencgc + not win32")
+         ("(and sb-linkable-runtime (not sb-dynamic-core))"
+          ":SB-LINKABLE-RUNTIME requires :SB-DYNAMIC-CORE")
+         ("(and sb-linkable-runtime (not (or x86 x86-64)))"
+          ":SB-LINKABLE-RUNTIME not supported on selected architecture")
+         ("(and sb-linkable-runtime (not (or darwin linux win32)))"
+          ":SB-LINKABLE-RUNTIME not supported on selected operating system")
          ("(and sb-eval sb-fasteval)"
           ;; It sorta kinda works to have both, but there should be no need,
           ;; and it's not really supported.
           "At most one interpreter can be selected")
+         ("(and immobile-space (not x86-64))"
+          ":IMMOBILE-SPACE is supported only on x86-64")
+        ("(and compact-instance-header (not immobile-space))"
+          ":COMPACT-INSTANCE-HEADER requires :IMMOBILE-SPACE feature")
+        ("(and immobile-code (not immobile-space))"
+          ":IMMOBILE-CODE requires :IMMOBILE-SPACE feature")
+        ("(and immobile-symbols (not immobile-space))"
+          ":IMMOBILE-SYMBOLS requires :IMMOBILE-SPACE feature")
          ;; There is still hope to make multithreading on DragonFly x86-64
          ("(and sb-thread x86 dragonfly)"
           ":SB-THREAD not supported on selected architecture")))
@@ -217,6 +265,11 @@
     ;; SBCL. ("not target code" -- but still presumably host code,
     ;; used to support the cross-compilation process)
     :not-target
+    ;; meaning: This file must always be compiled by 'slam.lisp' even if
+    ;; the object is not out of date with respect to its source.
+    ;; Necessary if there are compile-time-too effects that are not
+    ;; reflected into make-host-2 by load-time actions of make-host-1.
+    :slam-forcibly
     ;; meaning: The #'COMPILE-STEM argument :TRACE-FILE should be T.
     ;; When the compiler is SBCL's COMPILE-FILE or something like it,
     ;; compiling "foo.lisp" will generate "foo.trace" which contains lots
@@ -235,15 +288,16 @@
     ;; never figured out but which were apparently acceptable in CMU
     ;; CL. Eventually, it would be great to just get rid of all
     ;; warnings and remove support for this flag. -- WHN 19990323)
-    :ignore-failure-p))
-
-(defparameter *stems-and-flags* (read-from-file "build-order.lisp-expr"))
+    :ignore-failure-p
+    ;; meaning: Build this file, but don't put it on the list for
+    ;; genesis to include in the cold core.
+    :not-genesis))
 
 (defvar *array-to-specialization* (make-hash-table :test #'eq))
 
 (defmacro do-stems-and-flags ((stem flags) &body body)
   (let ((stem-and-flags (gensym "STEM-AND-FLAGS")))
-    `(dolist (,stem-and-flags *stems-and-flags*)
+    `(dolist (,stem-and-flags (get-stems-and-flags))
        (let ((,stem (first ,stem-and-flags))
              (,flags (rest ,stem-and-flags)))
          ,@body
@@ -256,15 +310,7 @@
     (if position
       (concatenate 'string
                    (subseq stem 0 (1+ position))
-                   #!+x86 "x86"
-                   #!+x86-64 "x86-64"
-                   #!+sparc "sparc"
-                   #!+ppc "ppc"
-                   #!+mips "mips"
-                   #!+alpha "alpha"
-                   #!+hppa "hppa"
-                   #!+arm "arm"
-                   #!+arm64 "arm64"
+                   (target-platform-name)
                    (subseq stem (+ position 7)))
       stem)))
 (compile 'stem-remap-target)
@@ -294,8 +340,13 @@
     (concatenate 'string obj-prefix (stem-remap-target stem) obj-suffix)))
 (compile 'stem-object-path)
 
+(defvar *stems-and-flags* nil)
 ;;; Check for stupid typos in FLAGS list keywords.
-(let ((stems (make-hash-table :test 'equal)))
+(defun get-stems-and-flags ()
+ (when *stems-and-flags*
+   (return-from get-stems-and-flags *stems-and-flags*))
+ (setf *stems-and-flags* (read-from-file "build-order.lisp-expr"))
+ (let ((stems (make-hash-table :test 'equal)))
   (do-stems-and-flags (stem flags)
     ;; We do duplicate stem comparison based on the object path in
     ;; order to cover the case of stems with an :assem flag, which
@@ -315,6 +366,7 @@
       (when set-difference
         (error "found unexpected flag(s) in *STEMS-AND-FLAGS*: ~S"
                set-difference)))))
+  *stems-and-flags*)
 
 ;;;; tools to compile SBCL sources to create the cross-compiler
 
@@ -340,7 +392,11 @@
          ;; -- WHN 19990815
          (src (stem-source-path stem))
          (obj (stem-object-path stem flags mode))
-         (tmp-obj (concatenate 'string obj "-tmp"))
+         ;; Compile-for-effect happens simultaneously with a forked compile,
+         ;; so we need the for-effect output not to stomp on the real output.
+         (tmp-obj
+          (concatenate 'string obj
+                       (if *compile-for-effect-only* "-scratch" "-tmp")))
 
          (compile-file (ecase mode
                          (:host-compile #'compile-file)
@@ -357,7 +413,7 @@
     ;; delete any preexisting object file in order to avoid confusing
     ;; ourselves later should we happen to bail out of compilation
     ;; with an error.
-    (when (probe-file obj)
+    (when (and (not *compile-for-effect-only*) (probe-file obj))
       (delete-file obj))
 
     ;; Original comment:
@@ -398,10 +454,14 @@
       (tagbody
        retry-compile-file
          (multiple-value-bind (output-truename warnings-p failure-p)
-            (if trace-file
-                (funcall compile-file src :output-file tmp-obj
-                         :trace-file t :allow-other-keys t)
-                (funcall compile-file src :output-file tmp-obj))
+             (restart-case
+                 (if trace-file
+                     (funcall compile-file src :output-file tmp-obj
+                                               :trace-file t :allow-other-keys t)
+                     (funcall compile-file src :output-file tmp-obj))
+               (recompile ()
+                 :report report-recompile-restart
+                 (go retry-compile-file)))
            (declare (ignore warnings-p))
            (cond ((not output-truename)
                   (error "couldn't compile ~S" src))
@@ -428,7 +488,10 @@
 
     ;; If we get to here, compilation succeeded, so it's OK to rename
     ;; the temporary output file to the permanent object file.
-    (rename-file-a-la-unix tmp-obj obj)
+    (cond ((not *compile-for-effect-only*)
+           (rename-file-a-la-unix tmp-obj obj))
+          ((probe-file tmp-obj)
+           (delete-file tmp-obj))) ; clean up the trash
 
     ;; nice friendly traditional return value
     (pathname obj)))
@@ -438,12 +501,7 @@
 ;;; cross-compiler's source code in the cross-compilation host.
 (defun in-host-compilation-mode (fn)
   (declare (type function fn))
-  (let ((*features* (cons :sb-xc-host *features*))
-        ;; the CROSS-FLOAT-INFINITY-KLUDGE, as documented in
-        ;; base-target-features.lisp-expr:
-        (*shebang-features* (set-difference *shebang-features*
-                                            '(:sb-propagate-float-type
-                                              :sb-propagate-fun-type))))
+  (let ((*features* (cons :sb-xc-host *features*)))
     (with-additional-nickname ("SB-XC" "SB!XC")
       (funcall fn))))
 (compile 'in-host-compilation-mode)
@@ -452,15 +510,20 @@
 ;;; (if necessary) in the appropriate environment, then loading it
 ;;; into the cross-compilation host Common lisp.
 (defun host-cload-stem (stem flags)
-  (let ((compiled-filename (in-host-compilation-mode
-                            (lambda ()
-                              (compile-stem stem flags :host-compile)))))
-    (load compiled-filename)))
+  (loop
+   (with-simple-restart (recompile "Recompile")
+     (let ((compiled-filename (in-host-compilation-mode
+                               (lambda ()
+                                 (compile-stem stem flags :host-compile)))))
+       (return
+         (load compiled-filename))))))
 (compile 'host-cload-stem)
 
 ;;; like HOST-CLOAD-STEM, except that we don't bother to compile
 (defun host-load-stem (stem flags)
-  (load (stem-object-path stem flags :host-compile)))
+  (loop
+   (with-simple-restart (recompile "Reload")
+     (return (load (stem-object-path stem flags :host-compile))))))
 (compile 'host-load-stem)
 
 ;;;; tools to compile SBCL sources to create object files which will
@@ -483,3 +546,4 @@
            (lambda ()
              (funcall *target-compile-file* filename))))
 (compile 'target-compile-file)
+
