@@ -16,7 +16,7 @@
   ;; Imports from this package into SB-VM
   (import '(conditional-opcode emit-word
             composite-immediate-instruction encodable-immediate
-            lsl lsr asr ror cpsr @) 'sb!vm)
+            lsl lsr asr ror cpsr @) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(sb!vm::nil-value sb!vm::registers sb!vm::null-tn sb!vm::null-offset
             sb!vm::pc-tn sb!vm::pc-offset sb!vm::code-offset)))
@@ -1116,7 +1116,7 @@
                                                                         :down
                                                                         :up)
                                                                     mode))
-                                            (tn-offset base) (tn-offset data)
+                                            pc-offset (tn-offset data)
                                             (ldb (byte 4 4) absolute-delta)
                                             opcode2 absolute-delta)))))
            ((integerp offset)
