@@ -14,9 +14,7 @@
 ;;;; unary operations.
 
 (define-vop (fast-safe-arith-op)
-  (:policy :fast-safe)
-  (:effects)
-  (:affected))
+  (:policy :fast-safe))
 
 (define-vop (fixnum-unop fast-safe-arith-op)
   (:args (x :scs (any-reg)))
@@ -410,8 +408,8 @@
     (inst b END)
     LEFT
     (inst cmp temp n-word-bits)
-    (inst csinv temp temp zr-tn :lt)
-    (inst lsl result number temp)
+    (inst csel result number zr-tn :lt)
+    (inst lsl result result temp)
     END))
 
 (define-vop (fast-ash/signed=>signed fast-ash/signed/unsigned)
@@ -445,16 +443,16 @@
                 (:result-types ,type)
                 (:policy :fast-safe)
                 (:generator ,cost
-                  (let ((amount (cond (cut
-                                       (inst cmp amount n-word-bits)
-                                       ;; Only the first 6 bits count for shifts.
-                                       ;; This sets all bits to 1 if AMOUNT is larger than 63,
-                                       ;; cutting the amount to 63.
-                                       (inst csinv tmp-tn amount zr-tn :lt)
-                                       tmp-tn)
-                                      (t
-                                       amount))))
-                    (inst lsl result number amount))))))
+                  (cond (cut
+                         (inst cmp amount n-word-bits)
+                         (cond ((location= amount result)
+                                (inst csel tmp-tn number zr-tn :lt)
+                                (inst lsl result tmp-tn amount))
+                               (t
+                                (inst csel result number zr-tn :lt)
+                                (inst lsl result result amount))))
+                        (t
+                         (inst lsl result number amount)))))))
   ;; FIXME: There's the opportunity for a sneaky optimization here, I
   ;; think: a FAST-ASH-LEFT-C/FIXNUM=>SIGNED vop.  -- CSR, 2003-09-03
   (def fast-ash-left/fixnum=>fixnum any-reg tagged-num any-reg 2)
@@ -767,8 +765,6 @@
 
 (define-vop (fast-conditional)
   (:conditional :eq)
-  (:effects)
-  (:affected)
   (:policy :fast-safe))
 
 (define-vop (fast-conditional/fixnum fast-conditional)
@@ -1191,26 +1187,4 @@
   (:translate sb!bignum:%ashl)
   (:generator 1
     (inst lsl result digit count)))
-
-;;;; Static functions.
 
-(define-static-fun two-arg-gcd (x y) :translate gcd)
-(define-static-fun two-arg-lcm (x y) :translate lcm)
-
-(define-static-fun two-arg-+ (x y) :translate +)
-(define-static-fun two-arg-- (x y) :translate -)
-(define-static-fun two-arg-* (x y) :translate *)
-(define-static-fun two-arg-/ (x y) :translate /)
-
-(define-static-fun two-arg-< (x y) :translate <)
-(define-static-fun two-arg-> (x y) :translate >)
-(define-static-fun two-arg-= (x y) :translate =)
-
-(define-static-fun two-arg-and (x y) :translate logand)
-(define-static-fun two-arg-ior (x y) :translate logior)
-(define-static-fun two-arg-xor (x y) :translate logxor)
-(define-static-fun two-arg-eqv (x y) :translate logeqv)
-
-(define-static-fun eql (x y) :translate eql)
-
-(define-static-fun %negate (x) :translate %negate)
