@@ -376,15 +376,8 @@
                   (t
                    (bug "Unknown alien type: ~S" type)))))
         ;; arg0 to FUNCALL3 (function)
-        ;;
-        ;; Indirect the access to ENTER-ALIEN-CALLBACK through
-        ;; the symbol-value slot of SB-ALIEN::*ENTER-ALIEN-CALLBACK*
-        ;; to ensure it'll work even if the GC moves ENTER-ALIEN-CALLBACK.
-        ;; Skip any SB-THREAD TLS magic, since we don't expect anyone
-        ;; to rebind the variable. -- JES, 2006-01-01
-        (load-immediate-word r0-tn (+ nil-value (static-symbol-offset
-                                                 'sb!alien::*enter-alien-callback*)))
-        (loadw r0-tn r0-tn symbol-value-slot other-pointer-lowtag)
+        (load-immediate-word r0-tn (static-fdefn-fun-addr 'enter-alien-callback))
+        (loadw r0-tn r0-tn)
         ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
         (inst mov r1-tn (fixnumize index))
         ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
