@@ -90,10 +90,10 @@
 ;;; variables dumped, compute a bit-vector representing the set of
 ;;; live variables. If the TN is environment-live, we only mark it as
 ;;; live when it is in scope at NODE.
+(declaim (ftype (sfunction (local-tn-bit-vector node ir2-block hash-table (or null vop))
+                           simple-bit-vector)
+                compute-live-vars))
 (defun compute-live-vars (live node block var-locs vop)
-  (declare (type ir2-block block) (type local-tn-bit-vector live)
-           (type hash-table var-locs) (type node node)
-           (type (or vop null) vop))
   (let ((res (make-array (logandc2 (+ (hash-table-count var-locs) 7) 7)
                          :element-type 'bit
                          :initial-element 0))
@@ -406,7 +406,8 @@
     (when (and same-name-p
                (not (or more minimal)))
       (setf flags (logior flags compiled-debug-var-same-name-p)))
-    #!+64-bit
+    #!+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
+              ; which early-vm.lisp claims to work
     (cond (indirect
            (setf (ldb (byte 27 8) flags) (tn-sc-offset tn))
            (when save-tn
@@ -698,7 +699,15 @@
     (let* ((sorted (sort dfuns #'< :key #'car))
            (fun-map (compute-debug-fun-map sorted)))
       (make-compiled-debug-info
-       :name (component-name component)
+       ;; COMPONENT-NAME is often not useful, and sometimes completely fubar.
+       ;; Function names, on the other hand, are seldom unhelpful,
+       ;; so if there's only one function, pick that as the component name.
+       ;; Otherwise preserve whatever crummy name was already assigned.
+       :name (let* ((2comp (component-info component))
+                    (entries (sb!c::ir2-component-entries 2comp)))
+               (or (and (not (cdr entries))
+                        (sb!c::entry-info-name (car entries)))
+                   (component-name component)))
        :fun-map fun-map
        :tlf-number component-tlf-num
        :char-offset
