@@ -1054,6 +1054,11 @@
          (aver (null name))
          (dump-fop 'fop-code-object-fixup fasl-output))
         #!+immobile-space
+        (:layout
+         (dump-non-immediate-object (classoid-name (layout-classoid name))
+                                    fasl-output)
+         (dump-fop 'fop-layout-fixup fasl-output))
+        #!+immobile-space
         (:immobile-object
          (dump-non-immediate-object (the symbol name) fasl-output)
          (dump-fop 'fop-immobile-obj-fixup fasl-output))
@@ -1287,6 +1292,9 @@
   (when (layout-invalid obj)
     (compiler-error "attempt to dump reference to obsolete class: ~S"
                     (layout-classoid obj)))
+  ;; STANDARD-OBJECT could in theory be dumpable, but nothing else,
+  ;; because all its subclasses can evolve to have new layouts.
+  (aver (not (logtest (layout-%flags obj) +pcl-object-layout-flag+)))
   (let ((name (classoid-name (layout-classoid obj))))
     ;; Q: Shouldn't we aver that NAME is the proper name for its classoid?
     (unless name
