@@ -108,6 +108,7 @@
 (defvar *eval-tlf-index*)
 (defvar *dynamic-counts-tn*)
 (defvar *elsewhere*)
+(defvar *elsewhere-label*)
 (defvar *event-info*)
 (defvar *event-note-threshold*)
 (defvar *failure-p*)
@@ -229,12 +230,13 @@ the stack without triggering overflow protection.")
     (style-warn 'asterisks-around-lexical-variable-name
                 :format-control
                 "using the lexical binding of the symbol ~
-                 ~/sb-impl::print-symbol-with-prefix/, not the~@
+                 ~/sb-ext:print-symbol-with-prefix/, not the~@
                  dynamic binding"
                 :format-arguments (list symbol)))
   (values))
 
-(def!struct (debug-name-marker (:print-function print-debug-name-marker)))
+(def!struct (debug-name-marker (:print-function print-debug-name-marker)
+                               (:copier nil)))
 
 (defvar *debug-name-level* 4)
 (defvar *debug-name-length* 12)
@@ -309,7 +311,7 @@ the stack without triggering overflow protection.")
 (in-package "SB!ALIEN")
 
 ;;; Information describing a heap-allocated alien.
-(def!struct (heap-alien-info)
+(def!struct (heap-alien-info (:copier nil))
   ;; The type of this alien.
   (type (missing-arg) :type alien-type)
   ;; Its name.
