@@ -12,7 +12,14 @@
 
 (in-package "SB!VM")
 
+;;; When building the cross-compiler (and called by the host), read the
+;;; dynamic-space-size file.
+;;; When called by the cross-compiler (in the host), use the previously chosen value.
+;;; The target function is never called, but if omitted via #-sb-xc-host,
+;;; compilation of !GENCGC-SPACE-SETUP would issue an "undefined" warning.
 (defun !read-dynamic-space-size ()
+  (unless (member :sb-xc-host *features*)
+    (return-from !read-dynamic-space-size (symbol-value 'default-dynamic-space-size)))
   (with-open-file (f "output/dynamic-space-size.txt")
     (let ((line (read-line f)))
       (multiple-value-bind (number end)
@@ -58,7 +65,9 @@
     (small-spaces-start
           &key ((:dynamic-space-start dynamic-space-start*))
                ((:default-dynamic-space-size default-dynamic-space-size*))
-               #!+immobile-space (immobile-space-size (* 128 1024 1024))
+               #!+immobile-space
+               ((:immobile-space-size immobile-space-size*) (* 128 1024 1024))
+               #!+immobile-space (immobile-code-space-size (* 104 1024 1024))
                ;; Smallest os_validate()able alignment; used as safepoint
                ;; page size.  Default suitable for POSIX platforms.
                (alignment            #x1000)
@@ -66,20 +75,34 @@
                (small-space-spread #x100000)
                ;; traditional margin between spaces
                (margin-size          #x1000))
-  (let* ((spaces '(read-only static
+  (declare (ignorable dynamic-space-start*)) ; might be unused in make-host-2
+  (flet ((defconstantish (relocatable symbol value)
+           (if (not relocatable) ; easy case
+               `(defconstant ,symbol ,value)
+               ;; Genesis needs to know the gspace start, but it's not constant.
+               ;; This value will not be exposed to C code.
+               #+sb-xc-host `(defparameter ,symbol ,value)
+               ;; Ideally the #-sb-xc-host code be a DEFINE-ALIEN-VARIABLE,
+               ;; but can't be due to dependency order problem.
+               )))
+    (let*
+        ((spaces '(read-only static
                    #!+linkage-table linkage-table
                    #!+immobile-space immobile))
          (ptr small-spaces-start)
          safepoint-address
          (small-space-forms
           (loop for (space next-space) on spaces appending
-                (let* ((next-start
+                (let* ((relocatable
+                        ;; TODO: immobile and linkage-table should be relocatable
+                        #!+relocatable-heap (member space '(immobile)))
+                       (next-start
                         (+ ptr (cond #!+immobile-space
                                      ((eq space 'immobile)
                                       ;; We subtract margin-size when
                                       ;; computing FOO-SPACE-END,
                                       ;; so add it in here to compensate.
-                                      (+ immobile-space-size margin-size))
+                                      (+ immobile-space-size* margin-size))
                                      (t
                                       small-space-spread))))
                        (end next-start))
@@ -87,12 +110,16 @@
                     ;; margin becomes safepoint page; substract margin again.
                     (decf end alignment)
                     (setf safepoint-address end))
-                  (prog1
-                      `((defconstant ,(symbolicate space "-SPACE-START")
-                            ,ptr)
-                        (defconstant ,(symbolicate space "-SPACE-END")
-                            ,(- end margin-size)))
-                    (setf ptr next-start)))))
+                  (let* ((start-sym (symbolicate space "-SPACE-START"))
+                         (start-val ptr)
+                         (end-val (- end margin-size)))
+                    (setf ptr next-start)
+                    `(,(defconstantish relocatable start-sym start-val)
+                      ,(if relocatable
+                           `(defconstant ,(symbolicate space "-SPACE-SIZE")
+                              ,(- end-val start-val))
+                           `(defconstant ,(symbolicate space "-SPACE-END")
+                              ,end-val)))))))
          (safepoint-page-forms
           (list #!+sb-safepoint
                 `(defconstant gc-safepoint-page-addr ,safepoint-address)))
@@ -102,14 +129,16 @@
        ,@safepoint-page-forms
        ,@small-space-forms
        #!+immobile-space
-       (defconstant immobile-fixedobj-subspace-size (* 24 1024 1024))
-       (defconstant dynamic-space-start ,(or dynamic-space-start* ptr))
+       (defconstant immobile-fixedobj-subspace-size
+         ,(- immobile-space-size* immobile-code-space-size))
+       ,(defconstantish (or #!+relocatable-heap t) 'dynamic-space-start
+          (or dynamic-space-start* ptr))
        (defconstant default-dynamic-space-size
          (or ,(!read-dynamic-space-size)
              ,default-dynamic-space-size*
              (ecase n-word-bits
                (32 (expt 2 29))
-               (64 (expt 2 30))))))))
+               (64 (expt 2 30)))))))))
 
 (defconstant-eqx +c-callable-fdefns+
   '(sub-gc
@@ -134,58 +163,68 @@
     sb!unix::signal-handler-callback)
   #'equal)
 
+;;; Static symbols that C code must be able to assign to,
+;;; as contrasted with static for other reasons such as:
+;;;  - garbage collections roots (namely NIL)
+;;;  - other symbols that Lisp codegen must hardwire (T)
+;;;  - static for efficiency of access but need not be
+(defconstant-eqx !per-thread-c-interface-symbols
+  `((*free-interrupt-context-index* 0)
+    (sb!sys:*allow-with-interrupts* t)
+    (sb!sys:*interrupts-enabled* t)
+    *alloc-signal*
+    sb!sys:*interrupt-pending*
+    #!+sb-thruption sb!sys:*thruption-pending*
+    #!+sb-thruption sb!kernel:*restart-clusters*
+    *in-without-gcing*
+    *gc-inhibit*
+    *gc-pending*
+    #!+sb-safepoint sb!impl::*gc-safe*
+    #!+sb-safepoint sb!impl::*in-safepoint*
+    #!+sb-thread *stop-for-gc-pending*
+    ;; non-x86oid gencgc object pinning
+    #!+(and gencgc (not (or x86 x86-64))) *pinned-objects*
+    ;; things needed for non-local-exit
+    (*current-catch-block* 0)
+    (*current-unwind-protect-block* 0)
+    )
+  #'equal)
+
 (defconstant-eqx +common-static-symbols+
   `(t
-    ;; free pointers.  Note that these are FIXNUM word counts, not (as
-    ;; one might expect) byte counts or SAPs. The reason seems to be
-    ;; that by representing them this way, we can avoid consing
-    ;; bignums.  -- WHN 2000-10-02
-    *read-only-space-free-pointer*
-    *static-space-free-pointer*
-    *immobile-fixedobj-free-pointer*
-    *immobile-space-free-pointer*
-    *immobile-freelist*
+    ;; These symbols are accessed from C only through TLS,
+    ;; never the symbol-value slot
+    #!-sb-thread ,@(mapcar (lambda (x) (car (ensure-list x)))
+                           !per-thread-c-interface-symbols)
+    ;; NLX variables are thread slots on x86-64.  A static sym is needed
+    ;; for arm64, ppc, and x86 because we haven't implemented TLS index fixups,
+    ;; so must lookup the TLS index given the symbol.
+    #!+(and sb-thread (not x86-64)) ,@'(*current-catch-block*
+                                        *current-unwind-protect-block*)
 
-    ;; things needed for non-local-exit
-    *current-catch-block*
-    *current-unwind-protect-block*
+    ;; sb-safepoint in addition to accessing this symbol via TLS,
+    ;; uses the symbol itself as a value. Kinda weird.
+    #!+sb-safepoint *in-without-gcing*
+
+    #!+immobile-space *immobile-freelist* ; not per-thread (yet...)
 
     #!+hpux *c-lra*
 
     ;; stack pointers
-    *binding-stack-start*
-    *control-stack-start*
-    *control-stack-end*
+    #!-sb-thread *binding-stack-start* ; a thread slot if #!+sb-thread
+    #!-sb-thread *control-stack-start* ; ditto
+    #!-sb-thread *control-stack-end*   ; ditto
 
-    ;; interrupt handling
-    *alloc-signal*
-    *free-interrupt-context-index*
-    sb!unix::*allow-with-interrupts*
-    sb!unix::*interrupts-enabled*
-    sb!unix::*interrupt-pending*
-    #!+sb-thruption sb!unix::*thruption-pending*
-    #!+sb-thruption sb!impl::*restart-clusters*
-    *in-without-gcing*
-    *gc-inhibit*
-    *gc-pending*
-    #!-sb-thread
-    *stepping*
-    #!+sb-safepoint sb!impl::*gc-safe*
-    #!+sb-safepoint sb!impl::*in-safepoint*
+    #!-sb-thread *stepping*
 
     ;; threading support
-    #!+sb-thread *stop-for-gc-pending*
     #!+sb-thread *free-tls-index*
     ;; Keep in sync with 'compiler/early-backend.lisp':
     ;;  "only PPC uses a separate symbol for the TLS index lock"
     #!+(and sb-thread ppc) *tls-index-lock*
 
     ;; dynamic runtime linking support
-    #!+sb-dynamic-core +required-runtime-c-symbols+
-
-    ;; non-x86oid gencgc object pinning
-    #!+(and gencgc (not (or x86 x86-64)))
-    *pinned-objects*
+    #!+sb-dynamic-core +required-foreign-symbols+
 
     ;; for looking up assembler routine by name
     ;; and patching them on runtime startup
