@@ -122,8 +122,6 @@
 (defknown vector-sap ((simple-unboxed-array (*))) system-area-pointer
   (flushable))
 
-(defknown lowtag-of (t) (unsigned-byte #.sb!vm:n-lowtag-bits)
-  (flushable movable))
 ;;; WIDETAG-OF needs extra code to handle LIST and FUNCTION lowtags.
 ;;; When dealing with known other-pointers (dispatching on array
 ;;; element type for example), %OTHER-POINTER-WIDETAG is faster.
@@ -310,7 +308,11 @@
 (defknown fun-code-header (t) t (movable flushable))
 (defknown %make-lisp-obj (sb!vm:word) t (movable flushable))
 (defknown get-lisp-obj-address (t) sb!vm:word (movable flushable))
-(defknown fun-word-offset (function) index (movable flushable))
+(defknown fun-word-offset (function)
+    (unsigned-byte #.(- sb!vm:n-word-bits sb!vm:n-widetag-bits
+                        ;; Exclude the layout
+                        #!+(and 64-bit immobile-space) 32))
+    (movable flushable))
 
 ;;;; 32-bit logical operations
 
