@@ -99,7 +99,7 @@
                          `(ecase raw-type
                             ((t)
                              (vop init-slot node block object arg-tn
-                                  name (+ sb!vm:instance-slots-offset slot) lowtag))
+                                  name dx-p (+ sb!vm:instance-slots-offset slot) lowtag))
                             ,@(map 'list
                                (lambda (rsd)
                                  `(,(sb!kernel::raw-slot-data-raw-type rsd)
@@ -111,7 +111,7 @@
            (:dd
             (vop init-slot node block object
                  (emit-constant (sb!kernel::dd-layout-or-lose slot))
-                 name
+                 name dx-p
                  ;; Layout has no index if compact headers.
                  (or #!+compact-instance-header :layout sb!vm:instance-slots-offset)
                  lowtag))
@@ -148,7 +148,7 @@
                                              (sc-number-or-lose 'sb!vm::any-reg))))
                                     (vop make-funcallable-instance-tramp node block tn)
                                     tn)))))
-                     name slot lowtag))))))))
+                     name dx-p slot lowtag))))))))
   (unless (null args)
     (bug "Leftover args: ~S" args)))
 
@@ -191,8 +191,12 @@
     (let* ((c-dd (lvar-value dd))
            (c-slot-specs (lvar-value slot-specs))
            (words (+ (dd-length c-dd) words)))
+      #!+compact-instance-header
+      (progn (aver (= type sb!vm:instance-widetag))
+             (emit-constant (setq type (sb!kernel::dd-layout-or-lose c-dd))))
       (emit-fixed-alloc node block name words type lowtag result lvar)
-      (emit-inits node block name result lowtag `((:dd . ,c-dd) ,@c-slot-specs) args)
+      (emit-inits node block name result lowtag
+                  `(#!-compact-instance-header (:dd . ,c-dd) ,@c-slot-specs) args)
       (move-lvar-result node block locs lvar))))
 
 (defoptimizer (initialize-vector ir2-convert)
@@ -314,10 +318,9 @@
          ;; a vector object should fit in one page -- otherwise it might go past
          ;; stack guard pages.
          (values-subtypep (lvar-derived-type words)
-                          (load-time-value
-                           (specifier-type `(integer 0 ,(- (/ sb!vm::*backend-page-bytes*
-                                                              sb!vm:n-word-bytes)
-                                                           sb!vm:vector-data-offset))))))))
+                          (specifier-type
+                           `(integer 0 ,(- (/ +backend-page-bytes+ sb!vm:n-word-bytes)
+                                           sb!vm:vector-data-offset)))))))
 
   (defoptimizer (allocate-vector ltn-annotate) ((type length words) call ltn-policy)
     (declare (ignore type length words))
@@ -381,9 +384,8 @@
         ;; could be removed, because allocation would never miss the guard page
         ;; if it tries to consume too much stack space.
         (values-subtypep (lvar-derived-type length)
-                         (load-time-value
-                          (specifier-type `(integer 0 ,(/ sb!vm::*backend-page-bytes*
-                                                          sb!vm:n-word-bytes 2)))))))
+                         (specifier-type
+                          `(integer 0 ,(/ +backend-page-bytes+ sb!vm:n-word-bytes 2))))))
   (defoptimizer (%make-list ltn-annotate) ((length element) call ltn-policy)
     (declare (ignore length element))
     (vectorish-ltn-annotate-helper call ltn-policy
