@@ -32,7 +32,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB!IMPL")
 
 #-no-ansi-print-object
 (defmethod print-object ((x meta-info) stream)
@@ -40,41 +40,6 @@
     (format stream "~S ~S, ~D" (meta-info-category x) (meta-info-kind x)
             (meta-info-number x))))
 
-(!begin-collecting-cold-init-forms)
-#!+sb-show (!cold-init-forms (/show0 "early in globaldb.lisp cold init"))
-
-;;; This is sorta semantically equivalent to SXHASH, but better-behaved for
-;;; legal function names. It performs more work by not cutting off as soon
-;;; in the CDR direction, thereby improving the distribution of method names.
-;;; More work here equates to less work in the global hashtable.
-;;; To wit: (eq (sxhash '(foo a b c bar)) (sxhash '(foo a b c d))) => T
-;;; but the corresponding globaldb-sxhashoids differ.
-;;; This is no longer inline because for the cases where it is needed -
-;;; names which are not just symbols or (SETF F) - an extra call has no impact.
-(defun globaldb-sxhashoid (name)
-  ;; we can't use MIX because it's in 'target-sxhash',
-  ;; so use the host's sxhash, but ensure that the result is a target fixnum.
-  #+sb-xc-host (logand (sxhash name) sb!xc:most-positive-fixnum)
-  #-sb-xc-host
-  (locally
-      (declare (optimize (safety 0))) ; after the argc check
-    ;; TRAVERSE will walk across more cons cells than RECURSE will descend.
-    ;; That's why this isn't just one self-recursive function.
-    (labels ((traverse (accumulator x length-limit)
-               (declare (fixnum length-limit))
-               (cond ((atom x) (sb!int:mix (sxhash x) accumulator))
-                     ((zerop length-limit) accumulator)
-                     (t (traverse (sb!int:mix (recurse (car x) 4) accumulator)
-                                  (cdr x) (1- length-limit)))))
-             (recurse (x depthoid) ; depthoid = a blend of level and length
-               (declare (fixnum depthoid))
-               (cond ((atom x) (sxhash x))
-                     ((zerop depthoid)
-                      #.(logand sb!xc:most-positive-fixnum #36Rglobaldbsxhashoid))
-                     (t (sb!int:mix (recurse (car x) (1- depthoid))
-                                    (recurse (cdr x) (1- depthoid)))))))
-      (traverse 0 name 10))))
-
 ;;; Given any non-negative integer, return a prime number >= to it.
 ;;;
 ;;; FIXME: This logic should be shared with ALMOST-PRIMIFY in
@@ -101,7 +66,6 @@
 ;;; sources partway through bootstrapping, tch tch, overwriting its
 ;;; version with our version would be unlikely to help, because that
 ;;; would make the cross-compiler very confused.)
-(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 (defun !register-meta-info (metainfo)
   (let* ((name (meta-info-kind metainfo))
          (list (!get-meta-infos name)))
@@ -123,8 +87,6 @@
            (!make-meta-info id category kind type-spec type-checker
                             validate-function default)))))
 
-) ; EVAL-WHEN
-
 #-sb-xc
 (setf (get '!%define-info-type :sb-cold-funcall-handler/for-effect)
       (lambda (category kind type-spec checker validator default id)
@@ -142,12 +104,6 @@
             :category category :kind kind :type-spec type-spec
             :type-checker checker :validate-function validator
             :default default :number id)))))
-
-(!cold-init-forms
- (dovector (x (the simple-vector *info-types*))
-   ;; Genesis writes the *INFO-TYPES* array, but setting up the mapping
-   ;; from keyword-pair to object is deferred until cold-init.
-   (when x (!register-meta-info x))))
 
 ;;;; info types, and type numbers, part II: what's
 ;;;; needed only at compile time, not at run time
@@ -181,30 +137,16 @@
   ;; There was formerly a remark that (COPY-TREE TYPE-SPEC) ensures repeatable
   ;; fasls. That's not true now, probably never was. A compiler is permitted to
   ;; coalesce EQUAL quoted lists and there's no defense against it, so why try?
-  (let ((form
-         `(!%define-info-type
+  `(!%define-info-type
            ,category ,kind ',type-spec
-           ,(cond ((eq type-spec 't) '#'identity)
-                  ;; evil KLUDGE to avoid "undefined type" warnings
-                  ;; when building the cross-compiler.
-                  #+sb-xc-host
-                  ((member type-spec
-                           '((or fdefn null)
-                             (or alien-type null) (or heap-alien-info null))
-                           :test 'equal)
-                   `(lambda (x)
-                      (declare (notinline typep))
-                      (if (typep x ',type-spec)
-                          x
-                          (error "~S is not a ~S" x ',type-spec))))
-                  (t
-                   `(named-lambda "check-type" (x) (the ,type-spec x))))
+           ,(if (eq type-spec 't)
+                '#'identity
+                `(named-lambda "check-type" (x) (the ,type-spec x)))
            ,validate-function ,default
            ;; Rationale for hardcoding here is explained at INFO-VECTOR-FDEFN.
            ,(or (and (eq category :function) (eq kind :definition)
                      +fdefn-info-num+)
                 #+sb-xc (meta-info-number (meta-info category kind))))))
-    `(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute) ,form))))
 
 
 (macrolet ((meta-info-or-lose (category kind)
@@ -269,9 +211,12 @@
 
 ;;;; *INFO-ENVIRONMENT*
 
-(!cold-init-forms
-  (setq *info-environment* (make-info-hashtable))
-  (/show0 "done setting *INFO-ENVIRONMENT*"))
+(defun !globaldb-cold-init ()
+  ;; Genesis writes the *INFO-TYPES* array, but setting up the mapping
+  ;; from keyword-pair to object is deferred until cold-init.
+  (dovector (x (the simple-vector *info-types*))
+    (when x (!register-meta-info x)))
+  (setq *info-environment* (make-info-hashtable)))
 
 ;;;; GET-INFO-VALUE
 
@@ -315,25 +260,6 @@
         (funcall (truly-the function (cdr hook)) name info-number answer nil))
       (values answer nil))))
 
-;; interface to %ATOMIC-SET-INFO-VALUE
-;; GET-INFO-VALUE-INITIALIZING is a restricted case of this,
-;; and perhaps could be implemented as such.
-;; Atomic update will be important for making the fasloader threadsafe
-;; using a predominantly lock-free design, and other nice things.
-(def!macro atomic-set-info-value (category kind name lambda)
-  (with-unique-names (info-number proc)
-    `(let ((,info-number
-            ,(if (and (keywordp category) (keywordp kind))
-                 (meta-info-number (meta-info category kind))
-                 `(meta-info-number (meta-info ,category ,kind)))))
-       ,(if (and (listp lambda) (eq (car lambda) 'lambda))
-            ;; rewrite as FLET because the compiler is unable to dxify
-            ;;   (DX-LET ((x (LAMBDA <whatever>))) (F x))
-            (destructuring-bind (lambda-list . body) (cdr lambda)
-              `(dx-flet ((,proc ,lambda-list ,@body))
-                 (%atomic-set-info-value ,name ,info-number #',proc)))
-            `(%atomic-set-info-value ,name ,info-number ,lambda)))))
-
 ;; Call FUNCTION once for each Name in globaldb that has information associated
 ;; with it, passing the function the Name as its only argument.
 ;;
@@ -380,21 +306,6 @@
 (declaim (ftype (sfunction (t) ctype)
                 specifier-type ctype-of sb!kernel::ctype-of-array))
 
-;;; The type specifier for this function.
-(define-info-type (:function :type)
-  :type-spec ctype
-  ;; Again [as in (DEFINE-INFO-TYPE (:FUNCTION :TYPE) ...)] it's
-  ;; not clear how to generalize the FBOUNDP expression to the
-  ;; cross-compiler. -- WHN 19990330
-  :default
-  ;; Delay evaluation of (SPECIFIER-TYPE) since it can't work yet
-  #+sb-xc-host (lambda (x) (declare (ignore x)) (specifier-type 'function))
-  #-sb-xc-host (lambda (name)
-                 (if (fboundp name)
-                     (handler-bind ((style-warning #'muffle-warning))
-                       (specifier-type (sb!impl::%fun-type (fdefinition name))))
-                     (specifier-type 'function))))
-
 ;;; the ASSUMED-TYPE for this function, if we have to infer the type
 ;;; due to not having a declaration or definition
 (define-info-type (:function :assumed-type)
@@ -434,7 +345,7 @@
 
 ;;; This specifies whether this function may be expanded inline. If
 ;;; null, we don't care.
-(define-info-type (:function :inlinep) :type-spec inlinep)
+(define-info-type (:function :inlinep) :type-spec sb!c::inlinep)
 
 ;;; Track how many times IR2 converted a call to this function as a full call
 ;;; that was not in the scope of a local or global notinline declaration.
@@ -447,6 +358,13 @@
 ;;; expansion is inhibited.
 ;;; As an exception, a cons of two atoms represents structure metadata
 ;;; which is recognized and transformed in a stylized way.
+;;;
+;;; This item is almost mutually exclusive with an inline expansion,
+;;; but both are possible in the rare case of a system-defined transform
+;;; that may decline to expand. If it does, an inline expansion could win.
+;;; We don't actually have anything like that any more though.
+;;; For user-defined functions, the invariant is maintained that at most
+;;; one of :source-transform and an inline-expansion exist.
 (define-info-type (:function :source-transform)
   :type-spec (or function null (cons atom atom)))
 
@@ -462,7 +380,7 @@
 
 ;;; If a function is "known" to the compiler, then this is a FUN-INFO
 ;;; structure containing the info used to special-case compilation.
-(define-info-type (:function :info) :type-spec (or fun-info null))
+(define-info-type (:function :info) :type-spec (or sb!c::fun-info null))
 
 ;;; This is a type specifier <t> such that if an argument X to the function
 ;;; does not satisfy (TYPEP x <t>) then the function definitely returns NIL.
@@ -492,8 +410,10 @@
 ;;; the declared type for this variable
 (define-info-type (:variable :type)
   :type-spec ctype
-  ;; This gets set to *UNIVERSAL-TYPE* in 'late-type'
-  :default (lambda (x) (declare (ignore x)) (error "Too early for INFO")))
+  :default #+sb-xc-host (lambda (x)
+                          (declare (special *universal-type*) (ignore x))
+                          *universal-type*)
+           #-sb-xc-host *universal-type*)
 
 ;;; where this type and kind information came from
 (define-info-type (:variable :where-from)
@@ -503,7 +423,7 @@
 (define-info-type (:variable :macro-expansion) :type-spec t)
 
 (define-info-type (:variable :alien-info)
-  :type-spec (or heap-alien-info null))
+  :type-spec (or null sb!alien-internals:heap-alien-info))
 
 (define-info-type (:variable :documentation) :type-spec (or string null))
 
@@ -568,71 +488,20 @@
 
 (define-info-type (:type :documentation) :type-spec (or string null))
 
-;;; A function that parses type specifiers into CTYPE structures.
-(define-info-type (:type :translator)
-  :type-spec (or function null)
-  ;; This error is never seen by a user. After meta-compile there is no
-  ;; means to define additional types with custom translators.
-  :validate-function (lambda (name new-value)
-                       ;; The compiler-macro signals an error
-                       ;; on forward-referenced info-types.
-                       #+sb-xc-host (declare (notinline info))
-                       (when (and new-value (info :type :expander name))
-                         (bug "Type has an expander"))))
+;;; The expander function for a defined type,
+;;; or a cons whose CAR is a function which is a builtin type translator.
+(define-info-type (:type :expander) :type-spec (or function list))
 
-;;; The expander function for a defined type.
-;;; It returns a type expression, not a CTYPE.
-(define-info-type (:type :expander)
-  :type-spec (or function null)
-  ;; This error is never seen by a user.
-  ;; The user sees "illegal to redefine standard type".
-  :validate-function (lambda (name new-value)
-                       (when (and new-value (info :type :translator name))
-                         (bug "Type has a translator"))))
-
-;;; If true, then the type coresponding to this name. Note that if
+;;; If non-nil, then the type coresponding to this name. Note that if
 ;;; this is a built-in class with a translation, then this is the
 ;;; translation, not the class object. This info type keeps track of
 ;;; various atomic types (NIL etc.) and also serves as a means to
 ;;; ensure that common standard types are only consed once.
-(define-info-type (:type :builtin)
-  :type-spec (or ctype null)
-  :validate-function (lambda (name new-value)
-                       (when (and (ctype-p new-value)
-                                  (ctype-p (info :type :translator name)))
-                         (bug ":BUILTIN and :TRANSLATOR are incompatible"))))
+(define-info-type (:type :builtin) :type-spec (or ctype null))
 
 ;;; The classoid-cell for this type
 (define-info-type (:type :classoid-cell) :type-spec t)
 
-(defun find-classoid-cell (name &key create)
-  (let ((real-name (uncross name)))
-    (cond ((info :type :classoid-cell real-name))
-          (create
-           (get-info-value-initializing
-            :type :classoid-cell real-name
-            (sb!kernel::make-classoid-cell real-name))))))
-
-;;; Return the classoid with the specified NAME. If ERRORP is false,
-;;; then NIL is returned when no such class exists.
-(defun find-classoid (name &optional (errorp t))
-  (declare (type symbol name))
-  (let ((cell (find-classoid-cell name)))
-    (cond ((and cell (classoid-cell-classoid cell)))
-          (errorp
-           (error 'simple-type-error
-                  :datum nil
-                  :expected-type 'class
-                  :format-control "Class not yet defined: ~S"
-                  :format-arguments (list name))))))
-
-;;; layout for this type being used by the compiler
-(define-info-type (:type :compiler-layout)
-  :type-spec (or layout null)
-  :default (lambda (name)
-             (let ((class (find-classoid name nil)))
-               (when class (classoid-layout class)))))
-
 ;;; DEFTYPE lambda-list
 ;; FIXME: remove this after making swank-fancy-inspector not use it.
 (define-info-type (:type :lambda-list) :type-spec t)
@@ -677,16 +546,19 @@
   :type-spec (member :primitive :defined :unknown)
   :default :unknown)
 (define-info-type (:alien-type :translator) :type-spec (or function null))
-(define-info-type (:alien-type :definition) :type-spec (or alien-type null))
-(define-info-type (:alien-type :struct) :type-spec (or alien-type null))
-(define-info-type (:alien-type :union) :type-spec (or alien-type null))
-(define-info-type (:alien-type :enum) :type-spec (or alien-type null))
+(define-info-type (:alien-type :definition)
+  :type-spec (or null sb!alien-internals:alien-type))
+(define-info-type (:alien-type :struct)
+  :type-spec (or null sb!alien-internals:alien-type))
+(define-info-type (:alien-type :union)
+  :type-spec (or null sb!alien-internals:alien-type))
+(define-info-type (:alien-type :enum)
+  :type-spec (or null sb!alien-internals:alien-type))
 
 ;;;; ":SETF" subsection - Data pertaining to expansion of the omnipotent macro.
-(define-info-type (:setf :inverse) :type-spec (or symbol null))
 (define-info-type (:setf :documentation) :type-spec (or string null))
 (define-info-type (:setf :expander)
-    :type-spec (or function (cons integer function) null))
+    :type-spec (or symbol function (cons integer function) null))
 
 ;;;; ":CAS" subsection - Like SETF but there are no "inverses", just expanders
 (define-info-type (:cas :expander) :type-spec (or function null))
@@ -732,5 +604,3 @@
                      (list (meta-info-category type) (meta-info-kind type))))
          (write val :level 1)))
      sym)))
-
-(!defun-from-collected-cold-init-forms !globaldb-cold-init)
