@@ -14,9 +14,7 @@
 ;;;; Unary operations.
 
 (define-vop (fast-safe-arith-op)
-  (:policy :fast-safe)
-  (:effects)
-  (:affected))
+  (:policy :fast-safe))
 
 (define-vop (fixnum-unop fast-safe-arith-op)
   (:args (x :scs (any-reg)))
@@ -200,22 +198,30 @@
         (:note "inline ASH")
         (:policy :fast-safe)
         (:args (number :scs (,reg) :to :save)
-               (count  :scs (signed-reg)
-                       ,@(if save
-                           '(:to :save))))
+               (count  :scs (signed-reg)))
         (:arg-types ,num ,tag)
         (:results (result :scs (,reg)))
         (:result-types ,num)
-        (:temporary (:scs (unsigned-reg) :to (:result 0)) temp)
+        (:temporary (:scs (unsigned-reg)
+                          ,@(unless save
+                              '(:to (:result 0)))) temp)
         (:generator 8
           (inst comb :>= count zero-tn positive :nullify t)
           (inst sub zero-tn count temp)
-          (inst comiclr 31 temp zero-tn :>=)
-          (inst li 31 temp)
-          (inst mtctl temp :sar)
-          (inst extrs number 0 1 temp)
-          (inst b done)
-          (inst shd temp number :variable result)
+          ,@(if save
+                '(;; Unsigned case
+                  (inst comiclr 31 temp result :>=)
+                  (inst b done :nullify t)
+                  (inst mtctl temp :sar)
+                  (inst b done)
+                  (inst shd zero-tn number :variable result))
+                '(;; Signed case
+                  (inst comiclr 31 temp zero-tn :>=)
+                  (inst li 31 temp)
+                  (inst mtctl temp :sar)
+                  (inst extrs number 0 1 temp)
+                  (inst b done)
+                  (inst shd temp number :variable result)))
           POSITIVE
           (inst subi 31 count temp)
           (inst mtctl temp :sar)
@@ -435,7 +441,7 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 30
-    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+    (let ((zero (generate-error-code vop 'division-by-zero-error x y)))
       (inst bc := nil y zero-tn zero))
     (move x x-pass)
     (move y y-pass)
@@ -468,7 +474,7 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 35
-    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+    (let ((zero (generate-error-code vop 'division-by-zero-error x y)))
       (inst bc := nil y zero-tn zero))
     (move x x-pass)
     (move y y-pass)
@@ -512,7 +518,7 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 35
-    (let ((zero (generate-error-code vop division-by-zero-error x y)))
+    (let ((zero (generate-error-code vop 'division-by-zero-error x y)))
       (inst bc := nil y zero-tn zero))
     (move x x-pass)
     (move y y-pass)
@@ -529,8 +535,6 @@
 (define-vop (fast-conditional)
   (:conditional)
   (:info target not-p)
-  (:effects)
-  (:affected)
   (:policy :fast-safe))
 
 (define-vop (fast-conditional/fixnum fast-conditional)
@@ -846,8 +850,6 @@
   (:results (hi-res :scs (unsigned-reg) :from (:result 1))
             (lo-res :scs (unsigned-reg) :from (:result 0)))
   (:result-types unsigned-num unsigned-num)
-  (:affected)
-  (:effects)
   (:generator 3
     (inst add lo extra lo-res)
     (inst addc hi zero-tn hi-res)))
@@ -928,28 +930,3 @@
     (inst subi 31 count temp)
     (inst mtctl temp :sar)
     (inst zdep digit :variable 32 result)))
-
-
-;;;; Static functions.
-
-(define-static-fun two-arg-gcd (x y) :translate gcd)
-(define-static-fun two-arg-lcm (x y) :translate lcm)
-
-(define-static-fun two-arg-+ (x y) :translate +)
-(define-static-fun two-arg-- (x y) :translate -)
-(define-static-fun two-arg-* (x y) :translate *)
-(define-static-fun two-arg-/ (x y) :translate /)
-
-(define-static-fun two-arg-< (x y) :translate <)
-(define-static-fun two-arg-<= (x y) :translate <=)
-(define-static-fun two-arg-> (x y) :translate >)
-(define-static-fun two-arg->= (x y) :translate >=)
-(define-static-fun two-arg-= (x y) :translate =)
-(define-static-fun two-arg-/= (x y) :translate /=)
-
-(define-static-fun %negate (x) :translate %negate)
-
-(define-static-fun two-arg-and (x y) :translate logand)
-(define-static-fun two-arg-ior (x y) :translate logior)
-(define-static-fun two-arg-xor (x y) :translate logxor)
-
