@@ -10,26 +10,6 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
-
-;;;; Interfaces to IR2 conversion:
-
-;;; Return a wired TN describing the N'th full call argument passing
-;;; location.
-(defun standard-arg-location (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-wired-tn *backend-t-primitive-type*
-                     register-arg-scn
-                     (elt *register-arg-offsets* n))
-      (make-wired-tn *backend-t-primitive-type*
-                     control-stack-arg-scn n)))
-
-(defun standard-arg-location-sc (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-sc-offset register-arg-scn
-                      (nth n *register-arg-offsets*))
-      (make-sc-offset control-stack-arg-scn n)))
 
 (defconstant arg-count-sc (make-sc-offset immediate-arg-scn nargs-offset))
 (defconstant closure-sc (make-sc-offset descriptor-reg-sc-number lexenv-offset))
@@ -40,8 +20,8 @@
 ;;; desire to use a subroutine call instruction.
 (defun make-return-pc-passing-location (standard)
   (if standard
-      (make-wired-tn *backend-t-primitive-type* register-arg-scn lra-offset)
-      (make-restricted-tn *backend-t-primitive-type* register-arg-scn)))
+      (make-wired-tn *backend-t-primitive-type* descriptor-reg-sc-number lra-offset)
+      (make-restricted-tn *backend-t-primitive-type* descriptor-reg-sc-number)))
 
 ;;; This is similar to MAKE-RETURN-PC-PASSING-LOCATION, but makes a
 ;;; location to pass OLD-FP in. This is (obviously) wired in the
@@ -74,38 +54,6 @@
 ;;; are using non-standard conventions.
 (defun make-arg-count-location ()
   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nargs-offset))
-
-
-;;; Make a TN to hold the number-stack frame pointer.  This is allocated
-;;; once per component, and is component-live.
-(defun make-nfp-tn ()
-  (component-live-tn
-   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nfp-offset)))
-
-(defun make-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-(defun make-number-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-;;; Return a list of TNs that can be used to represent an unknown-values
-;;; continuation within a function.
-(defun make-unknown-values-locations ()
-  (list (make-stack-pointer-tn)
-        (make-normal-tn *fixnum-primitive-type*)))
-
-
-;;; This function is called by the ENTRY-ANALYZE phase, allowing
-;;; VM-dependent initialization of the IR2-COMPONENT structure.  We push
-;;; placeholder entries in the Constants to leave room for additional
-;;; noise in the code object header.
-(defun select-component-format (component)
-  (declare (type component component))
-  (dotimes (i code-constants-offset)
-    (vector-push-extend nil
-                        (ir2-component-constants (component-info component))))
-  (values))
-
 
 ;;; bytes-needed-for-non-descriptor-stack-frame is the amount
 ;;; we grow or shrink the NSP/NFP stack. This stack is used
@@ -133,7 +81,7 @@
   (:generator 1
     (let ((nfp (current-nfp-tn vop)))
       (when nfp
-        (inst addi (- (bytes-needed-for-non-descriptor-stack-frame))
+        (inst ldo (- (bytes-needed-for-non-descriptor-stack-frame))
               nfp val)))))
 
 ;;; Accessing a slot from an earlier stack frame is definite hackery.
@@ -156,7 +104,12 @@
 
 (define-vop (xep-allocate-frame)
   (:info start-lab)
-  (:temporary (:scs (non-descriptor-reg)) temp)
+  ;; KLUDGE: Specify an explicit offset for TEMP because NARGS is a
+  ;; non-descriptor-reg, but is also live, yet the register allocator
+  ;; does not know that it is, and if TEMP collides NARGS and
+  ;; COMPUTE-CODE-FROM-LIP needs TEMP then we run into trouble very
+  ;; quickly.
+  (:temporary (:sc non-descriptor-reg :offset nl5-offset) temp)
   (:generator 1
     ;; Make sure the function is aligned, and drop a label pointing to this
     ;; function header.
@@ -178,12 +131,12 @@
 (define-vop (xep-setup-sp)
   (:vop-var vop)
   (:generator 1
-    (inst addi (* n-word-bytes (sb-allocated-size 'control-stack))
+    (inst ldo (* n-word-bytes (sb-allocated-size 'control-stack))
           cfp-tn csp-tn)
     (let ((nfp (current-nfp-tn vop)))
       (when nfp
         (move nsp-tn nfp)
-        (inst addi (bytes-needed-for-non-descriptor-stack-frame)
+        (inst ldo (bytes-needed-for-non-descriptor-stack-frame)
                    nsp-tn nsp-tn)))))
 
 (define-vop (allocate-frame)
@@ -192,11 +145,11 @@
   (:info callee)
   (:generator 2
     (move csp-tn res)
-    (inst addi (* n-word-bytes (sb-allocated-size 'control-stack))
+    (inst ldo (* n-word-bytes (sb-allocated-size 'control-stack))
           csp-tn csp-tn)
     (when (ir2-physenv-number-stack-p callee)
       (move nsp-tn nfp)
-      (inst addi (bytes-needed-for-non-descriptor-stack-frame)
+      (inst ldo (bytes-needed-for-non-descriptor-stack-frame)
                  nsp-tn nsp-tn))))
 
 ;;; Allocate a partial frame for passing stack arguments in a full call.  Nargs
@@ -209,7 +162,7 @@
   (:generator 2
     (when (> nargs register-arg-count)
       (move csp-tn res)
-      (inst addi (* nargs n-word-bytes) csp-tn csp-tn))))
+      (inst ldo (* nargs n-word-bytes) csp-tn csp-tn))))
 
 
 ;;; Fix: boil down below notes into something nicer
@@ -631,7 +584,6 @@ default-value-8
 ;;; more arg, but there is no new-FP, since the arguments have been set up in
 ;;; the current frame.
 ;;;
-
 (macrolet ((define-full-call (name named return variable)
   (aver (not (and variable (eq return :tail))))
   `(define-vop (,name
@@ -640,10 +592,12 @@ default-value-8
      (:args
       ,@(unless (eq return :tail)
           '((new-fp :scs (any-reg) :to :eval)))
-
-      ,(if named
-           '(name :target name-pass)
-           '(arg-fun :target lexenv))
+      ,@(case named
+          ((nil)
+           '((arg-fun :target lexenv)))
+          (:direct)
+          (t
+           '((name :target name-pass))))
 
       ,@(when (eq return :tail)
           '((ocfp :target ocfp-pass)
@@ -662,6 +616,7 @@ default-value-8
      (:vop-var vop)
      (:info ,@(unless (or variable (eq return :tail)) '(arg-locs))
             ,@(unless variable '(nargs))
+            ,@(when (eq named :direct) '(fun))
             ,@(when (eq return :fixed) '(nvals))
             step-instrumenting)
 
@@ -682,18 +637,19 @@ default-value-8
                   :to :eval)
                  return-pc-pass)
 
-     ,@(if named
-         `((:temporary (:sc descriptor-reg :offset fdefn-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       name-pass))
-
-         `((:temporary (:sc descriptor-reg :offset lexenv-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       lexenv)
-           (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
-                       function)))
+     ,@(case named
+         ((t)
+          `((:temporary (:sc descriptor-reg :offset fdefn-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        name-pass)))
+         ((nil)
+          `((:temporary (:sc descriptor-reg :offset lexenv-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        lexenv)
+            (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
+                        function))))
 
      (:temporary (:sc any-reg :offset nargs-offset :to :eval)
                  nargs-pass)
@@ -774,7 +730,7 @@ default-value-8
                                                  word-shift)
                                             cfp-tn return-pc-pass))))
                               (:frob-nfp
-                               (inst addi (- (bytes-needed-for-non-descriptor-stack-frame))
+                               (inst ldo (- (bytes-needed-for-non-descriptor-stack-frame))
                                           nsp-tn nsp-tn)))
                             `((:comp-lra
                                (inst compute-lra-from-code code-tn lra-label
@@ -807,49 +763,54 @@ default-value-8
                                           (ash (reg-tn-encoding callable-tn)
                                                5)))
                     (emit-label step-done-label))))
-           ,@(if named
-                 `((sc-case name
-                     (descriptor-reg (move name name-pass))
-                     (control-stack
-                      (inst ldw (ash (tn-offset name) word-shift)
-                                cfp-tn name-pass)
-                      (do-next-filler))
-                     (constant
-                      (inst ldw (- (ash (tn-offset name) word-shift)
-                                   other-pointer-lowtag)
-                                code-tn name-pass)
-                      (do-next-filler)))
-                   ;; The step instrumenting must be done after
-                   ;; FUNCTION is loaded, but before ENTRY-POINT is
-                   ;; calculated.
-                   (insert-step-instrumenting name-pass)
-                   (inst ldw (- (ash fdefn-raw-addr-slot word-shift)
-                                other-pointer-lowtag)
-                             name-pass entry-point)
-                   (do-next-filler))
-                 `((sc-case arg-fun
-                     (descriptor-reg
-                       (move arg-fun lexenv))
-                     (control-stack
-                      (inst ldw (ash (tn-offset arg-fun) word-shift)
-                                cfp-tn lexenv)
-                      (do-next-filler))
-                     (constant
-                      (inst ldw
-                            (- (ash (tn-offset arg-fun) word-shift)
-                               other-pointer-lowtag) code-tn lexenv)
-                      (do-next-filler)))
-                   (inst ldw (- (ash closure-fun-slot word-shift)
+           (declare (ignorable #'insert-step-instrumenting))
+           ,@(case named
+               ((t)
+                `((sc-case name
+                    (descriptor-reg (move name name-pass))
+                    (control-stack
+                     (inst ldw (ash (tn-offset name) word-shift)
+                           cfp-tn name-pass)
+                     (do-next-filler))
+                    (constant
+                     (inst ldw (- (ash (tn-offset name) word-shift)
+                                  other-pointer-lowtag)
+                           code-tn name-pass)
+                     (do-next-filler)))
+                  ;; The step instrumenting must be done after
+                  ;; FUNCTION is loaded, but before ENTRY-POINT is
+                  ;; calculated.
+                  (insert-step-instrumenting name-pass)
+                  (inst ldw (- (ash fdefn-raw-addr-slot word-shift)
+                               other-pointer-lowtag)
+                        name-pass entry-point)
+                  (do-next-filler)))
+               ((nil)
+                `((sc-case arg-fun
+                    (descriptor-reg
+                     (move arg-fun lexenv))
+                    (control-stack
+                     (inst ldw (ash (tn-offset arg-fun) word-shift)
+                           cfp-tn lexenv)
+                     (do-next-filler))
+                    (constant
+                     (inst ldw
+                           (- (ash (tn-offset arg-fun) word-shift)
+                              other-pointer-lowtag) code-tn lexenv)
+                     (do-next-filler)))
+                  (inst ldw (- (ash closure-fun-slot word-shift)
+                               fun-pointer-lowtag)
+                        lexenv function)
+                  (do-next-filler)
+                  ;; The step instrumenting must be done before
+                  ;; after FUNCTION is loaded, but before ENTRY-POINT
+                  ;; is calculated.
+                  (insert-step-instrumenting function)
+                  (inst addi (- (ash simple-fun-code-offset word-shift)
                                 fun-pointer-lowtag)
-                             lexenv function)
-                   (do-next-filler)
-                   ;; The step instrumenting must be done before
-                   ;; after FUNCTION is loaded, but before ENTRY-POINT
-                   ;; is calculated.
-                   (insert-step-instrumenting function)
-                   (inst addi (- (ash simple-fun-code-offset word-shift)
-                                 fun-pointer-lowtag)
-                              function entry-point)))
+                        function entry-point)))
+               (:direct
+                `((inst ldw (static-fun-offset fun) null-tn entry-point))))
            (loop
              (if (cdr filler)
                  (do-next-filler)
@@ -877,10 +838,13 @@ default-value-8
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
+  (define-full-call static-call-named :direct :fixed nil)
   (define-full-call multiple-call nil :unknown nil)
   (define-full-call multiple-call-named t :unknown nil)
+  (define-full-call static-multiple-call-named :direct :unknown nil)
   (define-full-call tail-call nil :tail nil)
   (define-full-call tail-call-named t :tail nil)
+  (define-full-call static-tail-call-named :direct :tail nil)
 
   (define-full-call call-variable nil :fixed t)
   (define-full-call multiple-call-variable nil :unknown t))
@@ -990,7 +954,7 @@ default-value-8
         ;; restore the frame pointer and clear as much of the control
         ;; stack as possible.
         (move ocfp cfp-tn)
-        (inst addi (* nvals n-word-bytes) val-ptr csp-tn)
+        (inst ldo (* nvals n-word-bytes) val-ptr csp-tn)
         (aver (= (* nvals n-word-bytes) (fixnumize nvals)))
         ;; pre-default any argument register that need it.
         (when (< nvals register-arg-count)
@@ -1213,7 +1177,7 @@ default-value-8
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
-           (generate-error-code vop invalid-arg-count-error nargs)))
+           (generate-error-code vop 'invalid-arg-count-error nargs)))
       (cond ((zerop count)
              (inst bc :<> nil nargs zero-tn err-lab))
             (t
@@ -1229,7 +1193,7 @@ default-value-8
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
-           (generate-error-code vop invalid-arg-count-error nargs)))
+           (generate-error-code vop 'invalid-arg-count-error nargs)))
       (cond ((not min)
              (if (zerop max)
                  (inst bc :<> nil nargs zero-tn err-lab)
@@ -1241,32 +1205,6 @@ default-value-8
             ((plusp min)
              (inst bci :> nil (fixnumize min) nargs err-lab))))))
 
-;;; Signal argument errors.
-;;;
-(macrolet ((frob (name error translate &rest args)
-             `(define-vop (,name)
-                ,@(when translate
-                    `((:policy :fast-safe)
-                      (:translate ,translate)))
-                (:args ,@(mapcar #'(lambda (arg)
-                                     `(,arg :scs (any-reg descriptor-reg)))
-                                 args))
-                (:vop-var vop)
-                (:save-p :compute-only)
-                (:generator 1000
-                  (error-call vop ,error ,@args)))))
-  (frob arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs fname)
-  (frob type-check-error object-not-type-error sb!c::%type-check-error
-    object type)
-  (frob layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
-    object layout)
-  (frob odd-key-args-error odd-key-args-error
-    sb!c::%odd-key-args-error)
-  (frob unknown-key-arg-error unknown-key-arg-error
-    sb!c::%unknown-key-arg-error key)
-  (frob nil-fun-returned-error nil-fun-returned-error nil fun))
-
 ;;; Single-stepping
 
 (define-vop (step-instrument-before-vop)
