@@ -598,6 +598,8 @@
                       (reference-leaf start next result form))
                      (t
                       (reference-constant start next result form))))
+              ((not (proper-list-p form))
+               (compiler-error "~@<~S is not a proper list.~@:>" form))
               (t
                (ir1-convert-functoid start next result form)))))
     (values))
@@ -977,9 +979,8 @@
                    ;; Get an interned record cons for the path. A cons
                    ;; with the same object identity must be used for
                    ;; each instrument for the same block.
-                   (or (gethash path *code-coverage-records*)
-                       (setf (gethash path *code-coverage-records*)
-                             (cons path +code-coverage-unmarked+))))
+                   (ensure-gethash path *code-coverage-records*
+                                   (cons path +code-coverage-unmarked+)))
                   (next (make-ctran))
                   (*allow-instrumenting* nil))
               (push (ctran-block start)
