@@ -13,10 +13,6 @@
 
 (in-package "SB!C")
 
-;;; Whether reference to a thing which cannot be defined causes a full
-;;; warning.
-(defvar *flame-on-necessarily-undefined-thing* nil)
-
 (defvar *check-consistency* nil)
 
 ;;; Set to NIL to disable loop analysis for register allocation.
@@ -271,10 +267,8 @@ Examples:
                   (warnings (undefined-warning-warnings undef))
                   (undefined-warning-count (undefined-warning-count undef)))
               (dolist (*compiler-error-context* warnings)
-                (if #-sb-xc-host (and (member kind '(:function :type))
-                                      (name-reserved-by-ansi-p name kind)
-                                      *flame-on-necessarily-undefined-thing*)
-                    #+sb-xc-host nil
+                (if (and (member kind '(:function :type))
+                         (name-reserved-by-ansi-p name kind))
                     (ecase kind
                       (:function
                        (compiler-warn
@@ -361,7 +355,8 @@ Examples:
   ;; This binding could just as well be in WITH-IR1-NAMESPACE, but
   ;; since it's primarily a debugging tool, it's nicer to have
   ;; a wider unique scope by ID.
-  `(let ((*compiler-ir-obj-map* (make-compiler-ir-obj-map)))
+  `(let ((*compiler-ir-obj-map* (make-compiler-ir-obj-map))
+         (*finite-sbs* ,(finite-sbs-ctor-form)))
      (unwind-protect
          (let ((*warnings-p* nil)
                (*failure-p* nil))
@@ -567,6 +562,7 @@ necessary, since type inference may take arbitrarily long to converge.")
 (defun %compile-component (component)
   (let ((*code-segment* nil)
         (*elsewhere* nil)
+        (*elsewhere-label* nil)
         #!+inline-constants (*unboxed-constants* nil))
     (maybe-mumble "GTN ")
     (gtn-analyze component)
@@ -2097,9 +2093,10 @@ SPEED and COMPILATION-SPEED optimization values, and the
 
 ;;;; Host compile time definitions
 #+sb-xc-host
-(defun compile-in-lexenv (name lambda lexenv)
+(defun compile-in-lexenv (lambda lexenv &rest rest)
   (declare (ignore lexenv))
-  (compile name lambda))
+  (aver (null rest))
+  (compile nil lambda))
 
 #+sb-xc-host
 (defun eval-tlf (form index &optional lexenv)
