@@ -1783,36 +1783,33 @@
                (t
                 (values nil nil)))))))
 
-(defvar *assembler-routines-by-addr* nil)
-
-;;; Build an address-name hash-table from the name-address hash
-(defun invert-address-hash (htable &optional (addr-hash (make-hash-table)))
-  (maphash (lambda (name address)
-             (setf (gethash address addr-hash) name))
-           htable)
-  addr-hash)
+(defglobal *assembler-routines-by-addr* nil)
 
 ;;; Return the name of the primitive Lisp assembler routine or foreign
 ;;; symbol located at ADDRESS, or NIL if there isn't one.
-(defun find-assembler-routine (address)
+(defun find-assembler-routine (address &aux (addr->name *assembler-routines-by-addr*))
   (declare (type address address))
-  (when (null *assembler-routines-by-addr*)
-    (setf *assembler-routines-by-addr*
-          (invert-address-hash sb!fasl:*assembler-routines*))
+  (when (null addr->name)
+    (setf addr->name (make-hash-table) *assembler-routines-by-addr* addr->name)
+    (flet ((invert (name->addr)
+             (maphash (lambda (name address)
+                        (setf (gethash address addr->name) name))
+                      name->addr)))
+       (invert sb!fasl:*assembler-routines*)
     #!-sb-dynamic-core
-    (setf *assembler-routines-by-addr*
-          (invert-address-hash *static-foreign-symbols*
-                               *assembler-routines-by-addr*))
+       (invert *static-foreign-symbols*))
     (loop for name across sb!vm:+static-fdefns+
           for address =
           #!+immobile-code (sb!vm::function-raw-address name)
           #!-immobile-code (+ sb!vm::nil-value (sb!vm::static-fun-offset name))
-          do (setf (gethash address *assembler-routines-by-addr*) name))
+          do (setf (gethash address addr->name) name))
+    #!+immobile-code
+    (compute-extra-asm-routines addr->name)
     ;; Not really a routine, but it uses the similar logic for annotations
     #!+sb-safepoint
-    (setf (gethash sb!vm::gc-safepoint-page-addr *assembler-routines-by-addr*)
+    (setf (gethash sb!vm::gc-safepoint-page-addr addr->name)
           "safepoint"))
-  (gethash address *assembler-routines-by-addr*))
+  (gethash address addr->name))
 
 ;;;; some handy function for machine-dependent code to use...
 
@@ -1996,23 +1993,34 @@
       t)))
 
 (defun maybe-note-static-symbol (address dstate)
-  (dovector (symbol sb!vm:+static-symbols+)
-    (when (= (get-lisp-obj-address symbol) address)
-      (return (note (lambda (s) (prin1 symbol s)) dstate))))
-  ;; Guess whether 'address' is an immobile-space symbol by looking at
-  ;; code header constants. If it matches any constant, assume that it
-  ;; is a use of the constant.  This has false positives of course,
-  ;; as does MAYBE-NOTE-STATIC-SYMBOL in general - any random immediate
-  ;; used in an unboxed context, such as an ADD instruction,
-  ;; might be seen as an address.
-  #!+immobile-space
-  (unless (eql address 0)
-    (let ((code (seg-code (dstate-segment dstate))))
-      (when code
-        (loop for i downfrom (1- (code-header-words code)) to sb!vm:code-constants-offset
-              for const = (code-header-ref code i)
-              when (eql (get-lisp-obj-address const) address)
-              return (note (lambda (s) (prin1-quoted-short const s)) dstate))))))
+  (declare (type disassem-state dstate))
+  (when (or (not (typep address `(unsigned-byte ,sb!vm:n-machine-word-bits)))
+            (eql address 0))
+    (return-from maybe-note-static-symbol))
+  (let ((symbol
+         (block found
+           (when (eq address sb!vm::nil-value)
+             (return-from found nil))
+           (when (< address (sap-int sb!vm:*static-space-free-pointer*))
+             (dovector (symbol sb!vm:+static-symbols+)
+               (when (= (get-lisp-obj-address symbol) address)
+                 (return-from found symbol))))
+           ;; Guess whether 'address' is an immobile-space symbol by looking at
+           ;; code header constants. If it matches any constant, assume that it
+           ;; is a use of the constant.  This has false positives of course,
+           ;; as does MAYBE-NOTE-STATIC-SYMBOL in general - any random immediate
+           ;; used in an unboxed context, such as an ADD instruction,
+           ;; might be wrongly construed as an address.
+           #!+immobile-space
+           (let ((code (seg-code (dstate-segment dstate))))
+             (when code
+               (loop for i downfrom (1- (code-header-words code))
+                     to sb!vm:code-constants-offset
+                     for const = (code-header-ref code i)
+                     when (eql (get-lisp-obj-address const) address)
+                     do (return-from found const))))
+           (return-from maybe-note-static-symbol))))
+    (note (lambda (s) (prin1 symbol s)) dstate)))
 
 (defun get-internal-error-name (errnum)
   (cadr (svref sb!c:+backend-internal-errors+ errnum)))
