@@ -32,9 +32,7 @@
 ;;;; unary operations
 
 (define-vop (fast-safe-arith-op)
-  (:policy :fast-safe)
-  (:effects)
-  (:affected))
+  (:policy :fast-safe))
 
 (define-vop (fixnum-unop fast-safe-arith-op)
   (:args (x :scs (any-reg) :target res))
@@ -245,10 +243,10 @@
            (inst xor r y))))))
 
 (define-vop (fast-logior-unsigned-signed=>signed fast-safe-arith-op)
-  (:args (x :scs (unsigned-reg))
+  (:args (x :scs (unsigned-reg) :to (:result 1))
          (y :target r :scs (signed-reg)))
   (:arg-types unsigned-num signed-num)
-  (:results (r :scs (signed-reg) :from (:argument 1)))
+  (:results (r :scs (signed-reg)))
   (:result-types signed-num)
   (:note "inline (unsigned-byte 64) arithmetic")
   (:translate logior)
@@ -1200,8 +1198,6 @@ constant shift greater than word length")))
 (define-vop (fast-conditional)
   (:conditional :e)
   (:info)
-  (:effects)
-  (:affected)
   (:policy :fast-safe))
 
 (define-vop (fast-conditional/fixnum fast-conditional)
@@ -1258,12 +1254,10 @@ constant shift greater than word length")))
                          (:generator ,cost
                           (emit-optimized-test-inst x
                            ,(case suffix
-                             (-c/fixnum
-                              `(constantize (fixnumize y)))
-                             ((-c/signed -c/unsigned)
-                              `(constantize y))
-                             (t
-                              'y)))))))))
+                             (-c/fixnum `(constantize (fixnumize y)))
+                             ((-c/signed -c/unsigned) `(constantize y))
+                             (t 'y))
+                           nil)))))))
   (define-logtest-vops))
 
 (defknown %logbitp (integer unsigned-byte) boolean
@@ -1279,7 +1273,8 @@ constant shift greater than word length")))
   (:conditional :c)
   (:arg-types tagged-num (:constant (integer 0 #.(- 63 n-fixnum-tag-bits))))
   (:generator 4
-    (inst bt x (+ y n-fixnum-tag-bits))))
+    (let ((bit (+ y n-fixnum-tag-bits)))
+      (inst bt (if (<= bit 31) (reg-in-size x :dword) x) bit))))
 
 (define-vop (fast-logbitp/signed fast-conditional/signed)
   (:args (x :scs (signed-reg signed-stack))
@@ -2023,18 +2018,6 @@ constant shift greater than word length")))
           (inst and r (or (immediate32-p mask)
                           (constantize mask))))))))
 
-;;;; static functions
-
-(define-static-fun two-arg-/ (x y) :translate /)
-
-(define-static-fun two-arg-gcd (x y) :translate gcd)
-(define-static-fun two-arg-lcm (x y) :translate lcm)
-
-(define-static-fun two-arg-and (x y) :translate logand)
-(define-static-fun two-arg-ior (x y) :translate logior)
-(define-static-fun two-arg-xor (x y) :translate logxor)
-
-
 (in-package "SB!C")
 
 (defun *-transformer (y)
