@@ -186,13 +186,16 @@
     ;; Save the return-pc.
     (popw rbp-tn (frame-word-offset return-pc-save-offset))))
 
+(defun emit-lea (target source disp)
+  (if (eql disp 0)
+      (inst mov target source)
+      (inst lea target (make-ea :qword :base source :disp disp))))
+
 (define-vop (xep-setup-sp)
   (:generator 1
-    (inst lea rsp-tn
-          (make-ea :qword :base rbp-tn
-                          :disp (- (* n-word-bytes
-                                      (- (max 3 (sb-allocated-size 'stack))
-                                         sp->fp-offset)))))))
+    (emit-lea rsp-tn rbp-tn     (- (* n-word-bytes
+                                      (- (sb-allocated-size 'stack)
+                                         sp->fp-offset))))))
 
 ;;; This is emitted directly before either a known-call-local, call-local,
 ;;; or a multiple-call-local. All it does is allocate stack space for the
@@ -217,7 +220,8 @@
   (:generator 2
     (inst lea res (make-ea :qword :base rsp-tn
                            :disp (- (* sp->fp-offset n-word-bytes))))
-    (inst sub rsp-tn (* (max nargs 3) n-word-bytes))))
+    (inst sub rsp-tn (* (max nargs (sb!c::sb-size (sb-or-lose 'stack)))
+                        n-word-bytes))))
 
 ;;; Emit code needed at the return-point from an unknown-values call
 ;;; for a fixed number of values. Values is the head of the TN-REF
@@ -286,11 +290,11 @@
              (defaulting-done (gen-label))
              (default-stack-slots (gen-label)))
          (note-this-location vop :unknown-return)
+         (inst mov rax-tn nil-value)
          ;; Branch off to the MV case.
          (inst jmp :c regs-defaulted)
          ;; Do the single value case.
          ;; Default the register args
-         (inst mov rax-tn nil-value)
          (do ((i 1 (1+ i))
               (val (tn-ref-across values) (tn-ref-across val)))
              ((= i (min nvals register-arg-count)))
@@ -300,7 +304,6 @@
          (move rbx-tn rsp-tn)
          (inst jmp default-stack-slots)
          (emit-label regs-defaulted)
-         (inst mov rax-tn nil-value)
          (collect ((defaults))
            (do ((i register-arg-count (1+ i))
                 (val (do ((i 0 (1+ i))
@@ -345,6 +348,7 @@
              (no-stack-args (gen-label))
              (default-stack-vals (gen-label))
              (count-okay (gen-label)))
+         (inst std)
          (note-this-location vop :unknown-return)
          ;; Branch off to the MV case.
          (inst jmp :c regs-defaulted)
@@ -363,7 +367,6 @@
          ;; Load RAX with NIL so we can quickly store it, and set up
          ;; stuff for the loop.
          (inst mov rax-tn nil-value)
-         (inst std)
          (inst mov rcx-tn (- nvals register-arg-count))
          ;; Jump into the default loop.
          (inst jmp default-stack-vals)
@@ -395,7 +398,6 @@
                                (+ sp->fp-offset register-arg-count))))
          ;; Do the copy.
          (inst shr rcx-tn n-fixnum-tag-bits)   ; make word count
-         (inst std)
          (inst rep)
          (inst movs :qword)
          ;; Restore RSI.
@@ -502,13 +504,13 @@
 (defun check-ocfp-and-return-pc (old-fp return-pc)
   #+nil
   (format t "*known-return: old-fp ~S, tn-kind ~S; ~S ~S~%"
-          old-fp (sb!c::tn-kind old-fp) (sb!c::tn-save-tn old-fp)
-          (sb!c::tn-kind (sb!c::tn-save-tn old-fp)))
+          old-fp (tn-kind old-fp) (sb!c::tn-save-tn old-fp)
+          (tn-kind (sb!c::tn-save-tn old-fp)))
   #+nil
   (format t "*known-return: return-pc ~S, tn-kind ~S; ~S ~S~%"
-          return-pc (sb!c::tn-kind return-pc)
+          return-pc (tn-kind return-pc)
           (sb!c::tn-save-tn return-pc)
-          (sb!c::tn-kind (sb!c::tn-save-tn return-pc)))
+          (tn-kind (sb!c::tn-save-tn return-pc)))
   (unless (and (sc-is old-fp control-stack)
                (= (tn-offset old-fp) ocfp-save-offset))
     (error "ocfp not on stack in standard save location?"))
@@ -666,17 +668,22 @@
 ;;; In tail call with fixed arguments, the passing locations are
 ;;; passed as a more arg, but there is no new-FP, since the arguments
 ;;; have been set up in the current frame.
-(macrolet ((define-full-call (name named return variable)
+(macrolet ((define-full-call (vop-name named return variable)
             (aver (not (and variable (eq return :tail))))
-            `(define-vop (,name
-                          ,@(when (eq return :unknown)
-                              '(unknown-values-receiver)))
+            #!+immobile-code (when named (setq named :direct))
+            `(define-vop (,vop-name ,@(when (eq return :unknown)
+                                        '(unknown-values-receiver)))
                (:args
                ,@(unless (eq return :tail)
                    '((new-fp :scs (any-reg) :to (:argument 1))))
 
-               (fun :scs (descriptor-reg control-stack)
-                    :target rax :to (:argument 0))
+               ;; If immobile-space is in use, then named call does not require
+               ;; a register unless the caller is NOT in immobile space,
+               ;; in which case the register is needed because there is no
+               ;; absolute addressing mode for jmp/call.
+               ,@(unless (eq named :direct)
+                   '((fun :scs (descriptor-reg control-stack)
+                          :target rax :to (:argument 0))))
 
                ,@(when (eq return :tail)
                    '((old-fp)
@@ -685,7 +692,7 @@
                ,@(unless variable '((args :more t :scs (descriptor-reg)))))
 
                ,@(when (eq return :fixed)
-               '((:results (values :more t))))
+                   '((:results (values :more t))))
 
                (:save-p ,(if (eq return :tail) :compute-only t))
 
@@ -693,9 +700,14 @@
                '((:move-args :full-call)))
 
                (:vop-var vop)
+               (:node-var node)
                (:info
                ,@(unless (or variable (eq return :tail)) '(arg-locs))
                ,@(unless variable '(nargs))
+               ;; Intuitively you might want FUN to be the first codegen arg,
+               ;; but that won't work, because EMIT-ARG-MOVES wants the
+               ;; passing locs in (FIRST (vop-codegen-info vop)).
+               ,@(when (eq named :direct) '(fun))
                ,@(when (eq return :fixed) '(nvals))
                step-instrumenting)
 
@@ -709,12 +721,9 @@
                ;; with the real function and invoke the real function
                ;; for closures. Non-closures do not need this value,
                ;; so don't care what shows up in it.
-               (:temporary
-               (:sc descriptor-reg
-                    :offset rax-offset
-                    :from (:argument 0)
-                    :to :eval)
-               rax)
+               ,@(unless (eq named :direct)
+                   '((:temporary (:sc descriptor-reg :offset rax-offset
+                                  :from (:argument 0) :to :eval) rax)))
 
                ;; We pass the number of arguments in RCX.
                (:temporary (:sc unsigned-reg :offset rcx-offset :to :eval) rcx)
@@ -745,11 +754,14 @@
                                (if (eq return :tail) 0 10)
                                15
                                (if (eq return :unknown) 25 0))
+
+               (progn node) ; always "use" it
+
                ;; This has to be done before the frame pointer is
                ;; changed! RAX stores the 'lexical environment' needed
                ;; for closures.
-               (move rax fun)
-
+               ,@(unless (eq named :direct)
+                   '((move rax fun)))
 
                ,@(if variable
                      ;; For variable call, compute the number of
@@ -847,14 +859,39 @@
 
                (note-this-location vop :call-site)
 
-               (inst ,(if (eq return :tail) 'jmp 'call)
-                     (make-ea :qword :base rax
-                              :disp ,(if named
-                                         '(- (* fdefn-raw-addr-slot
-                                                n-word-bytes)
-                                             other-pointer-lowtag)
-                                       '(- (* closure-fun-slot n-word-bytes)
-                                           fun-pointer-lowtag))))
+               ,(if (eq named :direct)
+                    #!+immobile-code
+                    `(let* ((fixup (make-fixup
+                                    fun
+                                    (if (static-fdefn-offset fun)
+                                        :static-call
+                                        :named-call)))
+                            (target
+                              (if (and (sb!c::code-immobile-p node)
+                                       (not step-instrumenting))
+                                  fixup
+                                  (progn
+                                    ;; RAX-TN was not declared as a temp var,
+                                    ;; however it's sole purpose at this point is
+                                    ;; for function call, so even if it was used
+                                    ;; to compute a stack argument, it's free now.
+                                    ;; If the call hits the undefined fun trap,
+                                    ;; RAX will get loaded regardless.
+                                    (inst mov rax-tn fixup)
+                                    rax-tn))))
+                       (inst ,(if (eq return :tail) 'jmp 'call) target))
+                    #!-immobile-code
+                    `(inst ,(if (eq return :tail) 'jmp 'call)
+                           (make-ea :qword :disp
+                                    (+ nil-value (static-fun-offset fun))))
+                    `(inst ,(if (eq return :tail) 'jmp 'call)
+                           (make-ea :qword :base rax
+                                           :disp ,(if named
+                                                      '(- (* fdefn-raw-addr-slot
+                                                           n-word-bytes)
+                                                        other-pointer-lowtag)
+                                                      '(- (* closure-fun-slot n-word-bytes)
+                                                        fun-pointer-lowtag)))))
                ,@(ecase return
                    (:fixed
                     '((default-unknown-values vop values nvals node)))
@@ -866,14 +903,22 @@
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
+  #!-immobile-code
+  (define-full-call static-call-named :direct :fixed nil)
   (define-full-call multiple-call nil :unknown nil)
   (define-full-call multiple-call-named t :unknown nil)
+  #!-immobile-code
+  (define-full-call static-multiple-call-named :direct :unknown nil)
   (define-full-call tail-call nil :tail nil)
   (define-full-call tail-call-named t :tail nil)
+  #!-immobile-code
+  (define-full-call static-tail-call-named :direct :tail nil)
 
   (define-full-call call-variable nil :fixed t)
   (define-full-call multiple-call-variable nil :unknown t))
 
+
+
 ;;; This is defined separately, since it needs special code that BLT's
 ;;; the arguments down. All the real work is done in the assembly
 ;;; routine. We just set things up so that it can find what it needs.
@@ -885,14 +930,14 @@
   (:temporary (:sc unsigned-reg :offset rsi-offset :from (:argument 0)) rsi)
   (:temporary (:sc unsigned-reg :offset rax-offset :from (:argument 1)) rax)
   (:temporary (:sc unsigned-reg) call-target)
+  (:vop-var vop)
   (:generator 75
     (check-ocfp-and-return-pc old-fp return-pc)
     ;; Move these into the passing locations if they are not already there.
     (move rsi args)
     (move rax function)
     ;; And jump to the assembly routine.
-    (inst mov call-target (make-fixup 'tail-call-variable :assembly-routine))
-    (inst jmp call-target)))
+    (invoke-asm-routine 'jmp 'tail-call-variable vop call-target)))
 
 ;;;; unknown values return
 
@@ -1012,6 +1057,7 @@
   (:temporary (:sc descriptor-reg :offset (first *register-arg-offsets*)
                    :from (:eval 0)) a0)
   (:node-var node)
+  (:vop-var vop)
   (:generator 13
     (check-ocfp-and-return-pc old-fp return-pc)
     (unless (policy node (> space speed))
@@ -1032,8 +1078,7 @@
         (emit-label not-single)))
     (move rsi vals)
     (move rcx nvals)
-    (inst mov return-asm (make-fixup 'return-multiple :assembly-routine))
-    (inst jmp return-asm)))
+    (invoke-asm-routine 'jmp 'return-multiple vop return-asm)))
 
 ;;;; XEP hackery
 
@@ -1052,11 +1097,15 @@
   (:temporary (:sc any-reg :offset r8-offset) copy-index)
   (:temporary (:sc any-reg :offset r9-offset) source)
   (:temporary (:sc descriptor-reg :offset r10-offset) temp)
-  (:info fixed)
+  (:info fixed min-verified)
   (:generator 20
     ;; Avoid the copy if there are no more args.
     (cond ((zerop fixed)
            (inst jrcxz JUST-ALLOC-FRAME))
+          ((and (eql min-verified fixed)
+                (> fixed 1))
+           ;; verify-arg-count will do a CMP
+           (inst jmp :e JUST-ALLOC-FRAME))
           (t
            (inst cmp rcx-tn (fixnumize fixed))
            (inst jmp :be JUST-ALLOC-FRAME)))
@@ -1067,45 +1116,45 @@
     (inst neg temp)
 
     ;; Allocate the space on the stack.
-    ;; stack = rbp + sp->fp-offset - (max 3 frame-size) - (nargs - fixed)
+    ;; stack = rbp + sp->fp-offset - frame-size - (nargs - fixed)
     ;; if we'd move SP backward, swap the meaning of rsp and source;
     ;; otherwise, we'd be accessing values below SP, and that's no good
     ;; if a signal interrupts this code sequence.  In that case, store
     ;; the final value in rsp after the stack-stack memmove loop.
-    (inst lea (if (<= fixed (max 3 (sb-allocated-size 'stack)))
+    (inst lea (if (<= fixed (sb-allocated-size 'stack))
                   rsp-tn
                   source)
           (make-ea :qword :base rbp-tn
                           :index temp :scale (ash 1 (- word-shift n-fixnum-tag-bits))
                           :disp (* n-word-bytes
                                    (- (+ sp->fp-offset fixed)
-                                      (max 3 (sb-allocated-size 'stack))))))
+                                      (sb-allocated-size 'stack)))))
 
     ;; Now: nargs>=1 && nargs>fixed
 
-    ;; Save the original count of args.
-    (inst mov rbx-tn rcx-tn)
-
     (cond ((< fixed register-arg-count)
            ;; the code above only moves the final value of rsp in
            ;; rsp directly if that condition is satisfied.  Currently,
            ;; r-a-c is 3, so the aver is OK. If the calling convention
            ;; ever changes, the logic above with LEA will have to be
            ;; adjusted.
-           (aver (<= fixed (max 3 (sb-allocated-size 'stack))))
+           (aver (<= fixed (sb-allocated-size 'stack)))
            ;; We must stop when we run out of stack args, not when we
            ;; run out of more args.
            ;; Number to copy = nargs-3
+           ;; Save the original count of args.
+           (inst mov rbx-tn rcx-tn)
            (inst sub rbx-tn (fixnumize register-arg-count))
            ;; Everything of interest in registers.
            (inst jmp :be DO-REGS))
           (t
            ;; Number to copy = nargs-fixed
-           (inst sub rbx-tn (fixnumize fixed))))
+           (inst lea rbx-tn (make-ea :qword :base rcx-tn
+                                     :disp (- (fixnumize fixed))))))
 
     ;; Initialize R8 to be the end of args.
     ;; Swap with SP if necessary to mirror the previous condition
-    (inst lea (if (<= fixed (max 3 (sb-allocated-size 'stack)))
+    (inst lea (if (<= fixed (sb-allocated-size 'stack))
                   source
                   rsp-tn)
           (make-ea :qword :base rbp-tn
@@ -1113,8 +1162,8 @@
                           :disp (* sp->fp-offset n-word-bytes)))
 
     ;; src: rbp + temp + sp->fp
-    ;; dst: rbp + temp + sp->fp + (fixed - (max 3 [stack-size]))
-    (let ((delta (- fixed (max 3 (sb-allocated-size 'stack))))
+    ;; dst: rbp + temp + sp->fp + (fixed - [stack-size])
+    (let ((delta (- fixed (sb-allocated-size 'stack)))
           (loop (gen-label))
           (fixnum->word (ash 1 (- word-shift n-fixnum-tag-bits))))
       (cond ((zerop delta)) ; no-op move
@@ -1157,8 +1206,7 @@
                                     (- sp->fp-offset
                                        (+ 1
                                           (- i fixed)
-                                          (max 3 (sb-allocated-size
-                                                  'stack))))))
+                                          (sb-allocated-size 'stack)))))
               (nth i *register-arg-tns*))
 
         (incf i)
@@ -1174,11 +1222,10 @@
     (inst jmp DONE)
 
     JUST-ALLOC-FRAME
-    (inst lea rsp-tn
-          (make-ea :qword :base rbp-tn
-                   :disp (* n-word-bytes
+    (emit-lea rsp-tn rbp-tn
+                         (* n-word-bytes
                             (- sp->fp-offset
-                               (max 3 (sb-allocated-size 'stack))))))
+                               (sb-allocated-size 'stack))))
 
     DONE))
 
@@ -1311,21 +1358,27 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
-    (let ((err-lab (generate-error-code vop 'invalid-arg-count-error)))
-      (cond ((not min)
-             (if (zerop max)
-                 (inst test nargs nargs)
-                 (inst cmp nargs (fixnumize max)))
-             (inst jmp :ne err-lab))
-            (max
-             (when (plusp min)
-               (inst cmp nargs (fixnumize min))
-               (inst jmp :b err-lab))
-             (inst cmp nargs (fixnumize max))
-             (inst jmp :a err-lab))
-            ((plusp min)
-             (inst cmp nargs (fixnumize min))
-             (inst jmp :b err-lab))))))
+    ;; NOTE: copy-more-arg expects this to issue a CMP for min > 1
+    (let ((err-lab
+            (generate-error-code vop 'invalid-arg-count-error nargs)))
+      (flet ((check-min ()
+               (cond ((= min 1)
+                      (inst test nargs nargs)
+                      (inst jmp :e err-lab))
+                     ((plusp min)
+                      (inst cmp nargs (fixnumize min))
+                      (inst jmp :b err-lab)))))
+        (cond ((not min)
+               (if (zerop max)
+                   (inst test nargs nargs)
+                   (inst cmp nargs (fixnumize max)))
+               (inst jmp :ne err-lab))
+              (max
+               (check-min)
+               (inst cmp nargs (fixnumize max))
+               (inst jmp :a err-lab))
+              (t
+               (check-min)))))))
 
 ;; Signal an error about an untagged number.
 ;; These are pretty much boilerplate and could be generic except:
@@ -1341,8 +1394,9 @@
   (:args (object :scs (signed-reg unsigned-reg))
          ;; Types are trees of symbols, so 'any-reg' is not
          ;; really possible.
-         (type :scs (any-reg descriptor-reg)))
-  (:arg-types untagged-num *)
+         (type :scs (any-reg descriptor-reg constant)))
+  (:arg-types untagged-num * (:constant t))
+  (:info *location-context*)
   (:vop-var vop)
   (:save-p :compute-only)
   ;; cost is a smidgen less than type-check-error
@@ -1353,8 +1407,8 @@
   (:policy :fast-safe)
   (:translate sb!c::%type-check-error/c)
   (:args (object :scs (signed-reg unsigned-reg)))
-  (:arg-types untagged-num (:constant symbol))
-  (:info errcode)
+  (:arg-types untagged-num (:constant symbol) (:constant t))
+  (:info errcode *location-context*)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 899 ; smidgen less than type-check-error/c
