@@ -32,30 +32,31 @@
     (progn name) ; ignore it
     (generate-set-slot object value temp offset lowtag)))
 
-(defun generate-set-slot (object value temp offset lowtag)
+(defun generate-set-slot (object value temp offset lowtag &optional zeroed)
     (if (sc-is value immediate)
         (move-immediate (make-ea :qword
                                  :base object
                                  :disp (- (* offset n-word-bytes)
                                           lowtag))
                         (encode-value-if-immediate value)
-                        temp)
+                        temp zeroed)
         ;; Else, value not immediate.
         (storew value object offset lowtag)))
 
 ;; INIT-SLOT has to know about the :COMPACT-INSTANCE-HEADER feature.
 (define-vop (init-slot set-slot)
+  (:info name dx-p offset lowtag)
   (:generator 1
-    (progn name)
+    (progn name dx-p)
     (if (or #!+compact-instance-header
             (and (eq name '%make-structure-instance) (eql offset :layout)))
         ;; The layout is in the upper half of the header word.
-        ;; FIXME: It would be nice if FIXED-ALLOC could write the header word
-        ;; in one shot, but this is an acceptable workaround.
         (inst mov
               (make-ea :dword :base object :disp (- 4 instance-pointer-lowtag))
-              (reg-in-size value :dword))
-        (generate-set-slot object value temp offset lowtag))))
+              (if (sc-is value immediate)
+                  (make-fixup (tn-value value) :layout)
+                  (reg-in-size value :dword)))
+        (generate-set-slot object value temp offset lowtag (not dx-p)))))
 
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg) :to :eval)
@@ -209,14 +210,18 @@
     ;; TODO: SET could be shorter for any known wired-tls symbol.
     (define-vop (set)
       (:args (symbol :scs (descriptor-reg))
-             (value :scs (descriptor-reg any-reg)))
+             (value :scs (descriptor-reg any-reg)
+                    :load-if (or (not (sc-is value immediate))
+                                 (not (typep (encode-value-if-immediate value)
+                                             '(or (signed-byte 32) fixup))))))
       (:temporary (:sc descriptor-reg) cell)
       (:generator 4
         ;; Compute the address into which to store. CMOV can only move into
         ;; a register, so we can't conditionally move into the TLS and
         ;; conditionally move in the opposite flag sense to the symbol.
         (compute-virtual-symbol)
-        (inst mov (access-value-slot cell) value)))
+        (inst mov (access-value-slot cell)
+              (encode-value-if-immediate value))))
 
   ;; With Symbol-Value, we check that the value isn't the trap object. So
   ;; Symbol-Value of NIL is NIL.
@@ -250,9 +255,12 @@
           (inst cmp (reg-in-size value :dword) no-tls-value-marker-widetag)
           (inst cmov :e value (access-value-slot symbol-reg)))
         (when check-boundp
-          (inst cmp (reg-in-size value :dword) unbound-marker-widetag)
-          (inst jmp :e (generate-error-code vop 'unbound-symbol-error
-                                            symbol-reg)))))
+          (assemble ()
+            (inst cmp (reg-in-size value :dword) unbound-marker-widetag)
+            (let ((*location-context* (make-restart-location RETRY value)))
+              (inst jmp :e (generate-error-code vop 'unbound-symbol-error
+                                                symbol-reg)))
+            RETRY))))
 
     (define-vop (fast-symbol-value symbol-value)
     ;; KLUDGE: not really fast, in fact, because we're going to have to
