@@ -140,22 +140,15 @@
 ;;; we could profitably keep these in registers on x86-64 now we have
 ;;; r8-r15 as well
 ;;;     Note these spaces grow from low to high addresses.
-(defvar *allocation-pointer*)
 (defvar *binding-stack-pointer*)
 
 (defconstant-eqx +static-symbols+
  `#(,@+common-static-symbols+
-    *alien-stack-pointer*
-
+    #!+immobile-space function-layout
+    #!-sb-thread *alien-stack-pointer*    ; a thread slot if #!+sb-thread
      ;; interrupt handling
-     *pseudo-atomic-bits*
-
-     *allocation-pointer*
-     *binding-stack-pointer*
-
-     ;; For GC-AND-SAVE
-     *restart-lisp-function*
-     )
+    #!-sb-thread *pseudo-atomic-bits*     ; ditto
+    #!-sb-thread *binding-stack-pointer*) ; ditto
   #'equalp)
 
 ;;; FIXME: with #!+immobile-space, this should be the empty list,
