@@ -13,17 +13,6 @@
 
 ;;;; type frobbing VOPs
 
-(define-vop (lowtag-of)
-  (:translate lowtag-of)
-  (:policy :fast-safe)
-  (:args (object :scs (any-reg descriptor-reg control-stack)
-                 :target result))
-  (:results (result :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:generator 1
-    (move result object)
-    (inst and result lowtag-mask)))
-
 (define-vop (widetag-of)
   (:translate widetag-of)
   (:policy :fast-safe)
@@ -62,6 +51,50 @@
     DONE
     (move result rax)))
 
+#!+compact-instance-header
+;; ~20 instructions vs. 35
+(define-vop (layout-of) ; no translation
+    (:policy :fast-safe)
+    (:args (object :scs (descriptor-reg))
+           (layouts :scs (constant)))
+    (:temporary (:sc unsigned-reg :offset rax-offset) rax)
+    (:results (result :scs (descriptor-reg)))
+    (:generator 6
+      ;; Lowtag: #b0011 instance
+      ;;         #b0111 list
+      ;;         #b1011 fun
+      ;;         #b1111 other
+      (inst mov  rax object)
+      (inst xor  (reg-in-size rax :byte) #b0011)
+      (inst test (reg-in-size rax :byte) #b0111)
+      (inst jmp  :ne try-other)
+      ;; It's an instance or function. Both have the layout in the header.
+      (inst and  (reg-in-size rax :byte) #b11110111)
+      (inst mov  (reg-in-size result :dword) (make-ea :dword :base rax :disp 4))
+      (inst jmp  done)
+      TRY-OTHER
+      (inst xor  (reg-in-size rax :byte) #b1100)
+      (inst test (reg-in-size rax :byte) #b1111)
+      (inst jmp  :ne imm-or-list)
+      ;; It's an other-pointer. Read the widetag.
+      (inst movzx (reg-in-size rax :dword) (make-ea :byte :base rax))
+      (inst jmp  load-from-vector)
+      IMM-OR-LIST
+      (inst cmp  object nil-value)
+      (inst jmp  :eq NULL)
+      (inst movzx (reg-in-size rax :dword) (reg-in-size object :byte))
+      LOAD-FROM-VECTOR
+      (inst mov  result layouts)
+      (inst mov  (reg-in-size result :dword)
+            (make-ea :dword :base result
+                            :index rax :scale 8
+                            :disp (+ (ash vector-data-offset word-shift)
+                                     (- other-pointer-lowtag))))
+      (inst jmp  done)
+      NULL
+      (inst mov  result (make-fixup (find-layout 'null) :layout))
+      DONE))
+
 (define-vop (%other-pointer-widetag)
   (:translate %other-pointer-widetag)
   (:policy :fast-safe)
@@ -97,9 +130,10 @@
   (:results (res :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 6
-    (loadw res x 0 fun-pointer-lowtag)
-    (inst shr res n-widetag-bits)
-    (inst and res short-header-max-words)))
+    (let ((res (reg-in-size res :dword)))
+      (inst mov res (make-ea-for-object-slot-half x 0 fun-pointer-lowtag))
+      (inst shr res n-widetag-bits)
+      (inst and res short-header-max-words))))
 
 (define-vop (set-header-data)
   (:translate set-header-data)
@@ -164,14 +198,6 @@
 
 ;;;; allocation
 
-(define-vop (dynamic-space-free-pointer)
-  (:results (int :scs (sap-reg)))
-  (:result-types system-area-pointer)
-  (:translate dynamic-space-free-pointer)
-  (:policy :fast-safe)
-  (:generator 1
-    (load-symbol-value int *allocation-pointer*)))
-
 (define-vop (binding-stack-pointer-sap)
   (:results (int :scs (sap-reg)))
   (:result-types system-area-pointer)
