@@ -411,7 +411,7 @@
                           ;; FIXME: It'd be good to have the special
                           ;; variables here be named with the *FOO*
                           ;; convention.
-                          (forms `(defparameter ,tn-name
+                          (forms `(defglobal ,tn-name
                                     (make-random-tn :kind :normal
                                                     :sc (sc-or-lose ',sc-name)
                                                     :offset
@@ -443,7 +443,7 @@
 ;; A register that's never used by the code generator, and can therefore
 ;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
 ;; be used.
-(defparameter temp-reg-tn r11-tn)
+(defglobal temp-reg-tn r11-tn)
 
 ;;; TNs for registers used to pass arguments
 (defparameter *register-arg-tns*
@@ -451,8 +451,8 @@
             (symbol-value (symbolicate register-arg-name "-TN")))
           *register-arg-names*))
 
-(defparameter thread-base-tn
-  (make-random-tn :kind :normal :sc (sc-or-lose 'unsigned-reg )
+(defglobal thread-base-tn
+  (make-random-tn :kind :normal :sc (sc-or-lose 'unsigned-reg)
                   :offset r12-offset))
 
 ;;; If value can be represented as an immediate constant, then return
@@ -462,18 +462,29 @@
     ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
          character)
      (sc-number-or-lose 'immediate))
-    (symbol
-     ;; If #!+immobile-symbols, then ALL symbols are static in placement
-     ;; and in the sub-2GB space, therefore immediate constants.
-     ;; Otherwise, if #!+immobile-space, and either compilation is to memory
-     ;; and the symbol is in immobile-space, or if in the cross-compiler which
-     ;; dumps all symbols as immobile if possible, then it's immediate.
-     (when (or #!+(or immobile-symbols (and immobile-space (host-feature sb-xc-host))) t
-               #!+(and immobile-space (not (host-feature sb-xc-host)))
-               (and (sb!c::core-object-p sb!c::*compile-object*)
-                    (typep (get-lisp-obj-address value) '(signed-byte 32)))
+    (symbol ; Symbols in static and immobile space are immediate
+     (when (or ;; With #!+immobile-symbols, all symbols are in immobile-space.
+               ;; And the cross-compiler always uses immobile-space if enabled.
+               #!+(or immobile-symbols (and immobile-space (host-feature sb-xc-host))) t
+
+               ;; Otherwise, if #!-immobile-symbols, and the symbol was present
+               ;; in the initial core image as indicated by the symbol header, then
+               ;; it's in immobile-space. There is a way in which the bit can be wrong,
+               ;; but it's highly unlikely - the symbol would have to be uninterned from
+               ;; the loading SBCL, reallocated in dynamic space and re-interned into its
+               ;; initial package. All without breaking anything. Hence, unlikely.
+               ;; Also note that if compiling to memory, the symbol's current address
+               ;; is used to determine whether it's immediate.
+               #!+(and (not (host-feature sb-xc-host)) immobile-space (not immobile-symbols))
+               (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
+                   (and (sb!c::core-object-p sb!c::*compile-object*)
+                        (immobile-space-obj-p value)))
+
                (static-symbol-p value))
        (sc-number-or-lose 'immediate)))
+    #!+immobile-space
+    (layout
+       (sc-number-or-lose 'immediate))
     (single-float
        (sc-number-or-lose
         (if (eql value 0f0) 'fp-single-zero 'fp-single-immediate)))
@@ -508,6 +519,9 @@
           (symbol   (if (static-symbol-p val)
                         (+ nil-value (static-symbol-offset val))
                         (make-fixup val :immobile-object)))
+          #!+immobile-space
+          (layout
+           (make-fixup val :layout))
           (character (if tag
                          (logior (ash (char-code val) n-widetag-bits)
                                  character-widetag)
