@@ -10,24 +10,6 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
-
-;;;; interfaces to IR2 conversion
-
-;;; Return a wired TN describing the N'th full call argument passing
-;;; location.
-(defun standard-arg-location (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-wired-tn *backend-t-primitive-type* descriptor-reg-sc-number
-                     (nth n *register-arg-offsets*))
-      (make-wired-tn *backend-t-primitive-type* control-stack-sc-number n)))
-
-(defun standard-arg-location-sc (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-sc-offset descriptor-reg-sc-number
-                      (nth n *register-arg-offsets*))
-      (make-sc-offset control-stack-sc-number n)))
 
 (defconstant arg-count-sc (make-sc-offset any-reg-sc-number ecx-offset))
 (defconstant closure-sc (make-sc-offset descriptor-reg-sc-number eax-offset))
@@ -81,44 +63,6 @@
 ;;; are using non-standard conventions.
 (defun make-arg-count-location ()
   (make-wired-tn *fixnum-primitive-type* any-reg-sc-number ecx-offset))
-
-;;; Make a TN to hold the number-stack frame pointer. This is allocated
-;;; once per component, and is component-live.
-(defun make-nfp-tn ()
-  (make-restricted-tn *fixnum-primitive-type* ignore-me-sc-number))
-
-(defun make-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-(defun make-number-stack-pointer-tn ()
-  (make-restricted-tn *fixnum-primitive-type* ignore-me-sc-number))
-
-;;; Return a list of TNs that can be used to represent an unknown-values
-;;; continuation within a function.
-(defun make-unknown-values-locations ()
-  (list (make-stack-pointer-tn)
-        (make-normal-tn *fixnum-primitive-type*)))
-
-;;; This function is called by the ENTRY-ANALYZE phase, allowing
-;;; VM-dependent initialization of the IR2-COMPONENT structure. We
-;;; push placeholder entries in the CONSTANTS to leave room for
-;;; additional noise in the code object header.
-(defun select-component-format (component)
-  (declare (type component component))
-  ;; The 1+ here is because for the x86 the first constant is a
-  ;; pointer to a list of fixups, or NIL if the code object has none.
-  ;; (If I understand correctly, the fixups are needed at GC copy
-  ;; time because the X86 code isn't relocatable.)
-  ;;
-  ;; KLUDGE: It'd be cleaner to have the fixups entry be a named
-  ;; element of the CODE (aka component) primitive object. However,
-  ;; it's currently a large, tricky, error-prone chore to change
-  ;; the layout of any primitive object, so for the foreseeable future
-  ;; we'll just live with this ugliness. -- WHN 2002-01-02
-  (dotimes (i (1+ code-constants-offset))
-    (vector-push-extend nil
-                        (ir2-component-constants (component-info component))))
-  (values))
 
 ;;;; frame hackery
 
@@ -614,13 +558,13 @@
 (defun check-ocfp-and-return-pc (old-fp return-pc)
   #+nil
   (format t "*known-return: old-fp ~S, tn-kind ~S; ~S ~S~%"
-          old-fp (sb!c::tn-kind old-fp) (sb!c::tn-save-tn old-fp)
-          (sb!c::tn-kind (sb!c::tn-save-tn old-fp)))
+          old-fp (tn-kind old-fp) (sb!c::tn-save-tn old-fp)
+          (tn-kind (sb!c::tn-save-tn old-fp)))
   #+nil
   (format t "*known-return: return-pc ~S, tn-kind ~S; ~S ~S~%"
-          return-pc (sb!c::tn-kind return-pc)
+          return-pc (tn-kind return-pc)
           (sb!c::tn-save-tn return-pc)
-          (sb!c::tn-kind (sb!c::tn-save-tn return-pc)))
+          (tn-kind (sb!c::tn-save-tn return-pc)))
   (unless (and (sc-is old-fp control-stack)
                (= (tn-offset old-fp) ocfp-save-offset))
     (error "ocfp not on stack in standard save location?"))
@@ -785,9 +729,9 @@
                (:args
                ,@(unless (eq return :tail)
                    '((new-fp :scs (any-reg) :to (:argument 1))))
-
-               (fun :scs (descriptor-reg control-stack)
-                    :target eax :to (:argument 0))
+               ,@(unless (eq named :direct)
+                   '((fun :scs (descriptor-reg control-stack)
+                          :target eax :to (:argument 0))))
 
                ,@(when (eq return :tail)
                    '((old-fp)
@@ -807,6 +751,7 @@
                (:info
                ,@(unless (or variable (eq return :tail)) '(arg-locs))
                ,@(unless variable '(nargs))
+               ,@(when (eq named :direct) '(fun))
                ,@(when (eq return :fixed) '(nvals))
                step-instrumenting)
 
@@ -820,12 +765,13 @@
                ;; with the real function and invoke the real function
                ;; for closures. Non-closures do not need this value,
                ;; so don't care what shows up in it.
-               (:temporary
-               (:sc descriptor-reg
-                    :offset eax-offset
-                    :from (:argument 0)
-                    :to :eval)
-               eax)
+               ,@(unless (eq named :direct)
+                   '((:temporary
+                      (:sc descriptor-reg
+                       :offset eax-offset
+                       :from (:argument 0)
+                       :to :eval)
+                      eax)))
 
                ;; We pass the number of arguments in ECX.
                (:temporary (:sc unsigned-reg :offset ecx-offset :to :eval) ecx)
@@ -860,9 +806,8 @@
                ;; This has to be done before the frame pointer is
                ;; changed! EAX stores the 'lexical environment' needed
                ;; for closures.
-               (move eax fun)
-
-
+               ,@(unless (eq named :direct)
+                   '((move eax fun)))
                ,@(if variable
                      ;; For variable call, compute the number of
                      ;; arguments and move some of the arguments to
@@ -945,8 +890,7 @@
                           (storew ebp-tn new-fp
                                   (frame-word-offset ocfp-save-offset))
 
-                          (move ebp-tn new-fp) ; NB - now on new stack frame.
-                          )))
+                          (move ebp-tn new-fp))))  ; NB - now on new stack frame.
 
                (when step-instrumenting
                  (emit-single-step-test)
@@ -957,11 +901,16 @@
                (note-this-location vop :call-site)
 
                (inst ,(if (eq return :tail) 'jmp 'call)
-                     ,(if named
-                          '(make-ea-for-object-slot eax fdefn-raw-addr-slot
-                                                    other-pointer-lowtag)
-                          '(make-ea-for-object-slot eax closure-fun-slot
-                                                    fun-pointer-lowtag)))
+                     ,(case named
+                        (:direct
+                         '(make-ea :dword :disp
+                           (+ nil-value (static-fun-offset fun))))
+                        ((t)
+                         '(make-ea-for-object-slot eax fdefn-raw-addr-slot
+                           other-pointer-lowtag))
+                        ((nil)
+                         '(make-ea-for-object-slot eax closure-fun-slot
+                           fun-pointer-lowtag))))
                ,@(ecase return
                    (:fixed
                     '((default-unknown-values vop values nvals node)))
@@ -973,10 +922,13 @@
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
+  (define-full-call static-call-named :direct :fixed nil)
   (define-full-call multiple-call nil :unknown nil)
   (define-full-call multiple-call-named t :unknown nil)
+  (define-full-call static-multiple-call-named :direct :unknown nil)
   (define-full-call tail-call nil :tail nil)
   (define-full-call tail-call-named t :tail nil)
+  (define-full-call static-tail-call-named :direct :tail nil)
 
   (define-full-call call-variable nil :fixed t)
   (define-full-call multiple-call-variable nil :unknown t))
@@ -1450,50 +1402,27 @@
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
-           (generate-error-code vop 'invalid-arg-count-error nargs)))
-      (cond ((not min)
-             (if (zerop max)
-                 (inst test nargs nargs)
-                 (inst cmp nargs (fixnumize max)))
-             (inst jmp :ne err-lab))
-            (max
-             (when (plusp min)
-               (inst cmp nargs (fixnumize min))
-               (inst jmp :b err-lab))
-             (inst cmp nargs (fixnumize max))
-             (inst jmp :a err-lab))
-            ((plusp min)
-             (inst cmp nargs (fixnumize min))
-             (inst jmp :b err-lab))))))
-
-;;; Various other error signallers.
-(macrolet ((def (name error translate &rest args)
-             `(define-vop (,name)
-                ,@(when translate
-                    `((:policy :fast-safe)
-                      (:translate ,translate)))
-                (:args ,@(mapcar (lambda (arg)
-                                   `(,arg :scs (any-reg descriptor-reg
-                                                control-stack constant)))
-                                 args))
-                (:vop-var vop)
-                (:save-p :compute-only)
-                (:generator 1000
-                  (error-call vop ',error ,@args)))))
-  (def arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs fname)
-  (def type-check-error object-not-type-error sb!c::%type-check-error
-    object type)
-  (def layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
-    object layout)
-  (def odd-key-args-error odd-key-args-error
-    sb!c::%odd-key-args-error)
-  (def unknown-key-arg-error unknown-key-arg-error
-    sb!c::%unknown-key-arg-error key)
-  (def nil-fun-returned-error nil-fun-returned-error nil fun))
+            (generate-error-code vop 'invalid-arg-count-error nargs)))
+      (flet ((check-min ()
+               (cond ((= min 1)
+                      (inst test nargs nargs)
+                      (inst jmp :e err-lab))
+                     ((plusp min)
+                      (inst cmp nargs (fixnumize min))
+                      (inst jmp :b err-lab)))))
+        (cond ((not min)
+               (if (zerop max)
+                   (inst test nargs nargs)
+                   (inst cmp nargs (fixnumize max)))
+               (inst jmp :ne err-lab))
+              (max
+               (check-min)
+               (inst cmp nargs (fixnumize max))
+               (inst jmp :a err-lab))
+              (t
+               (check-min)))))))
 
 ;;; Single-stepping
-
 (defun emit-single-step-test ()
   ;; We use different ways of representing whether stepping is on on
   ;; +SB-THREAD / -SB-THREAD: on +SB-THREAD, we use a slot in the
