@@ -43,10 +43,11 @@
     (reference-condition simple-error)
   ()
   (:default-initargs
-   :references (list '(:amop :generic-function allocate-instance)
-                     '(:amop :function set-funcallable-instance-function))))
+   :references '((:amop :generic-function allocate-instance)
+                 (:amop :function set-funcallable-instance-function))))
 
 (defun allocate-standard-funcallable-instance (wrapper)
+  (declare (layout wrapper))
   (let* ((slots (make-array (layout-length wrapper) :initial-element +slot-unbound+))
          (fin (cond #+(and compact-instance-header immobile-code)
                     ((not (eql (layout-bitmap wrapper) -1))
@@ -56,9 +57,10 @@
                        (setf (%funcallable-instance-layout f) wrapper)
                        (sb-vm::%set-fin-trampoline f)))
                     (t
-                     (let ((f (%make-standard-funcallable-instance
-                               slots
-                               #-compact-instance-header (sb-impl::new-instance-hash-code))))
+                     (let ((f (truly-the funcallable-instance
+                               (%make-standard-funcallable-instance
+                                slots
+                                #-compact-instance-header (sb-impl::new-instance-hash-code)))))
                        (setf (%funcallable-instance-layout f) wrapper)
                        f)))))
     #+compact-instance-header
@@ -727,7 +729,7 @@
                 ~I~_when called with arguments ~2I~_~S.~:>"
              (no-primary-method-generic-function c)
              (no-primary-method-args c))))
-  (:default-initargs :references (list '(:ansi-cl :section (7 6 6 2)))))
+  (:default-initargs :references '((:ansi-cl :section (7 6 6 2)))))
 (defmethod no-primary-method (generic-function &rest args)
   (error 'no-primary-method :generic-function generic-function :args args))
 
