@@ -157,7 +157,7 @@
                                  (fast-method-call
                                   (let* ((fmf (fast-method-call-function emf))
                                          (fun (method-function-from-fast-method-call emf))
-                                         (mf (%make-method-function fmf nil)))
+                                         (mf (%make-method-function fmf)))
                                     (set-funcallable-instance-function mf fun)
                                     (make-instance 'standard-method
                                                    :specializers nil ; XXX
@@ -218,6 +218,7 @@
 
 (defun expand-effective-method-function (gf effective-method &optional env)
   (declare (ignore env))
+  (declare (muffle-conditions code-deletion-note))
   (multiple-value-bind (nreq applyp)
       (get-generic-fun-info gf)
     (let ((ll (make-fast-method-call-lambda-list nreq applyp))
@@ -409,6 +410,12 @@
 (defun gf-requires-emf-keyword-checks (generic-function)
   (member '&key (gf-lambda-list generic-function)))
 
+(defconstant-eqx +standard-method-combination-qualifiers+
+    '(:around :before :after) #'equal)
+
+(defun standard-method-combination-qualifier-p (qualifier)
+  (member qualifier +standard-method-combination-qualifiers+))
+
 (defun standard-compute-effective-method
     (generic-function combin applicable-methods)
   (collect ((before) (primary) (after) (around))
@@ -459,6 +466,76 @@
                                (,@(rest (around))
                                   (make-method ,main-effective-method)))
                  main-effective-method))))))
+
+(defun short-method-combination-qualifiers (type-name)
+  (list type-name :around))
+
+(defun short-method-combination-qualifier-p (type-name qualifier)
+  (or (eq qualifier type-name) (eq qualifier :around)))
+
+(defun short-compute-effective-method
+    (generic-function combin applicable-methods)
+  (let ((type-name (method-combination-type-name combin))
+        (operator (short-combination-operator combin))
+        (ioa (short-combination-identity-with-one-argument combin))
+        (order (car (method-combination-options combin)))
+        (around ())
+        (primary ()))
+    (flet ((invalid (gf combin m)
+             (return-from short-compute-effective-method
+               `(%invalid-qualifiers ',gf ',combin ',m))))
+      (dolist (m applicable-methods)
+        (let ((qualifiers (method-qualifiers m)))
+          (cond ((null qualifiers) (invalid generic-function combin m))
+                ((cdr qualifiers) (invalid generic-function combin m))
+                ((eq (car qualifiers) :around)
+                 (push m around))
+                ((eq (car qualifiers) type-name)
+                 (push m primary))
+                (t (invalid generic-function combin m))))))
+    (setq around (nreverse around))
+    (ecase order
+      (:most-specific-last) ; nothing to be done, already in correct order
+      (:most-specific-first
+       (setq primary (nreverse primary))))
+    (let ((main-method
+            (if (and (null (cdr primary))
+                     (not (null ioa)))
+                `(call-method ,(car primary) ())
+                `(,operator ,@(mapcar (lambda (m) `(call-method ,m ()))
+                                      primary)))))
+      (cond ((null primary)
+             ;; As of sbcl-0.8.0.80 we don't seem to need to do
+             ;; anything messy like
+             ;;        `(APPLY (FUNCTION (IF AROUND
+             ;;                              'NO-PRIMARY-METHOD
+             ;;                              'NO-APPLICABLE-METHOD)
+             ;;                           ',GENERIC-FUNCTION
+             ;;                           .ARGS.)
+             ;; here because (for reasons I don't understand at the
+             ;; moment -- WHN) control will never reach here if there
+             ;; are no applicable methods, but instead end up
+             ;; in NO-APPLICABLE-METHODS first.
+             ;;
+             ;; FIXME: The way that we arrange for .ARGS. to be bound
+             ;; here seems weird. We rely on EXPAND-EFFECTIVE-METHOD-FUNCTION
+             ;; recognizing any form whose operator is %NO-PRIMARY-METHOD
+             ;; as magical, and carefully surrounding it with a
+             ;; LAMBDA form which binds .ARGS. But...
+             ;;   1. That seems fragile, because the magicalness of
+             ;;      %NO-PRIMARY-METHOD forms is scattered around
+             ;;      the system. So it could easily be broken by
+             ;;      locally-plausible maintenance changes like,
+             ;;      e.g., using the APPLY expression above.
+             ;;   2. That seems buggy w.r.t. to MOPpish tricks in
+             ;;      user code, e.g.
+             ;;         (DEFMETHOD COMPUTE-EFFECTIVE-METHOD :AROUND (...)
+             ;;           `(PROGN ,(CALL-NEXT-METHOD) (INCF *MY-CTR*)))
+             `(%no-primary-method ',generic-function .args.))
+            ((null around) main-method)
+            (t
+             `(call-method ,(car around)
+                           (,@(cdr around) (make-method ,main-method))))))))
 
 ;;; helper code for checking keywords in generic function calls.
 (defun compute-applicable-keywords (gf methods)
