@@ -236,7 +236,7 @@
     (reference-condition simple-error)
   ()
   (:default-initargs
-   :references (list '(:ansi-cl :macro define-method-combination))))
+   :references '((:ansi-cl :macro define-method-combination))))
 
 ;;; NOTE:
 ;;;
