@@ -39,7 +39,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
 (defvar *optimize-speed*
-  '(optimize (speed 3) (safety 0) (sb!ext:inhibit-warnings 3)))
+  '(optimize (speed 3) (safety 0) (sb!ext:inhibit-warnings 3) (debug 0)))
 ) ; EVAL-WHEN
 
 (defmacro dotimes-fixnum ((var count &optional (result nil)) &body body)
@@ -69,27 +69,6 @@
            (setf ,drops (random-fixnum)
                  ,drop-pos sb!vm:n-positive-fixnum-bits))))))
 
-;;;; PCL's view of funcallable instances
-
-(!defstruct-with-alternate-metaclass standard-funcallable-instance
-  ;; KLUDGE: Note that neither of these slots is ever accessed by its
-  ;; accessor name as of sbcl-0.pre7.63. Presumably everything works
-  ;; by puns based on absolute locations. Fun fun fun.. -- WHN 2001-10-30
-  :slot-names (clos-slots hash-code)
-  :boa-constructor %make-standard-funcallable-instance
-  :superclass-name function
-  :metaclass-name standard-classoid
-  :metaclass-constructor make-standard-classoid
-  :dd-type funcallable-structure
-  ;; Only internal implementation code will access these, and these
-  ;; accesses (slot readers in particular) could easily be a
-  ;; bottleneck, so it seems reasonable to suppress runtime type
-  ;; checks.
-  ;;
-  ;; (Except note KLUDGE above that these accessors aren't used at all
-  ;; (!) as of sbcl-0.pre7.63, so for now it's academic.)
-  :runtime-type-checks-p nil)
-
 (import 'sb!kernel:funcallable-instance-p) ; why?
 
 (defun set-funcallable-instance-function (fin new-value)
@@ -107,16 +86,11 @@
 ;;; the inline functions defined by
 ;;; !DEFSTRUCT-WITH-ALTERNATE-METACLASS are as efficient as they could
 ;;; be; ordinary defstruct accessors are defined as source transforms.
+(declaim (inline fsc-instance-p))
 (defun fsc-instance-p (fin)
   (funcallable-instance-p fin))
-(define-compiler-macro fsc-instance-p (fin)
-  `(funcallable-instance-p ,fin))
-(defmacro fsc-instance-wrapper (fin)
-  `(%funcallable-instance-layout ,fin))
 (defmacro fsc-instance-slots (fin)
-  `(%funcallable-instance-info ,fin 1))
-(defmacro fsc-instance-hash (fin)
-  `(%funcallable-instance-info ,fin 2))
+  `(%funcallable-instance-info ,fin sb!vm:instance-data-start))
 
 (declaim (inline clos-slots-ref (setf clos-slots-ref)))
 (declaim (ftype (function (simple-vector index) t) clos-slots-ref))
@@ -131,46 +105,9 @@
 ;;; and normal instances, so we can return true on structures also. A
 ;;; few uses of (OR STD-INSTANCE-P FSC-INSTANCE-P) are changed to
 ;;; PCL-INSTANCE-P.
+(declaim (inline std-instance-p))
 (defun std-instance-p (x)
   (%instancep x))
-(define-compiler-macro std-instance-p (x)
-  `(%instancep ,x))
-
-;; a temporary definition used for debugging the bootstrap
-#!+sb-show
-(defun print-std-instance (instance stream depth)
-  (declare (ignore depth))
-  (print-unreadable-object (instance stream :type t :identity t)
-    (let ((class (class-of instance)))
-      (when (or (eq class (find-class 'standard-class nil))
-                (eq class (find-class 'funcallable-standard-class nil))
-                (eq class (find-class 'system-class nil))
-                (eq class (find-class 'built-in-class nil)))
-        (princ (early-class-name instance) stream)))))
-
-;;; This is the value that we stick into a slot to tell us that it is
-;;; unbound. It may seem gross, but for performance reasons, we make
-;;; this an interned symbol. That means that the fast check to see
-;;; whether a slot is unbound is to say (EQ <val> '..SLOT-UNBOUND..).
-;;; That is considerably faster than looking at the value of a special
-;;; variable.
-;;;
-;;; It seems only reasonable to also export this for users, since
-;;; otherwise dealing with STANDARD-INSTANCE-ACCESS becomes harder
-;;; -- and slower -- than it needs to be.
-(defconstant +slot-unbound+ '..slot-unbound..
-  #!+sb-doc
-  "SBCL specific extensions to MOP: if this value is read from an
-instance using STANDARD-INSTANCE-ACCESS, the slot is unbound.
-Similarly, an :INSTANCE allocated slot can be made unbound by
-assigning this to it using (SETF STANDARD-INSTANCE-ACCESS).
-
-Value of +SLOT-UNBOUND+ is unspecified, and should not be relied to be
-of any particular type, but it is guaranteed to be suitable for EQ
-comparison.")
-
-(defmacro std-instance-class (instance)
-  `(wrapper-class* (std-instance-wrapper ,instance)))
 
 ;;; When given a funcallable instance, SET-FUN-NAME *must* side-effect
 ;;; that FIN to give it the name. When given any other kind of
@@ -183,12 +120,11 @@ comparison.")
 ;; This is an absolutely terrible name for a function which both assigns
 ;; the name slot of a function, and _sometimes_ binds a name to a function.
 (defun set-fun-name (fun new-name)
-  #!+sb-doc
   "Set the name of a compiled function object. Return the function."
   (when (valid-function-name-p fun)
     (setq fun (fdefinition fun)))
   (typecase fun
-    (%method-function (setf (%method-function-name fun) new-name))
+    (%method-function fun)
     ;; a closure potentially becomes a different closure
     (closure (setq fun (sb!impl::set-closure-name fun new-name)))
     (t (setf (%fun-name fun) new-name)))
@@ -237,36 +173,11 @@ comparison.")
 ;;; This definition is for interpreted code.
 (defun pcl-instance-p (x) (declare (explicit-check)) (%pcl-instance-p x))
 
-;;; CMU CL comment:
-;;;   We define this as STANDARD-INSTANCE, since we're going to
-;;;   clobber the layout with some standard-instance layout as soon as
-;;;   we make it, and we want the accessor to still be type-correct.
-#|
-(defstruct (standard-instance
-            (:predicate nil)
-            (:constructor %%allocate-instance--class ())
-            (:copier nil)
-            (:alternate-metaclass instance
-                                  cl:standard-class
-                                  make-standard-class))
-  (slots nil))
-|#
-(!defstruct-with-alternate-metaclass standard-instance
-  :slot-names (slots hash-code)
-  :boa-constructor %make-standard-instance
-  :superclass-name t
-  :metaclass-name standard-classoid
-  :metaclass-constructor make-standard-classoid
-  :dd-type structure
-  :runtime-type-checks-p nil)
-
 ;;; Both of these operations "work" on structures, which allows the above
 ;;; weakening of STD-INSTANCE-P.
-(defmacro std-instance-slots (x) `(%instance-ref ,x 1))
-(defmacro std-instance-wrapper (x) `(%instance-layout ,x))
-;;; KLUDGE: This one doesn't "work" on structures.  However, we
-;;; ensure, in SXHASH and friends, never to call it on structures.
-(defmacro std-instance-hash (x) `(%instance-ref ,x 2))
+;;; FIXME: what does the preceding comment mean? You can't use instance-slots
+;;; on a structure. (Consider especially a structure of 0 slots.)
+(defmacro std-instance-slots (x) `(%instance-ref ,x sb!vm:instance-data-start))
 
 ;;; FIXME: These functions are called every place we do a
 ;;; CALL-NEXT-METHOD, and probably other places too. It's likely worth
@@ -295,27 +206,6 @@ comparison.")
        (aver (layout-for-std-class-p ,wrapper))
        ,wrapper)))
 
-;;;; support for useful hashing of PCL instances
-
-(defvar *instance-hash-code-random-state* (make-random-state))
-(defun get-instance-hash-code ()
-  ;; ANSI SXHASH wants us to make a good-faith effort to produce
-  ;; hash-codes that are well distributed within the range of
-  ;; non-negative fixnums, and this RANDOM operation does that, unlike
-  ;; the sbcl<=0.8.16 implementation of this operation as
-  ;; (INCF COUNTER).
-  ;;
-  ;; Hopefully there was no virtue to the old counter implementation
-  ;; that I am insufficiently insightful to insee. -- WHN 2004-10-28
-  (random most-positive-fixnum
-          *instance-hash-code-random-state*))
-
-(defun sb!impl::sxhash-instance (x)
-  (cond
-    ((std-instance-p x) (std-instance-hash x))
-    ((fsc-instance-p x) (fsc-instance-hash x))
-    (t (bug "SXHASH-INSTANCE called on some weird thing: ~S" x))))
-
 ;;;; structure-instance stuff
 ;;;;
 ;;;; FIXME: Now that the code is SBCL-only, this extra layer of
