@@ -52,7 +52,7 @@ int arch_os_thread_init(struct thread *thread) {
      * swapped stacks, require that the control stack contain only
      * boxed data, and expands upwards while the C stack expands
      * downwards. */
-    sigstack.ss_sp=((void *) thread)+dynamic_values_bytes;
+    sigstack.ss_sp=((char *) thread)+dynamic_values_bytes;
     sigstack.ss_flags=0;
     sigstack.ss_size = 32*SIGSTKSZ;
     if(sigaltstack(&sigstack,0)<0)
@@ -67,13 +67,13 @@ int arch_os_thread_cleanup(struct thread *thread) {
 os_context_register_t   *
 os_context_register_addr(os_context_t *context, int offset)
 {
-    return &(context->uc_mcontext.regs[offset]);
+    return (os_context_register_t *)&(context->uc_mcontext.regs[offset]);
 }
 
 os_context_register_t *
 os_context_pc_addr(os_context_t *context)
 {
-    return &(context->uc_mcontext.pc);
+    return (os_context_register_t *)&(context->uc_mcontext.pc);
 }
 
 os_context_register_t *
