@@ -42,9 +42,7 @@
 #include <signal.h>
 /* #include <sys/sysinfo.h> */
 #include "validate.h"
-#if defined LISP_FEATURE_GENCGC
-#include "gencgc-internal.h"
-#endif
+#include "gc-internal.h"
 
 #if defined(LISP_FEATURE_SB_WTIMER) && !defined(LISP_FEATURE_DARWIN)
 # include <sys/event.h>
@@ -127,11 +125,26 @@ os_context_sigmask_addr(os_context_t *context)
 }
 
 os_vm_address_t
-os_validate(os_vm_address_t addr, os_vm_size_t len)
+os_validate(int movable, os_vm_address_t addr, os_vm_size_t len)
 {
     int flags = MAP_PRIVATE | MAP_ANON;
 
-    if (addr)
+    /* FIXME: use of MAP_FIXED here looks decidedly wrong! (and not what we do
+     * in linux-os.c). Granted there are differences between *BSD and Linux,
+     * but on MAP_FIXED they agree: it destroys an existing mapping.
+
+     * macOS says:
+       If the memory region specified by addr and len overlaps pages of any existing
+       mapping(s), then the overlapped part of the existing mapping(s) will be discarded.
+
+     * FreeBSD says:
+       If MAP_EXCL is not specified, a successful MAP_FIXED request replaces any
+       previous mappings for the process' pages ...
+
+     * OpenBSD says:
+       Except for MAP_FIXED mappings, the system will never replace existing mappings. */
+
+    if (!movable && addr)
         flags |= MAP_FIXED;
 
 #ifdef __NetBSD__
@@ -232,6 +245,7 @@ void
 os_install_interrupt_handlers(void)
 {
     SHOW("os_install_interrupt_handlers()/bsd-os/defined(GENCGC)");
+    if (INSTALL_SIG_MEMORY_FAULT_HANDLER) {
 #if defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER)
     undoably_install_low_level_interrupt_handler(SIG_MEMORY_FAULT,
                                                  mach_error_memory_fault_handler);
@@ -242,6 +256,7 @@ os_install_interrupt_handlers(void)
 #endif
                                                  memory_fault_handler);
 #endif
+    }
 
 #ifdef LISP_FEATURE_SB_THREAD
 # ifdef LISP_FEATURE_SB_SAFEPOINT
