@@ -65,7 +65,7 @@ tv_diff(struct timeval *x, struct timeval *y)
 #endif
 
 void *
-gc_general_alloc(word_t bytes, int page_type_flag, int quick_p) {
+gc_general_alloc(sword_t bytes, int page_type_flag, int quick_p) {
     lispobj *new=new_space_free_pointer;
     new_space_free_pointer+=(bytes/N_WORD_BYTES);
     return new;
@@ -110,10 +110,7 @@ collect_garbage(generation_index_t ignore)
      * from a signal handler (e.g. with the sigsegv gc_trigger stuff) */
     block_blockable_signals(&old);
 
-    current_static_space_free_pointer =
-        (lispobj *) ((unsigned long)
-                     SymbolValue(STATIC_SPACE_FREE_POINTER,0));
-
+    current_static_space_free_pointer = static_space_free_pointer;
 
     /* Set up from space and new space pointers. */
 
@@ -156,7 +153,8 @@ collect_garbage(generation_index_t ignore)
     scavenge_control_stack(th);
 
     scav_binding_stack((lispobj*)th->binding_stack_start,
-                       (lispobj*)get_binding_stack_pointer(th));
+                       (lispobj*)get_binding_stack_pointer(th),
+                       0);
 
 #ifdef PRINTNOISE
     printf("Scavenging static space %p - %p (%d words) ...\n",
@@ -180,11 +178,12 @@ collect_garbage(generation_index_t ignore)
     print_garbage(from_space, from_space_free_pointer);
 #endif
 
+    scan_binding_stack();
     /* Scan the weak pointers. */
 #ifdef PRINTNOISE
     printf("Scanning weak hash tables ...\n");
 #endif
-    scan_weak_hash_tables();
+    scan_weak_hash_tables(weak_ht_alivep_funs);
 
     /* Scan the weak pointers. */
 #ifdef PRINTNOISE
@@ -265,7 +264,7 @@ scavenge_newspace(void)
                 here,new_space_free_pointer); */
         next = new_space_free_pointer;
         heap_scavenge(here, next);
-        scav_weak_hash_tables();
+        scav_weak_hash_tables(weak_ht_alivep_funs, gc_scav_pair);
         here = next;
     }
     /* printf("done with newspace\n"); */
@@ -347,7 +346,7 @@ lispobj *
 search_read_only_space(void *pointer)
 {
     lispobj* start = (lispobj*)READ_ONLY_SPACE_START;
-    lispobj* end = (lispobj*)SymbolValue(READ_ONLY_SPACE_FREE_POINTER,0);
+    lispobj* end = read_only_space_free_pointer;
     if ((pointer < (void *)start) || (pointer >= (void *)end))
         return NULL;
     return gc_search_space(start, pointer);
@@ -357,7 +356,7 @@ lispobj *
 search_static_space(void *pointer)
 {
     lispobj* start = (lispobj*)STATIC_SPACE_START;
-    lispobj* end = (lispobj*)SymbolValue(STATIC_SPACE_FREE_POINTER,0);
+    lispobj* end = static_space_free_pointer;
     if ((pointer < (void *)start) || (pointer >= (void *)end))
         return NULL;
     return gc_search_space(start, pointer);
@@ -442,7 +441,7 @@ void clear_auto_gc_trigger(void)
 
 #if defined(SUNOS) || defined(SOLARIS) || defined(LISP_FEATURE_HPUX)
     /* don't want to force whole space into swapping mode... */
-    os_validate(addr, length);
+    os_validate(NOT_MOVABLE, addr, length);
 #else
     os_protect(addr, length, OS_VM_PROT_ALL);
 #endif
