@@ -1,5 +1,15 @@
 /*
  * garbage collection - shared definitions for modules "inside" the GC system
+ * FIXME: despite the preceding remark, this header is total mashup of things
+ * that are "internal to GC" vs "for all SBCL-internal C code" as opposed to
+ * gc.h which is some kind of external API though it's unclear for what,
+ * since hardly anything includes it.
+ * We need at least 1 new header for truly private bits of the GC.
+ * It's very scary that 15 other C files believe that they need this header.
+ * It would be less scary if "internal" meant "stuff intended for everything
+ * else internal to SBCL", and "gc-private.h" held support functions such as
+ * weak_pointer_breakable_p(), though it's hard to tell for sure
+ * what will work without actually doing such a refactor
  */
 
 /*
@@ -22,11 +32,7 @@
 #include "thread.h"
 #include "interr.h"
 
-#ifdef LISP_FEATURE_GENCGC
-#include "gencgc-internal.h"
-#else
-#include "cheneygc-internal.h"
-#endif
+extern const char *widetag_names[];
 
 /// Enable extra debug-only checks if DEBUG
 #ifdef DEBUG
@@ -56,6 +62,12 @@ do {                                                                   \
 #define gc_abort()                                                     \
   lose("GC invariant lost, file \"%s\", line %d\n", __FILE__, __LINE__)
 
+#ifdef LISP_FEATURE_GENCGC
+#include "gencgc-internal.h"
+#else
+#include "cheneygc-internal.h"
+#endif
+
 #define CEILING(x,y) (((x) + ((y) - 1)) & (~((y) - 1)))
 
 /* FIXME: Shouldn't this be defined in sbcl.h? */
@@ -142,7 +154,7 @@ gc_general_alloc(sword_t nbytes, int page_type_flag, int quick_p)
     return gc_alloc_with_region(nbytes, page_type_flag, my_region, quick_p);
 }
 #else
-extern void *gc_general_alloc(word_t nbytes,int page_type_flag,int quick_p);
+extern void *gc_general_alloc(sword_t nbytes,int page_type_flag,int quick_p);
 #endif
 
 #define CHECK_COPY_PRECONDITIONS(object, nwords) \
@@ -154,6 +166,8 @@ extern void *gc_general_alloc(word_t nbytes,int page_type_flag,int quick_p);
     gc_dcheck(lowtag_of(copy) == lowtag); \
     gc_dcheck(!from_space_p(copy));
 
+#define note_transported_object(old, new) /* do nothing */
+
 static inline lispobj
 gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
 {
@@ -167,11 +181,12 @@ gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
     /* Copy the object. */
     memcpy(new,native_pointer(object),nwords*N_WORD_BYTES);
 
+    note_transported_object(object, new);
+
     return make_lispobj(new, lowtag_of(object));
 }
 
 extern sword_t (*scavtab[256])(lispobj *where, lispobj object);
-extern lispobj (*transother[256])(lispobj object);
 extern sword_t (*sizetab[256])(lispobj *where);
 #define OBJECT_SIZE(header,where) \
   (is_cons_half(header)?2:sizetab[widetag_of(header)](where))
@@ -182,10 +197,17 @@ extern struct hash_table *weak_hash_tables; /* in gc-common.c */
 extern void heap_scavenge(lispobj *start, lispobj *limit);
 extern sword_t scavenge(lispobj *start, sword_t n_words);
 extern void scavenge_interrupt_contexts(struct thread *thread);
-extern void scav_weak_hash_tables(void);
-extern void scav_binding_stack(lispobj*, lispobj*);
-extern void scan_weak_hash_tables(void);
+extern void scav_weak_hash_tables(int (*[5])(lispobj,lispobj),
+                                  void (*)(lispobj*));
+extern void scav_binding_stack(lispobj*, lispobj*, void(*)(lispobj));
+extern void scan_binding_stack(void);
+extern void scan_weak_hash_tables(int (*[5])(lispobj,lispobj));
 extern void scan_weak_pointers(void);
+extern void scav_hash_table_entries (struct hash_table *hash_table,
+                                     int (*[5])(lispobj,lispobj),
+                                     void (*)(lispobj*));
+extern int (*weak_ht_alivep_funs[5])(lispobj,lispobj);
+extern void gc_scav_pair(lispobj where[2]);
 
 lispobj  copy_large_unboxed_object(lispobj object, sword_t nwords);
 lispobj  copy_unboxed_object(lispobj object, sword_t nwords);
@@ -221,14 +243,14 @@ extern void scavenge_control_stack(struct thread *th);
 extern void scrub_control_stack(void);
 extern void scrub_thread_control_stack(struct thread *);
 
-#include "fixnump.h"
-
-#ifdef LISP_FEATURE_GENCGC
-#include "gencgc-internal.h"
+#ifdef LISP_FEATURE_X86
+void gencgc_apply_code_fixups(struct code *old_code, struct code *new_code);
 #else
-#include "cheneygc-internal.h"
+#define gencgc_apply_code_fixups(ignore1,ignore2)
 #endif
 
+#include "fixnump.h"
+
 #if N_WORD_BITS == 32
 # define SIMPLE_ARRAY_WORD_WIDETAG SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG
 #elif N_WORD_BITS == 64
@@ -236,14 +258,64 @@ extern void scrub_thread_control_stack(struct thread *);
 #endif
 
 extern void
-instance_scan(void (*proc)(), lispobj *instance_ptr, sword_t n_words, lispobj bitmap);
+instance_scan(void (*proc)(lispobj*, sword_t, uword_t),
+              lispobj *instance_ptr, sword_t n_words,
+              lispobj bitmap, uword_t arg);
+
+#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
+static inline lispobj funinstance_layout(lispobj* funinstance_ptr) { // native ptr
+    return instance_layout(funinstance_ptr);
+}
+static inline lispobj function_layout(lispobj* fun_ptr) { // native ptr
+    return instance_layout(fun_ptr);
+}
+static inline void set_function_layout(lispobj* fun_ptr, lispobj layout) {
+    instance_layout(fun_ptr) = layout;
+}
+#else
+static inline lispobj funinstance_layout(lispobj* instance_ptr) { // native ptr
+    // first 4 words are: header, trampoline, fin-fun, layout
+    return instance_ptr[3];
+}
+// No layout in simple-fun or closure, because there are no free bits
+static inline lispobj function_layout(lispobj* fun_ptr) { // native ptr
+    return 0;
+}
+static inline void set_function_layout(lispobj* fun_ptr, lispobj layout) {
+    lose("Can't assign layout");
+}
+#endif
 
 #include "genesis/bignum.h"
 extern boolean positive_bignum_logbitp(int,struct bignum*);
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+#ifndef LISP_FEATURE_IMMOBILE_SPACE
 
-extern lispobj fdefn_raw_referent(struct fdefn *fdefn);
+/* The callee_lispobj of an fdefn is the value in the 'raw_addr' slot to which
+ * control transfer occurs, but cast as a simple-fun or code component.
+ * It can momentarily disagree with the 'fun' slot when assigning a new value.
+ * Pointer tracing should almost always examine both slots, as scav_fdefn() does.
+ * If the raw_addr value points to read-only space, the callee is just raw_addr
+ * itself, which either looks like a simple-fun or a fixnum depending on platform.
+ * It is not critical that this exceptional situation be consistent by having
+ * a pointer lowtag because it only affects print_otherptr() and verify_space()
+ * neither of which materially impact garbage collection. */
+
+static inline boolean points_to_readonly_space(uword_t ptr) {
+    return READ_ONLY_SPACE_START <= ptr && ptr < READ_ONLY_SPACE_END;
+}
+static inline lispobj fdefn_callee_lispobj(struct fdefn *fdefn) {
+    return (lispobj)
+        (fdefn->raw_addr -
+         (points_to_readonly_space((uword_t)fdefn->raw_addr) ? 0 : FUN_RAW_ADDR_OFFSET));
+}
+
+static inline boolean immobile_filler_p(lispobj* obj) { return 0; }
+
+#else
+
+extern void fixup_immobile_refs(lispobj (*)(lispobj), lispobj, struct code*);
+extern lispobj fdefn_callee_lispobj(struct fdefn *fdefn);
 
 static inline boolean immobile_space_p(lispobj obj)
 {
@@ -265,13 +337,11 @@ static inline low_page_index_t find_immobile_page_index(void *addr)
   return -1;
 }
 int immobile_obj_younger_p(lispobj,generation_index_t);
-void promote_immobile_obj(lispobj*,int);
+void enliven_immobile_obj(lispobj*,int);
 
 #define IMMOBILE_OBJ_VISITED_FLAG    0x10
 #define IMMOBILE_OBJ_GENERATION_MASK 0x0f // mask off the VISITED flag
 
-#define IMMOBILE_VARYOBJ_SUBSPACE_START (IMMOBILE_SPACE_START+IMMOBILE_FIXEDOBJ_SUBSPACE_SIZE)
-
 // Note: this does not work on a SIMPLE-FUN
 // because a simple-fun header does not contain a generation.
 #define __immobile_obj_generation(x) (__immobile_obj_gen_bits(x) & IMMOBILE_OBJ_GENERATION_MASK)
@@ -297,9 +367,6 @@ static inline boolean immobile_filler_p(lispobj* obj) {
   return *(int*)obj == (2<<N_WIDETAG_BITS | CODE_HEADER_WIDETAG);
 }
 
-#define set_instance_layout(instance_ptr,layout) \
-  instance_ptr[0] = (layout << 32) | (instance_ptr[0] & 0xFFFFFFFF)
-
 #endif /* immobile space */
 
 #define WEAK_POINTER_NWORDS \
