@@ -366,7 +366,7 @@ void hopscotch_create(tableptr ht, int hashfun,
 }
 
 /* Delete the storage associated with 'ht' */
-void hopscotch_delete(tableptr ht)
+void hopscotch_destroy(tableptr ht)
 {
     if (ht->mem_size) { // Free it, zero-filling if ever used.
         cached_deallocate((char*)ht->keys, ht->count ? ht->mem_size : 0);
@@ -447,7 +447,7 @@ tableptr hopscotch_resize_up(tableptr ht)
                   break;
                 }
         }
-    } while (i >= 0 && (hopscotch_delete(&copy), 1));
+    } while (i >= 0 && (hopscotch_destroy(&copy), 1));
 
     // Zero-fill and release the old storage.
     cached_deallocate((char*)ht->keys, ht->mem_size);
@@ -455,7 +455,7 @@ tableptr hopscotch_resize_up(tableptr ht)
     // Move all of the data pointers from 'copy' into ht.
     // mem_size is passed to bzero() when resetting the table,
     // so definitely be sure to use the new, not the old.
-    // And of course _don't_ hopscotch_delete() copy when done.
+    // And of course _don't_ hopscotch_destroy() copy when done.
     ht->hash      = copy.hash;
     ht->mem_size  = copy.mem_size;
     ht->mask      = copy.mask;
@@ -503,6 +503,7 @@ static inline unsigned int bitmask_of_width(int n) {
 /* Add key/val to 'ht'. 'val' is ignored for a hash-set */
 int hopscotch_insert(tableptr ht, uword_t key, sword_t val)
 {
+    gc_dcheck(key);
     // 'desired_index' is where 'key' logically belongs, but it
     // may physically go in any cell to the right up to (range-1) away.
     int desired_index = hash(ht, key) & ht->mask;
@@ -642,6 +643,7 @@ int hopscotch_containsp(tableptr ht, uword_t key)
 /* Return the value associated with 'key', or 'notfound' if not found */
 sword_t hopscotch_get(tableptr ht, uword_t key, sword_t notfound)
 {
+    gc_dcheck(key);
     int index = hash(ht, key) & ht->mask;
     unsigned bits = get_hop_mask(ht, index);
     int __attribute__((unused)) probes = 0;
@@ -672,6 +674,7 @@ found0:
  * the key was inserted, or zero if the key existed. */
 int hopscotch_put(tableptr ht, uword_t key, sword_t val)
 {
+    gc_dcheck(key);
     int index = hash(ht, key) & ht->mask;
     unsigned bits = get_hop_mask(ht, index);
     int __attribute__((unused)) probes = 0;
