@@ -31,6 +31,8 @@
 # include <dlfcn.h>
 #endif
 
+/* Expose to Lisp the value of the preprocessor define. Don't touch! */
+int install_sig_memory_fault_handler = INSTALL_SIG_MEMORY_FAULT_HANDLER;
 
 /* Except for os_zero, these routines are only called by Lisp code.
  * These routines may also be replaced by os-dependent versions
@@ -63,7 +65,7 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
          * zero-filled. */
 
         os_invalidate(block_start, block_size);
-        addr = os_validate(block_start, block_size);
+        addr = os_validate(NOT_MOVABLE, block_start, block_size);
 
         if (addr == NULL || addr != block_start)
             lose("os_zero: block moved! 0x%08x ==> 0x%08x\n",
@@ -76,7 +78,7 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
 os_vm_address_t
 os_allocate(os_vm_size_t len)
 {
-    return os_validate((os_vm_address_t)NULL, len);
+    return os_validate(MOVABLE, (os_vm_address_t)NULL, len);
 }
 
 void
@@ -135,21 +137,18 @@ void* os_dlopen(char* name, int flags) {
 }
 #endif
 
-#if defined(LISP_FEATURE_SB_DYNAMIC_CORE)
-/* When this feature is enabled, the special category of /static/ foreign
+/* When :SB-DYNAMIC-CORE is enabled, the special category of /static/ foreign
  * symbols disappears. Foreign fixups are resolved to linkage table locations
  * during genesis, and for each of them a record is added to
- * REQUIRED_RUNTIME_C_SYMBOLS list, of the form (cons name datap).
- *
- * Name is a base-string of a symbol name, and non-nil datap marks data
- * references.
+ * REQUIRED_FOREIGN_SYMBOLS vector, of the form "name" for a function reference,
+ * or ("name") for a data reference. "name" is a base-string.
  *
  * Before any code in lisp image can be called, we have to resolve all
  * references to runtime foreign symbols that used to be static, adding linkage
- * table entry for each element of REQUIRED_RUNTIME_C_SYMBOLS.
+ * table entry for each element of REQUIRED_FOREIGN_SYMBOLS.
  */
 
-#ifndef LISP_FEATURE_WIN32
+#if defined(LISP_FEATURE_SB_DYNAMIC_CORE) && !defined(LISP_FEATURE_WIN32)
 void *
 os_dlsym_default(char *name)
 {
@@ -161,7 +160,10 @@ os_dlsym_default(char *name)
 
 void os_link_runtime()
 {
-    void *link_target = (void*)(intptr_t)LINKAGE_TABLE_SPACE_START;
+    extern void write_protect_immobile_space();
+
+#ifdef LISP_FEATURE_SB_DYNAMIC_CORE
+    char *link_target = (char*)(intptr_t)LINKAGE_TABLE_SPACE_START;
     void *validated_end = link_target;
     lispobj symbol_name;
     char *namechars;
@@ -169,15 +171,14 @@ void os_link_runtime()
     void* result;
     int n = 0, m = 0, j;
 
-    struct vector* symbols = VECTOR(SymbolValue(REQUIRED_RUNTIME_C_SYMBOLS,0));
+    struct vector* symbols = VECTOR(SymbolValue(REQUIRED_FOREIGN_SYMBOLS,0));
     n = fixnum_value(symbols->length);
     for (j = 0 ; j < n ; ++j)
     {
         lispobj item = symbols->data[j];
-        symbol_name = CONS(item)->car;
-        datap = CONS(item)->cdr != NIL;
-        namechars = (void*)(intptr_t)
-            (VECTOR(symbol_name)->data);
+        datap = lowtag_of(item) == LIST_POINTER_LOWTAG;
+        symbol_name = datap ? CONS(item)->car : item;
+        namechars = (void*)(intptr_t)(VECTOR(symbol_name)->data);
         result = os_dlsym_default(namechars);
         odxprint(runtime_link, "linking %s => %p", namechars, result);
 
@@ -196,12 +197,24 @@ void os_link_runtime()
             m++;
         }
 
-        link_target = (void*)(((uintptr_t)link_target)+LINKAGE_TABLE_ENTRY_SIZE);
+        link_target += LINKAGE_TABLE_ENTRY_SIZE;
     }
     odxprint(runtime_link, "%d total symbols linked, %d undefined",
              n, m);
+#endif /* LISP_FEATURE_SB_DYNAMIC_CORE */
+
+#ifdef LISP_FEATURE_X86_64
+    extern void arch_os_link_runtime();
+    arch_os_link_runtime();
+#endif
+
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    /* Delayed until after dynamic space has been mapped, fixups made,
+     * and/or immobile-space linkage entries written,
+     * since it was too soon earlier to handle write faults. */
+    write_protect_immobile_space();
+#endif
 }
-#endif  /* sb-dynamic-core */
 
 #ifndef LISP_FEATURE_WIN32
 
