@@ -207,9 +207,6 @@ fshow_fun(void __attribute__((__unused__)) *ignored,
     va_end(args);
 }
 
-/* This file can be skipped if we're not supporting LDB. */
-#if defined(LISP_FEATURE_SB_LDB)
-
 #include "monitor.h"
 #include "vars.h"
 #include "os.h"
@@ -436,21 +433,19 @@ static void brief_struct(lispobj obj)
     extern struct vector * instance_classoid_name(lispobj*);
     struct vector * classoid_name;
     classoid_name = instance_classoid_name((lispobj*)instance);
+    lispobj layout = instance_layout((lispobj*)instance);
     if ( classoid_name ) {
         char * namestring = simple_base_stringize(classoid_name);
-        printf("#<ptr to %p %s instance>",
-               (void*)instance_layout((lispobj*)instance), namestring);
+        printf("#<ptr to %"OBJ_FMTX" %s instance>", layout, namestring);
         if ( namestring != (char*)classoid_name->data )
             free(namestring);
     } else {
-        printf("#<ptr to %p instance>",
-               (void*)instance_layout((lispobj*)instance));
+        printf("#<ptr to %"OBJ_FMTX" instance>", layout);
     }
 }
 
 #include "genesis/layout.h"
-static boolean tagged_slot_p(struct layout * layout,
-                               int slot_index)
+static boolean tagged_slot_p(struct layout *layout, int slot_index)
 {
   lispobj bitmap = layout->bitmap;
   sword_t fixnum = (sword_t)bitmap >> N_FIXNUM_TAG_BITS; // optimistically
@@ -466,12 +461,11 @@ static void print_struct(lispobj obj)
     struct instance *instance = (struct instance *)native_pointer(obj);
     unsigned int i;
     char buffer[16];
-    lispobj layout_obj =  instance_layout(native_pointer(obj));
-    print_obj("type: ", layout_obj);
-    struct layout * layout = (struct layout*)native_pointer(layout_obj);
+    lispobj layout = instance_layout(native_pointer(obj));
+    print_obj("type: ", layout);
     for (i=INSTANCE_DATA_START; i<instance_length(instance->header); i++) {
         sprintf(buffer, "slot %d: ", i);
-        if (layout != NULL && tagged_slot_p(layout, i)) {
+        if (layout && tagged_slot_p(LAYOUT(layout), i)) {
             print_obj(buffer, instance->slots[i]);
         } else {
             newline(NULL);
@@ -794,12 +788,8 @@ static void print_otherptr(lispobj obj)
         break;
 
     case FDEFN_WIDETAG:
-#ifdef LISP_FEATURE_IMMOBILE_CODE
         print_slots(fdefn_slots, 2, ptr);
-        print_obj("entry: ", fdefn_raw_referent((struct fdefn*)(ptr-1)));
-#else
-        print_slots(fdefn_slots, count & SHORT_HEADER_MAX_WORDS, ptr);
-#endif
+        print_obj("entry: ", fdefn_callee_lispobj((struct fdefn*)(ptr-1)));
         break;
 
     default:
@@ -970,13 +960,3 @@ void safely_show_lstring(struct vector * string, int quotes, FILE *s)
     fprintf(s, "#<[widetag=%02X]>", widetag_of(string->header));
   }
 }
-
-#else
-
-void
-brief_print(lispobj obj)
-{
-    printf("lispobj 0x%lx\n", (unsigned long)obj);
-}
-
-#endif /* defined(LISP_FEATURE_SB_LDB) */
