@@ -34,6 +34,7 @@
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
 #include "genesis/layout.h"
+#include "genesis/defstruct-description.h"
 #include "genesis/hash-table.h"
 #include "gencgc.h"
 
@@ -54,6 +55,8 @@ static lispobj *dynamic_space_purify_pointer;
 
 static lispobj *read_only_end, *static_end;
 
+/* These are private to purify, not to be confused with the external symbols
+ * named 'read_only_space_free_pointer', respectively 'static_space' */
 static lispobj *read_only_free, *static_free;
 
 static lispobj *pscav(lispobj *addr, long nwords, boolean constant);
@@ -167,19 +170,17 @@ ptrans_boxed(lispobj thing, lispobj header, boolean constant)
 static lispobj
 ptrans_instance(lispobj thing, lispobj header, boolean /* ignored */ constant)
 {
-    struct layout *layout =
-      (struct layout *) native_pointer(((struct instance *)native_pointer(thing))->slots[0]);
-    lispobj pure = layout->pure;
-
-    switch (pure) {
-    case T:
-        return (ptrans_boxed(thing, header, 1));
-    case NIL:
-        return (ptrans_boxed(thing, header, 0));
-    default:
-        gc_abort();
-        return NIL; /* dummy value: return something ... */
+    constant = 0;
+    lispobj info = LAYOUT(instance_layout(native_pointer(thing)))->info;
+    if (info != NIL) {
+        lispobj pure = ((struct defstruct_description*)native_pointer(info))->pure;
+        if (pure != NIL && pure != T) {
+            gc_abort();
+            return NIL; /* dummy value: return something ... */
+        }
+        constant = (pure == T);
     }
+    return ptrans_boxed(thing, header, constant);
 }
 
 static lispobj
@@ -566,19 +567,18 @@ pscav(lispobj *addr, long nwords, boolean constant)
 
               case INSTANCE_WIDETAG:
                 {
-                    struct layout *layout
-                        = (struct layout *)native_pointer(instance_layout(addr));
+                    lispobj lbitmap = LAYOUT(instance_layout(addr))->bitmap;
                     lispobj* slots = addr + 1;
                     long nslots = instance_length(*addr) | 1;
                     int index;
-                    if (fixnump(layout->bitmap)) {
-                      sword_t bitmap = (sword_t)layout->bitmap >> N_FIXNUM_TAG_BITS;
+                    if (fixnump(lbitmap)) {
+                      sword_t bitmap = (sword_t)lbitmap >> N_FIXNUM_TAG_BITS;
                       for (index = 0; index < nslots ; index++, bitmap >>= 1)
                         if (bitmap & 1)
                           pscav(slots + index, 1, constant);
                     } else {
                       struct bignum * bitmap;
-                      bitmap = (struct bignum*)native_pointer(layout->bitmap);
+                      bitmap = (struct bignum*)native_pointer(lbitmap);
                       for (index = 0; index < nslots ; index++)
                         if (positive_bignum_logbitp(index, bitmap))
                           pscav(slots + index, 1, constant);
@@ -641,10 +641,8 @@ purify(lispobj static_roots, lispobj read_only_roots)
 
     dynamic_space_purify_pointer = dynamic_space_free_pointer;
 
-    read_only_end = read_only_free =
-        (lispobj *)SymbolValue(READ_ONLY_SPACE_FREE_POINTER,0);
-    static_end = static_free =
-        (lispobj *)SymbolValue(STATIC_SPACE_FREE_POINTER,0);
+    read_only_end = read_only_free = read_only_space_free_pointer;
+    static_end = static_free = static_space_free_pointer;
 
 #ifdef PRINTNOISE
     printf(" roots");
@@ -692,8 +690,7 @@ purify(lispobj static_roots, lispobj read_only_roots)
     if (SymbolValue(SCAVENGE_READ_ONLY_SPACE) != UNBOUND_MARKER_WIDETAG
         && SymbolValue(SCAVENGE_READ_ONLY_SPACE) != NIL) {
       unsigned  read_only_space_size =
-          (lispobj *)SymbolValue(READ_ONLY_SPACE_FREE_POINTER) -
-          (lispobj *)READ_ONLY_SPACE_START;
+          read_only_space_free_pointer - (lispobj *)READ_ONLY_SPACE_START;
       fprintf(stderr,
               "scavenging read only space: %d bytes\n",
               read_only_space_size * sizeof(lispobj));
@@ -749,8 +746,8 @@ purify(lispobj static_roots, lispobj read_only_roots)
 
     /* It helps to update the heap free pointers so that free_heap can
      * verify after it's done. */
-    SetSymbolValue(READ_ONLY_SPACE_FREE_POINTER, (lispobj)read_only_free,0);
-    SetSymbolValue(STATIC_SPACE_FREE_POINTER, (lispobj)static_free,0);
+    read_only_space_free_pointer = read_only_free;
+    static_space_free_pointer = static_free;
 
     dynamic_space_free_pointer = current_dynamic_space;
     set_auto_gc_trigger(bytes_consed_between_gcs);
