@@ -523,6 +523,7 @@ attach_os_thread(init_thread_data *scribble)
     pthread_getattr_np(os, &attr);
 #endif
     pthread_attr_getstack(&attr, &stack_addr, &stack_size);
+    pthread_attr_destroy(&attr);
 #endif
 
     th->control_stack_start = stack_addr;
@@ -538,7 +539,8 @@ attach_os_thread(init_thread_data *scribble)
      * tempting to just perform such unsafe allocation though.  So let's
      * at least try to suppress GC before consing, and hope that it
      * works: */
-    bind_variable(GC_INHIBIT, T, th);
+    // Just stomp on the value already set by create_thread_struct()
+    write_TLS(GC_INHIBIT, T, th);
 
     uword_t stacksize
         = (uword_t) th->control_stack_end - (uword_t) th->control_stack_start;
@@ -747,28 +749,6 @@ create_thread_struct(lispobj initial_function) {
 #if defined(LISP_FEATURE_X86) || defined (LISP_FEATURE_X86_64)
     SetSymbolValue(ALIEN_STACK_POINTER,(lispobj)th->alien_stack_pointer,th);
 #endif
-#endif
-    bind_variable(CURRENT_CATCH_BLOCK,make_fixnum(0),th);
-    bind_variable(CURRENT_UNWIND_PROTECT_BLOCK,make_fixnum(0),th);
-    bind_variable(FREE_INTERRUPT_CONTEXT_INDEX,make_fixnum(0),th);
-    bind_variable(INTERRUPT_PENDING, NIL,th);
-    bind_variable(INTERRUPTS_ENABLED,T,th);
-    bind_variable(ALLOW_WITH_INTERRUPTS,T,th);
-    bind_variable(GC_PENDING,NIL,th);
-    bind_variable(ALLOC_SIGNAL,NIL,th);
-#ifdef PINNED_OBJECTS
-    bind_variable(PINNED_OBJECTS,NIL,th);
-#endif
-#ifdef LISP_FEATURE_SB_THREAD
-    bind_variable(STOP_FOR_GC_PENDING,NIL,th);
-#endif
-#if defined(LISP_FEATURE_SB_SAFEPOINT)
-    bind_variable(GC_SAFE,NIL,th);
-    bind_variable(IN_SAFEPOINT,NIL,th);
-#endif
-#ifdef LISP_FEATURE_SB_THRUPTION
-    bind_variable(THRUPTION_PENDING,NIL,th);
-    bind_variable(RESTART_CLUSTERS,NIL,th);
 #endif
 #ifndef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
     access_control_stack_pointer(th)=th->control_stack_start;
@@ -779,6 +759,13 @@ create_thread_struct(lispobj initial_function) {
 #ifdef GENCGC_IS_PRECISE
     th->interrupt_data->allocation_trap_context = 0;
 #endif
+
+#include "genesis/thread-init.inc"
+#ifdef LISP_FEATURE_SB_THREAD
+    /* If a symbol assigned above had a TLS index of 0, then it'll
+     * mess up th->no_tls_value_marker. Fail now if that happened. */
+    gc_assert(th->no_tls_value_marker == NO_TLS_VALUE_MARKER_WIDETAG);
+#endif
     th->no_tls_value_marker=initial_function;
 
 #if defined(LISP_FEATURE_WIN32)
@@ -867,7 +854,7 @@ os_thread_t create_thread(lispobj initial_function) {
     os_thread_t kid_tid = 0;
 
     /* Must defend against async unwinds. */
-    if (SymbolValue(INTERRUPTS_ENABLED, thread) != NIL)
+    if (read_TLS(INTERRUPTS_ENABLED, thread) != NIL)
         lose("create_thread is not safe when interrupts are enabled.\n");
 
     /* Assuming that a fresh thread struct has no lisp objects in it,
