@@ -10,6 +10,7 @@
 #include "genesis/gc-tables.h"
 #include "genesis/layout.h"
 #include "genesis/package.h"
+#include "genesis/vector.h"
 #include "pseudo-atomic.h" // for get_alloc_pointer()
 
 #include <stdlib.h>
@@ -71,7 +72,6 @@ static int gen_of(lispobj obj) {
 const char* classify_obj(lispobj ptr)
 {
     extern lispobj* instance_classoid_name(lispobj*);
-    extern char *widetag_names[];
 
     lispobj* name; // a Lisp string
     switch(lowtag_of(ptr)) {
@@ -94,10 +94,7 @@ static void add_to_layer(lispobj* obj, int wordindex,
                          struct layer* layer, int* capacity)
 {
     // Resurrect the containing object's lowtag
-    lispobj word = *obj;
-    int lowtag = is_cons_half(word) ?
-      LIST_POINTER_LOWTAG : lowtag_for_widetag[widetag_of(word)>>2];
-    lispobj ptr = make_lispobj(obj, lowtag);
+    lispobj ptr = compute_lispobj(obj);
     int staticp = ptr <= STATIC_SPACE_END;
     int gen = staticp ? -1 : gen_of(ptr);
     if (heap_trace_verbose>2)
@@ -155,7 +152,7 @@ static int find_ref(lispobj* source, lispobj target)
         // here we just go for clarity by abstracting out logbitp.
         layout = instance_layout(source);
         check_ptr(0, layout);
-        bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+        bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
         for(i=1; i<scan_limit; ++i)
             if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(i, source[i]);
         return -1;
@@ -169,12 +166,10 @@ static int find_ref(lispobj* source, lispobj target)
         })
         scan_limit = code_header_words(header);
         break;
-#ifdef LISP_FEATURE_IMMOBILE_CODE
     case FDEFN_WIDETAG:
-        check_ptr(3, fdefn_raw_referent((struct fdefn*)source));
+        check_ptr(3, fdefn_callee_lispobj((struct fdefn*)source));
         scan_limit = 3;
         break;
-#endif
     }
     for(i=1; i<scan_limit; ++i) check_ptr(i, source[i]);
     return -1;
@@ -192,7 +187,7 @@ static lispobj* find_sym_by_tls_index(unsigned int tls_index)
     lispobj* end = 0;
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     where = (lispobj*)IMMOBILE_SPACE_START;
-    end = (lispobj*)SYMBOL(IMMOBILE_FIXEDOBJ_FREE_POINTER)->value;
+    end = immobile_fixedobj_free_pointer;
 #endif
     while (1) {
         while (where < end) {
@@ -225,13 +220,9 @@ static char* deduce_thread_pc(struct thread* th, void** addr)
     char* return_pc = 0;
 
     if (th != arch_os_get_current_thread()) {
-        int i = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,th));
+        int i = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,th));
         os_context_t *c = th->interrupt_contexts[i-1];
-#ifdef LISP_FEATURE_64_BIT
-        fp = (uword_t*)*os_context_register_addr(c,reg_RBP);
-#else
-        fp = (uword_t*)*os_context_register_addr(c,reg_EBP);
-#endif
+        fp = (uword_t*)*os_context_register_addr(c,reg_FP);
     }
     while (1) {
         if ((uword_t*)addr < fp)
@@ -266,7 +257,7 @@ static struct thread* deduce_thread(void (*context_scanner)(),
             esp = (void **)((void *)&pointer);
         else {
             void **esp1;
-            free = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,th));
+            free = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,th));
             for(i=free-1;i>=0;i--) {
                 os_context_t *c=th->interrupt_contexts[i];
                 esp1 = (void **) *os_context_register_addr(c,reg_SP);
@@ -617,13 +608,11 @@ static void trace1(lispobj object,
                 widetag_of(*native_pointer(ptr)) == CLOSURE_WIDETAG)
                 target -= FUN_RAW_ADDR_OFFSET;
             break;
-#ifdef LISP_FEATURE_IMMOBILE_CODE
         case 3:
             if (lowtag_of(ptr) == OTHER_POINTER_LOWTAG &&
                 widetag_of(FDEFN(ptr)->header) == FDEFN_WIDETAG)
-                target = fdefn_raw_referent((struct fdefn*)native_pointer(ptr));
+                target = fdefn_callee_lispobj((struct fdefn*)native_pointer(ptr));
             break;
-#endif
         }
         target = canonical_obj(target);
         struct layer* next_layer = top_layer->next;
@@ -638,6 +627,7 @@ static void trace1(lispobj object,
         }
     }
     fprintf(file, "%p.\n", (void*)target);
+    fflush(file);
 }
 
 static void record_ptr(lispobj* source, lispobj target,
@@ -700,7 +690,8 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             // mixed boxed/unboxed objects
             layout = instance_layout(where);
             check_ptr(layout);
-            bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+            // Partially initialized instance can't have nonzero words yet
+            bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
             // If no raw slots, just scan without use of the bitmap.
             if (bitmap == make_fixnum(-1)) break;
             for(i=1; i<scan_limit; ++i)
@@ -717,12 +708,10 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             })
             scan_limit = code_header_words(header);
             break;
-#ifdef LISP_FEATURE_IMMOBILE_CODE
         case FDEFN_WIDETAG:
-            check_ptr(fdefn_raw_referent((struct fdefn*)where));
+            check_ptr(fdefn_callee_lispobj((struct fdefn*)where));
             scan_limit = 3;
             break;
-#endif
         default:
             if (!(other_immediate_lowtag_p(widetag) && lowtag_for_widetag[widetag>>2]))
               lose("Unknown widetag %x\n", widetag);
@@ -748,16 +737,11 @@ static uword_t build_refs(lispobj* where, lispobj* end,
 
 static void scan_spaces(struct scan_state* ss)
 {
-    build_refs((lispobj*)STATIC_SPACE_START,
-               (lispobj*)SYMBOL(STATIC_SPACE_FREE_POINTER)->value,
-               ss);
+    build_refs((lispobj*)STATIC_SPACE_START, static_space_free_pointer, ss);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    build_refs((lispobj*)IMMOBILE_SPACE_START,
-               (lispobj*)SYMBOL(IMMOBILE_FIXEDOBJ_FREE_POINTER)->value,
-               ss);
+    build_refs((lispobj*)IMMOBILE_SPACE_START, immobile_fixedobj_free_pointer, ss);
     build_refs((lispobj*)IMMOBILE_VARYOBJ_SUBSPACE_START,
-               (lispobj*)SYMBOL(IMMOBILE_SPACE_FREE_POINTER)->value,
-               ss);
+               immobile_space_free_pointer, ss);
 #endif
     walk_generation((uword_t(*)(lispobj*,lispobj*,uword_t))build_refs,
                     -1, (uword_t)ss);
@@ -854,9 +838,9 @@ static void trace_paths(void (*context_scanner)(),
         }
     } while (weak_pointers != NIL);
     os_invalidate(scratchpad.base, scratchpad.end-scratchpad.base);
-    hopscotch_delete(&inverted_heap);
-    hopscotch_delete(&visited);
-    hopscotch_delete(&targets);
+    hopscotch_destroy(&inverted_heap);
+    hopscotch_destroy(&visited);
+    hopscotch_destroy(&targets);
 }
 
 void gc_prove_liveness(void(*context_scanner)(),
@@ -892,11 +876,7 @@ void gc_prove_liveness(void(*context_scanner)(),
     if (n_pins>0 && !is_lisp_pointer(pins[0])) {
         int i;
         for(i=n_pins-1; i>=0; --i) {
-            lispobj* obj = (lispobj*)pins[i];
-            lispobj header = *obj;
-            int lowtag = is_cons_half(header) ? LIST_POINTER_LOWTAG
-                : lowtag_for_widetag[widetag_of(header)>>2];
-            pins[i] = make_lispobj(obj, lowtag);
+            pins[i] = compute_lispobj((lispobj*)pins[i]);
         }
     }
     trace_paths(context_scanner, objects, n_pins, (lispobj*)pins, criterion);
@@ -914,13 +894,12 @@ void prove_liveness(lispobj objects, int criterion)
 
 #include "genesis/package.h"
 #include "genesis/instance.h"
-#include "genesis/vector.h"
 
 static boolean __attribute__((unused)) sym_stringeq(lispobj sym, const char *string, int len)
 {
-    struct vector* name = (struct vector*)native_pointer(SYMBOL(sym)->name);
+    struct vector* name = VECTOR(SYMBOL(sym)->name);
     return widetag_of(name->header) == SIMPLE_BASE_STRING_WIDETAG
-        && fixnum_value(name->length) == len
+        && name->length == make_fixnum(len)
         && !strcmp((char*)name->data, string);
 }
 
