@@ -9,6 +9,7 @@
  * files for more information.
  */
 
+# define _GNU_SOURCE /* needed for RTLD_DEFAULT from dlfcn.h */
 #include <stdio.h>
 
 #include "sbcl.h"
@@ -27,9 +28,14 @@
 #include "thread.h"
 #include "pseudo-atomic.h"
 
+#include "genesis/code.h"
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
 
+#if defined(LISP_FEATURE_OS_PROVIDES_DLOPEN) && !defined(LISP_FEATURE_WIN32)
+# include <dlfcn.h>
+#endif
+
 #define BREAKPOINT_INST 0xcc    /* INT3 */
 #define UD2_INST 0x0b0f         /* UD2 */
 
@@ -471,23 +477,12 @@ arch_install_interrupt_handlers()
 void
 arch_write_linkage_table_jmp(char *reloc_addr, void *target_addr)
 {
-    uword_t addr = (uword_t)target_addr;
-    int i;
-
-    *reloc_addr++ = 0xFF; /* Opcode for near jump to absolute reg/mem64. */
-    *reloc_addr++ = 0x25; /* ModRM #b00 100 101, i.e. RIP-relative. */
-    *reloc_addr++ = 0x00; /* 32-bit displacement field = 0 */
-    *reloc_addr++ = 0x00; /* ... */
-    *reloc_addr++ = 0x00; /* ... */
-    *reloc_addr++ = 0x00; /* ... */
-
-    for (i = 0; i < 8; i++) {
-        *reloc_addr++ = addr & 0xff;
-        addr >>= 8;
-    }
-
+    reloc_addr[0] = 0xFF; /* Opcode for near jump to absolute reg/mem64. */
+    reloc_addr[1] = 0x25; /* ModRM #b00 100 101, i.e. RIP-relative. */
+    *(uint32_t*)(reloc_addr+2) = 0; /* 32-bit displacement field = 0 */
+    *(uword_t *)(reloc_addr+6) = (uword_t)target_addr;
     /* write a nop for good measure. */
-    *reloc_addr = 0x90;
+    reloc_addr[14] = 0x90;
 }
 
 void
@@ -552,7 +547,9 @@ arch_set_fp_modes(unsigned int mxcsr)
     asm ("ldmxcsr %0" : : "m" (temp));
 }
 
-#ifdef LISP_FEATURE_IMMOBILE_CODE
+#ifndef LISP_FEATURE_IMMOBILE_CODE
+void arch_os_link_runtime() {}
+#else
 /// Return the Lisp object that fdefn's raw_addr slot jumps to.
 /// This will either be:
 /// (1) a simple-fun,
@@ -560,17 +557,17 @@ arch_set_fp_modes(unsigned int mxcsr)
 ///     it resemble a simple-fun in terms of call convention, or
 /// (3) a code-component with no simple-fun within it, that makes
 ///     closures and other funcallable-instances look like simple-funs.
-lispobj fdefn_raw_referent(struct fdefn* fdefn) {
+lispobj fdefn_callee_lispobj(struct fdefn* fdefn) {
+    extern unsigned asm_routines_end;
     if (((lispobj)fdefn->raw_addr & 0xFE) == 0xE8) {  // looks good
         unsigned int raw_fun = (int)(long)&fdefn->raw_addr + 5 // length of "JMP rel32"
           + *(int*)((char*)&fdefn->raw_addr + 1);
         switch (((unsigned char*)&fdefn->raw_addr)[5]) {
         case 0x00: // no closure/fin trampoline
-          // If the target address is in read-only space, then it's a jump or call
-          // to an asm routine, and there is no corresponding simple-fun.
-          // While we could locate and return the code-object, it's difficult to,
-          // and there's no reason to - scavenging it is unnecessary.
-          return raw_fun >= IMMOBILE_SPACE_START ? raw_fun - FUN_RAW_ADDR_OFFSET : 0;
+          // If the target is an assembly routine, there is no simple-fun
+          // that corresponds to the entry point. The code is kept live
+          // by *ASSEMBLER-OBJECTS*. Otherwise, return the simple-fun.
+          return raw_fun < asm_routines_end ? 0 : raw_fun - FUN_RAW_ADDR_OFFSET;
         case 0x48: // embedded funcallable instance trampoline
           return (raw_fun - (4<<WORD_SHIFT)) | FUN_POINTER_LOWTAG;
         case 0x90: // general closure/fin trampoline
@@ -580,4 +577,19 @@ lispobj fdefn_raw_referent(struct fdefn* fdefn) {
         return 0;
     lose("Can't decode fdefn raw addr @ %p: %p\n", fdefn, fdefn->raw_addr);
 }
+
+void arch_os_link_runtime()
+{
+    struct code* code = (struct code*)(IMMOBILE_SPACE_START +
+                                       IMMOBILE_FIXEDOBJ_SUBSPACE_SIZE);
+    char* link_target = (char*)code->constants;
+    // I wish it were possible to assert that this is the correct code object
+    // about to be touched, but I don't see what else to assert.
+    gc_assert(code->code_size >= make_fixnum(2*LINKAGE_TABLE_ENTRY_SIZE));
+    arch_write_linkage_table_jmp(link_target,
+                                 dlsym(RTLD_DEFAULT, "alloc_tramp"));
+    link_target += LINKAGE_TABLE_ENTRY_SIZE,
+    arch_write_linkage_table_jmp(link_target,
+                                 dlsym(RTLD_DEFAULT, "alloc_to_r11"));
+}
 #endif
